// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_block_parts.cpp 656 2015-02-25 11:40:08Z tzubcic $

#include "xl_local.h"

void xl_split_blocks(void)
{
	unsigned		split_num;
	unsigned		curr_split_num, curr_block_total_ins;
	xl_block_t		*b;
	xl_bp_t			bp;

	for (unsigned i = 0; i < xl.blocks.size(); i++) {
		// current block
		b = &xl.blocks[i];

		// get random number of instructions for this block part
#if 1
		split_num = x_rand_range(3, 7);
#else
		split_num = 100000000;
#endif

		curr_split_num = 0;
		curr_block_total_ins = b->disasm.size();

		b->bc_part_id = xl.parts.size() + 1;

		while (curr_split_num < curr_block_total_ins) {
#if 1
			split_num = x_rand_range(1, 5);
#else
			split_num = 100000000;
#endif
			split_num = x_value_range(split_num, 1, curr_block_total_ins - curr_split_num);

			bp.dis = &b->disasm[curr_split_num];
			bp.num_instructions = split_num;

			// no links by default
			bp.prev_id = 0;
			bp.next_id = 0;

			// 0 is invalid id
			bp.part_id = xl.parts.size() + 1;

			if (xl.parts.size()) {
				if (xl.parts.back().dis->block == b) {
					xl.parts.back().next_id = bp.part_id;
					bp.prev_id = xl.parts.back().part_id;
				}
			}

			// each block needs to have a reference to the first part
			if (curr_split_num == 0) {
				b->first_block_part_id = bp.part_id;
			}

			xl.parts.push_back(bp);

			curr_split_num += split_num;
		}

		if (curr_split_num > curr_block_total_ins) {
			xl_code_gen_error(XL_ERR_SPLIT_BLOCKS_BAD_NUM);
		}
	}
}

void xl_shuffle_block_parts(void)
{
	// shuffle
	random_shuffle(xl.parts.begin(), xl.parts.end(), x_rand_generator);

	// add links between block parts
	xl_update_block_part_links();
}

void xl_update_block_part_links(void)
{
	xl_bp_t		*bpi, *bpj;

	// no links by default
	for (unsigned i = 0; i < xl.parts.size(); i++) {
		xl.parts[i].prev = NULL;
		xl.parts[i].next = NULL;
	}

	// add link where needed
	for (unsigned i = 0; i < xl.parts.size(); i++) {
		bpi = &xl.parts[i];

		if (bpi->next_id) {
			for (unsigned j = 0; j < xl.parts.size(); j++) {
				bpj = &xl.parts[j];

				if (bpj->part_id == bpi->next_id) {
					bpi->next = bpj;
					bpj->prev = bpi;
				}
			}
		}
	}
}

void xl_process_block_parts(void)
{
	// shuffle block parts
	xl_shuffle_block_parts();

	// initialize references
	xl.vm_code.next_ref = 0;

	// virtualize all block parts
	for (unsigned i = 0; i < xl.parts.size(); i++) {
		xl_virtualize_block_part(&xl.parts[i]);
	}

	// resolve references
	xl_vm_resolve_references();
}

#define ___VM_VI_ELSEIF(x)		else if (dis->type == UD_I ## x) xl_virtualize_ ## x(vm, dis)
#define ___VM_VI_ELSE			else

void xl_virtualize_block_part(xl_bp_t *bp)
{
	xl_disasm_t		*dis;
	xl_vm_t			*vm;

	if (!bp->num_instructions) {
		xl_code_gen_error(XL_ERR_VIRTUALIZE_BLOCK_PART_NO_INSTRUCTIONS);
	}

	vm = bp->dis->block->vm;

	if (!bp->prev) {
		// update block entry offset in bytecode
		for (unsigned i = 0; i < xl.blocks.size(); i++) {
			if (xl.blocks[i].bc_part_id == bp->part_id) {
				xl.blocks[i].bc_offset = xl.vm_code.offset;
				break;
			}
		}
	} else {
		// emit a jump to random instruction
		xl_emit_fake_jmp(vm);
	}

	// virtualize all instructions in this block part
	for (unsigned i = 0; i < bp->num_instructions; i++) {
		dis = &bp->dis[i];

		// sanity check
		xl_abort_if_vm_regs_not_free(vm);

		// emit reference to this instruction
		xl_emit_fake_ins_with_id(vm, dis);

		if (dis->uses_unsupported_registers || dis->uses_unsupported_prefixes) {
			xl_virtualize_original(vm, dis);
		} else {
#if 0
			if (0) {

			}
#else
			if (0) {

			}

			// push*
			___VM_VI_ELSEIF(push);
			//___VM_VI_ELSEIF(pushad);
			//___VM_VI_ELSEIF(pushfd);

			// pop*
			___VM_VI_ELSEIF(pop);
			//___VM_VI_ELSEIF(popad);
			//___VM_VI_ELSEIF(popfd);

			// mov*
			___VM_VI_ELSEIF(mov);
			___VM_VI_ELSEIF(movzx);
			___VM_VI_ELSEIF(movsx);
			___VM_VI_ELSEIF(xchg);

			// arithmetic
			___VM_VI_ELSEIF(add);
			___VM_VI_ELSEIF(sub);
			___VM_VI_ELSEIF(inc);
			___VM_VI_ELSEIF(dec);

			// logical
			___VM_VI_ELSEIF(or);
			___VM_VI_ELSEIF(xor);
			___VM_VI_ELSEIF(and);
			___VM_VI_ELSEIF(not);
			___VM_VI_ELSEIF(shl);
			___VM_VI_ELSEIF(shr);
			___VM_VI_ELSEIF(rol);
			___VM_VI_ELSEIF(ror);

			// comparisons
			___VM_VI_ELSEIF(cmp);
			___VM_VI_ELSEIF(test);

			// jumps
			___VM_VI_ELSEIF(jmp);
			/*___VM_VI_ELSEIF(jo);
			___VM_VI_ELSEIF(jno);
			___VM_VI_ELSEIF(jb);
			___VM_VI_ELSEIF(jae);
			___VM_VI_ELSEIF(jz);
			___VM_VI_ELSEIF(jnz);
			___VM_VI_ELSEIF(jbe);
			___VM_VI_ELSEIF(ja);
			___VM_VI_ELSEIF(js);
			___VM_VI_ELSEIF(jns);
			___VM_VI_ELSEIF(jp);
			___VM_VI_ELSEIF(jnp);
			___VM_VI_ELSEIF(jl);
			___VM_VI_ELSEIF(jge);
			___VM_VI_ELSEIF(jle);
			___VM_VI_ELSEIF(jg);
			___VM_VI_ELSEIF(jcxz);
			___VM_VI_ELSEIF(jecxz);*/

			// procedures
			___VM_VI_ELSEIF(call);
			___VM_VI_ELSEIF(ret);

			// other
			/*___VM_VI_ELSEIF(nop);
			___VM_VI_ELSEIF(lea);*/
#endif
			// unsupported instruction
			___VM_VI_ELSE {
				xl_virtualize_original(vm, dis);
			}
		}

		// sanity check
		xl_abort_if_vm_regs_not_free(dis->block->vm);

		// check if relocations have been handled
		for (unsigned j = 0; j < dis->op.size(); j++) {
			if (dis->op[j].is_reloc) {
				if (!dis->op[j].reloc_handled) {
					xl_code_gen_error(XL_ERR_VIRTUALIZE_BLOCK_PART_RELOC_NOT_HANDLED);
				}
			}
		}
	}

	// continue execution at next block part
	if (bp->next) {
		xl_virt_jmp_by_va(vm, bp->next->dis->va);
	}
	
	// last block part, continue in unprotected code
	if (!bp->next) {
		xl_virt_jmp_to_x86(vm, bp->dis->block->end_va);
	}
}

void xl_emit_fake_ins_with_id(xl_vm_t *vm, xl_disasm_t *dis)
{
	UINT32		mn;
	UINT32		reference;
	UINT32		reg1, reg2;

	// usually fake instruction
	mn = X_VMI_MOV;

	reference = dis->va;

	if (reference & (1 << 31)) {
		xl_code_gen_error(XL_ERR_VA_HIGHEST_BIT);
	}

	reference |= (1 << 31);

	reg1 = x_rand_range(0, X_VMR_TOTAL - 1);
	reg2 = reg1;

	xl_add_vm_instruction(vm, reference, mn, 0, reg1, reg2, 0);
}
