// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_blocks.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// common includes
#include "../xobf/x_common.h"

// local headers
#include "xl_disassembly.h"
#include "xl_block_parts.h"

// markers
#define XM_START			"\xEB\x0D\x78\x38\x36\x6F\x62\x66\x2D\x66\x6C\x61\x67\x73\x2D"
#define XM_END				"\xEB\x0F\x78\x38\x36\x6F\x62\x66\x2D\x65\x6E\x64\x2D\x61\x72\x65\x61"

// marker sizes
#define XM_START_SIZE		(sizeof(XM_START) - 1)
#define XM_END_SIZE			(sizeof(XM_END) - 1)

typedef struct xl_block_s {
	// block info
	UINT8					*start;
	UINT32					len;
	UINT32					rva;
	UINT32					va;
	UINT32					end_va;

	// real code info
	UINT8					*code_start;
	UINT32					code_len;
	UINT32					code_rva;
	UINT32					code_va;

	// block disassembly
	vector<xl_disasm_t>		disasm;

	// assigned virtual machine
	struct xl_vm_s			*vm;

	// bytecode reference and address
	UINT32					bc_part_id;
	UINT32					bc_offset;
	UINT32					bc_va;
	UINT32					bc_rva;

	// first block part id
	unsigned				first_block_part_id;
} xl_block_t;

// locate code blocks
void xl_locate_code_blocks(void);

// add a new block to the list
void xl_add_block(UINT8 *, UINT8 *);

// block iteration callback
typedef void (*xl_block_iterate_t)(xl_block_t *);

// iterate blocks and call a function for each one
void xl_iterate_blocks(xl_block_iterate_t);

// disassemble a block
void xl_disassemble_block(xl_block_t *);

// update disasm block pointer
void xl_update_disasm_owner_block(xl_block_t *);

// analyze block's disassembly
void xl_analyze_disassembly_in_block(xl_block_t *);

// analyze relocation info for virtualization blocks
void xl_analyze_relocation(xl_block_t *);

// verify that all instructions have been analyzed
void xl_abort_if_instructions_not_analyzed(xl_block_t *);

// print block disassembly
void xl_print_block(xl_block_t *);

// check if an address belongs to a block
bool xl_addr_in_block(xl_block_t *, UINT32);
