// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_context.cpp 660 2015-02-25 12:11:30Z tzubcic $

#include "xl_local.h"

void xl_construct_context_data(void)
{
	unsigned	len_offset;

	// clear the buffer
	xl.ctx_data.clear();

	// context protocol version
	xl_ctx_write_dword(X_CTX_DATA_PROTOCOL, false);

	// placeholder for packet length
	len_offset = xl.ctx_data.size();
	xl_ctx_write_dword(0, false);

	// default base address
	xl_ctx_write_dword(xl.module_base);

	// are relocations used
	xl_ctx_write_bool(xl.has_relocs);

	// OEP rva
	xl_ctx_write_dword(xl.oep_rva);

	// new section rva
	xl_ctx_write_dword(xl.nsc_rva);

	// new section data size
	xl_ctx_write_dword(xl.data.current_len);

	// import addresses
	xl_construct_context_imports();

	// addresses of registered data blocks
	xl_construct_context_data_addresses();

	// vm architectures
	xl_construct_context_vm_architectures();

	// block information
	xl_construct_context_block_information();

	// update length
	*(uint32_t *) &xl.ctx_data[len_offset] = xl.ctx_data.size();
}

void xl_construct_context_vm_architectures(void)
{
	xl_vm_t		*vm;

	// number of VMs
	xl_ctx_write_dword(xl.vms.size());

	for (unsigned i = 0; i < xl.vms.size(); i++) {
		vm = &xl.vms[i];

		// instruction ids
		for (unsigned j = 0; j < X_VMI_TOTAL; j++) {
			xl_ctx_write_word(vm->ins_id[j]);
		}

		// register ids
		for (unsigned j = 0; j < X_VMR_TOTAL; j++) {
			xl_ctx_write_word(vm->reg_id[j]);
		}

		// longest instruction length
		xl_ctx_write_dword(vm->longest_instruction);

		// data addresses
		xl_ctx_write_dword(xl_get_data_rva(vm->data_vmi_rvas));
		xl_ctx_write_dword(xl_get_data_rva(vm->data_vmi_lengths));
		xl_ctx_write_dword(xl_get_data_va(vm->data_tls_slot));
	}
}

void xl_construct_context_block_information(void)
{
	xl_block_t		*b;

	// number of blocks for virtualization
	xl_ctx_write_dword(xl.blocks.size());

	for (unsigned i = 0; i < xl.blocks.size(); i++) {
		b = &xl.blocks[i];

		// vm owner of this block
		xl_ctx_write_dword(b->vm->id);

		// bytecode address
		xl_ctx_write_dword(b->bc_rva);
	}
}

void xl_construct_context_imports(void)
{
	for (unsigned i = 0; i < X_IMP_TOTAL; i++) {
		xl_ctx_write_dword(xl.imports[i]);
	}
}

void xl_construct_context_data_addresses(void)
{
	
}

void xl_ctx_write_byte(UINT8 ib, bool obfuscate)
{
	unsigned		offset;
	UINT8			b1;

	offset = xl.ctx_data.size();

	if (obfuscate) {
		b1 = x_ctx_obf_8(ib, offset);
	} else {
		b1 = ib;
	}

	xl.ctx_data.push_back(b1);
}

void xl_ctx_write_word(UINT16 iw, bool obfuscate)
{
	unsigned		offset;
	union {
		UINT16		w;
		struct {
			UINT8	b1;
			UINT8	b2;
		} s;
	} u;

	offset = xl.ctx_data.size();

	if (obfuscate) {
		u.w = x_ctx_obf_16(iw, offset);
	} else {
		u.w = iw;
	}

	xl.ctx_data.push_back(u.s.b1);
	xl.ctx_data.push_back(u.s.b2);
}

void xl_ctx_write_dword(UINT32 id, bool obfuscate)
{
	unsigned		offset;
	union {
		UINT32		d;
		struct {
			UINT8	b1;
			UINT8	b2;
			UINT8	b3;
			UINT8	b4;
		} s;
	} u;

	offset = xl.ctx_data.size();

	if (obfuscate) {
		u.d = x_ctx_obf_32(id, offset);
	} else {
		u.d = id;
	}

	xl.ctx_data.push_back(u.s.b1);
	xl.ctx_data.push_back(u.s.b2);
	xl.ctx_data.push_back(u.s.b3);
	xl.ctx_data.push_back(u.s.b4);
}

void xl_ctx_write_qword(UINT64 iq, bool obfuscate)
{
	union {
		UINT64		q;
		struct {
			UINT32	d1;
			UINT32	d2;
		} s;
	} u;

	u.q = iq;

	xl_ctx_write_dword(u.s.d1, obfuscate);
	xl_ctx_write_dword(u.s.d2, obfuscate);
}

void xl_ctx_write_bool(bool state, bool obfuscate)
{
	uint8_t		val;

	if (!state) {
		val = 0;
	} else {
		val = x_rand_range(1, 0xFF);
	}

	xl_ctx_write_byte(val, obfuscate);
}
