// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_context_reply.cpp 652 2015-02-24 21:15:39Z tzubcic $

#include "xl_local.h"

void xl_initialize_context_reply(void *data, unsigned data_len)
{
	// context data
	xl.ctx_reply_data = data;

	// context data length
	xl.ctx_reply_data_len = data_len;

	// initial offset
	xl.ctx_reply_data_offset = 0;
}

void xl_parse_context_reply_data(void)
{
	UINT32			protocol;

	// make sure minimum amount of data is generated
	if (xl.ctx_reply_data_len < 4) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_REPLY_NOT_ENOUGH_DATA);
	}

	xl_ctxr_read_dword(&protocol, false);

	if (protocol == X_CTX_REPLY_DATA_PROTOCOL) {
		xl_parse_context_reply_gen_data();
	} else if (protocol == X_CTX_REPLY_ERR_PROTOCOL) {
		xl_parse_context_reply_error_message();
	} else {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_REPLY_BAD_PROTOCOL);
	}

	// make sure all data has been read
	if (xl.ctx_reply_data_offset != xl.ctx_reply_data_len) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_REPLY_TOO_MUCH_DATA);
	}
}

void xl_parse_context_reply_gen_data(void)
{
	UINT32			data_len;

	// expected data length
	xl_ctxr_read_dword(&data_len, false);

	if (data_len != xl.ctx_reply_data_len) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_GEN_DATA_BAD_DATA_LENGTH);
	}

	// parse new section data
	xl_parse_context_reply_gen_new_data();

	// parse relocations
	xl_parse_context_reply_relocations();

	// parse new entry point address
	xl_ctxr_read_dword(&xl.new_oep_rva);
	xl.new_oep_va = xl.module_base + xl.new_oep_rva;

	// parse blocks' entry points
	xl_parse_context_reply_exec_bytecode();

	// vmi handler table
	xl_parse_context_reply_vmi_handlers();
}

void xl_parse_context_reply_error_message(void)
{
	
}

void xl_parse_context_reply_gen_new_data(void)
{
	UINT32			data_len;
	UINT8			b;

	xl_ctxr_read_dword(&data_len);

	for (unsigned i = 0; i < data_len; i++) {
		xl_ctxr_read_byte(&b);
		xl.nsc_data.push_back(b);
	}
}

void xl_parse_context_reply_relocations(void)
{
	UINT32			num_relocs;
	UINT32			reloc_va;

	xl_ctxr_read_dword(&num_relocs);

	for (unsigned i = 0; i < num_relocs; i++) {
		xl_ctxr_read_dword(&reloc_va);
		xl.new_relocs.push_back(reloc_va);
	}
}

void xl_parse_context_reply_exec_bytecode(void)
{
	for (unsigned i = 0; i < xl.blocks.size(); i++) {
		xl_ctxr_read_dword(&xl.blocks[i].bc_va);
		xl.blocks[i].bc_rva = xl.blocks[i].bc_va - xl.module_base;
	}
}

void xl_parse_context_reply_vmi_handlers(void)
{
	xl_vm_t			*vm;
	UINT32			*handlers;

	for (unsigned i = 0; i < xl.vms.size(); i++) {
		vm = &xl.vms[i];

		handlers = (UINT32 *) xl_get_data(vm->data_vmi_rvas);

		for (unsigned j = 0; j < X_VMI_TOTAL; j++) {
			xl_ctxr_read_dword(&handlers[j]);
		}
	}
}

void xl_ctxr_read_byte(UINT8 *db, bool deobfuscate)
{
	uint8_t		sb;

	if ((xl.ctx_reply_data_offset + 1) > xl.ctx_reply_data_len) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_REPLY_NOT_ENOUGH_DATA);
	}

	sb = *MakePtr(uint8_t *, xl.ctx_reply_data, xl.ctx_reply_data_offset);

	if (deobfuscate) {
		sb = x_ctx_de_obf_8(sb, xl.ctx_reply_data_offset);
	}

	*db = sb;

	xl.ctx_reply_data_offset += 1;
}

void xl_ctxr_read_word(UINT16 *dw, bool deobfuscate)
{
	uint16_t		sw;

	if ((xl.ctx_reply_data_offset + 2) > xl.ctx_reply_data_len) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_REPLY_NOT_ENOUGH_DATA);
	}

	sw = *MakePtr(uint16_t *, xl.ctx_reply_data, xl.ctx_reply_data_offset);

	if (deobfuscate) {
		sw = x_ctx_de_obf_16(sw, xl.ctx_reply_data_offset);
	}

	*dw = sw;

	xl.ctx_reply_data_offset += 2;
}

void xl_ctxr_read_dword(UINT32 *dd, bool deobfuscate)
{
	uint32_t		sd;

	if ((xl.ctx_reply_data_offset + 4) > xl.ctx_reply_data_len) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_REPLY_NOT_ENOUGH_DATA);
	}

	sd = *MakePtr(uint32_t *, xl.ctx_reply_data, xl.ctx_reply_data_offset);

	if (deobfuscate) {
		sd = x_ctx_de_obf_32(sd, xl.ctx_reply_data_offset);
	}

	*dd = sd;

	xl.ctx_reply_data_offset += 4;
}

void xl_ctxr_read_qword(UINT64 *dq, bool deobfuscate)
{
	union {
		uint64_t		q;
		struct {
			uint32_t	d1;
			uint32_t	d2;
		} s;
	} u;

	xl_ctxr_read_dword(&u.s.d1, deobfuscate);
	xl_ctxr_read_dword(&u.s.d2, deobfuscate);

	*dq = u.q;
}

void xl_ctxr_read_bool(bool *dst, bool deobfuscate)
{
	UINT8		val;

	xl_ctxr_read_byte(&val, deobfuscate);

	*dst = (val != 0);
}
