// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_data.cpp 652 2015-02-24 21:15:39Z tzubcic $

#include "xl_local.h"

void xl_init_data_management(void)
{
	// clear the data block vector
	xl.data.data_blocks.clear();

	// initialize starting address
	xl.data.current_va = xl.nsc_va;

	// initial length
	xl.data.current_len = 0;

	// invalid ids
	for (unsigned i = 0; i < XL_DATA_TOTAL; i++) {
		xl.data_ids[i] = ~0;
	}
}

void xl_register_common_data(void)
{
	
}

unsigned xl_register_data(unsigned type, unsigned length)
{
	xl_data_block_t		db;
	unsigned			id;

	if (type >= XL_DT_TOTAL) {
		xl_code_gen_error(XL_ERR_REGISTER_DATA_BAD_TYPE);
	}

	id = xl.data.data_blocks.size();

	db.type = type;
	db.va = xl.data.current_va;
	db.rva = db.va - xl.module_base;
	db.length = length;
	db.offset = xl.data.current_len;

	xl.data.current_va += db.length;
	xl.data.current_len += db.length;

	xl.nsc_data.resize(xl.data.current_len);

	xl.data.data_blocks.push_back(db);

	x_random_fill(xl_get_data(id), db.length);

	return id;
}

void *xl_get_data(unsigned id)
{
	xl_data_block_t		*db;
	void				*ret;

	if (id >= xl.data.data_blocks.size()) {
		xl_code_gen_error(XL_ERR_GET_DATA_BAD_ID);
	}

	db = &xl.data.data_blocks[id];

	ret = &xl.nsc_data[db->offset];

	return ret;
}

UINT32 xl_get_data_va(unsigned id)
{
	xl_data_block_t		*db;

	if (id >= xl.data.data_blocks.size()) {
		xl_code_gen_error(XL_ERR_GET_DATA_VA_BAD_ID);
	}

	db = &xl.data.data_blocks[id];

	return db->va;
}

UINT32 xl_get_data_rva(unsigned id)
{
	xl_data_block_t		*db;

	if (id >= xl.data.data_blocks.size()) {
		xl_code_gen_error(XL_ERR_GET_DATA_RVA_BAD_ID);
	}

	db = &xl.data.data_blocks[id];

	return db->rva;
}

unsigned xl_get_data_length(unsigned id)
{
	xl_data_block_t		*db;

	if (id >= xl.data.data_blocks.size()) {
		xl_code_gen_error(XL_ERR_GET_DATA_LEN_BAD_ID);
	}

	db = &xl.data.data_blocks[id];

	return db->length;
}
