// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_disassembly.cpp 652 2015-02-24 21:15:39Z tzubcic $

#include "xl_local.h"

void xl_add_disasm(xl_block_t *b, ud_t *u)
{
	xl_disasm_t		dis;
	UINT32			block_offset;

	// instruction id
	dis.id = b->disasm.size();

	// block owner
	dis.block = b;

	// basic information
	dis.len = ud_insn_len(u);
	dis.txt = ud_insn_asm(u);
	dis.type = u->mnemonic;

	// addresses
	dis.va = (UINT32) u->insn_offset;
	dis.rva = dis.va - b->code_va;

	// instruction order
	dis.order = b->disasm.size();

	// copy instruction bytes
	block_offset = dis.va - b->va;

	memcpy(dis.ud.buffer, b->start + block_offset, dis.len);

	// stripped information
	xl_strip_ud_info(u, &dis.ud);

	// by default no relocations
	dis.reloc_count = 0;

	// by default not analyzed
	dis.is_analyzed = false;

	// by default not using prefixes
	dis.uses_unsupported_prefixes = false;

	// prefixes need to be checked here, ud info is stripped later
	if (u->pfx_adr || u->pfx_lock || /*u->pfx_opr ||*/ u->pfx_rep || u->pfx_repe || u->pfx_repne || u->pfx_rex || u->pfx_seg || u->pfx_str) {
		//printf("%s has prefixes(%u %u %u %u %u %u %u %u %u)\n", dis.txt.c_str(), (unsigned) u->pfx_adr, (unsigned) u->pfx_lock, (unsigned) u->pfx_opr, (unsigned) u->pfx_rep, (unsigned) u->pfx_repe, (unsigned) u->pfx_repne, (unsigned) u->pfx_rex, (unsigned) u->pfx_seg, (unsigned) u->pfx_str);
		dis.uses_unsupported_prefixes = true;
	}

	b->disasm.push_back(dis);
}

void xl_update_operands_owner_disasm(xl_disasm_t *d)
{
	for (unsigned i = 0; i < d->op.size(); i++) {
		d->op[i].dis = d;
	}
}

void xl_update_links_disasm(struct xl_block_s *b)
{
	xl_disasm_t		*last, *dis;

	last = NULL;

	for (unsigned i = 0; i < b->disasm.size(); i++) {
		dis = &b->disasm[i];

		dis->next = NULL;

		if (!last) {
			dis->prev = NULL;
		} else {
			dis->prev = last;
			dis->prev->next = dis;
		}

		last = dis;
	}
}

void xl_analyze_disassembly(vector<xl_disasm_t> &dis)
{
	// parse instruction operands
	for (unsigned i = 0; i < dis.size(); i++) {
		xl_add_operands(&dis[i]);
	}

	// analyze instruction and operands
	for (unsigned i = 0; i < dis.size(); i++) {
		xl_check_instruction(&dis[i]);
	}
}

void xl_check_instruction(xl_disasm_t *dis)
{
	dis->is_analyzed = true;

	if (dis->type == UD_Ijmp || dis->type == UD_Ijo || dis->type == UD_Ijno || dis->type == UD_Ijb || dis->type == UD_Ijae || dis->type == UD_Ijz || dis->type == UD_Ijnz || dis->type == UD_Ijbe || dis->type == UD_Ija || dis->type == UD_Ijs || dis->type == UD_Ijns || dis->type == UD_Ijp || dis->type == UD_Ijnp || dis->type == UD_Ijl || dis->type == UD_Ijge || dis->type == UD_Ijle || dis->type == UD_Ijg || dis->type == UD_Ijcxz || dis->type == UD_Ijecxz) {
		if (dis->op[0].data.val == (dis->block->code_va + dis->block->code_len)) {
			dis->op[0].data.val = dis->block->va + dis->block->len;
		}
	}
}
