// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_disassembly.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// common includes
#include "../xobf/x_common.h"

// local headers
#include "xl_operands.h"
#include "xl_udis86.h"

// forward declaration
struct xl_block_s;

typedef struct xl_disasm_s {
	// order id of this instruction
	unsigned				id;

	// block owner
	struct xl_block_s		*block;

	// instruction info
	UINT32					type;
	UINT32					rva;
	UINT32					va;
	UINT32					len;
	string					txt;

	// operands
	vector<xl_op_t>			op;

	// instruction reference
	UINT32					order;
	struct xl_disasm_s		*prev;
	struct xl_disasm_s		*next;

	// stripped udis86 info
	xl_ud_t					ud;

	// was the instruction analyzed
	bool					is_analyzed;

	// eflags
	bool					needs_flags;
	bool					mods_flags;

	// esp
	bool					needs_esp;
	bool					mods_esp;
	bool					uses_esp;

	// eip
	bool					mods_eip;

	// skip virtualization
	bool					uses_unsupported_registers;
	bool					uses_unsupported_prefixes;

	// relocation count
	unsigned				reloc_count;
} xl_disasm_t;

// add a new disassembly to the list
void xl_add_disasm(struct xl_block_s *, ud_t *);

// update operand disams pointer
void xl_update_operands_owner_disasm(xl_disasm_t *);

// update disasm prev/next links
void xl_update_links_disasm(struct xl_block_s *);

// analyze added disassembly
void xl_analyze_disassembly(vector<xl_disasm_t> &);

// check each instruction
void xl_check_instruction(xl_disasm_t *);
