// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_imports.cpp 652 2015-02-24 21:15:39Z tzubcic $

#include "xl_local.h"

// import definitions
xl_import_t xl_imports[] = {
	{ X_IMP_VIRTUALALLOC,		"kernel32.dll",		"VirtualAlloc"		},
	{ X_IMP_TLSALLOC,			"kernel32.dll",		"TlsAlloc"			},
	{ X_IMP_TLSGETVALUE,		"kernel32.dll",		"TlsGetValue"		},
	{ X_IMP_TLSSETVALUE,		"kernel32.dll",		"TlsSetValue"		},
	{ X_IMP_EXITPROCESS,		"kernel32.dll",		"ExitProcess"		},
};

void xl_get_import_addresses(void)
{
	// invalidate all pointers
	for (unsigned i = 0; i < X_IMP_TOTAL; i++) {
		xl.imports[i] = 0;
	}

	// make sure import table exists
	if (!xl.pe_input->has_imports()) {
		xl_error("Missing imports, make sure to include the SDK file(s) when compiling the application");
	}

	const imported_functions_list imports = get_imported_functions(*xl.pe_input);

	for (imported_functions_list::const_iterator it = imports.begin(); it != imports.end(); ++it) {
		const import_library& lib = *it;
		const import_library::imported_list& functions = lib.get_imported_functions();

		unsigned offset = 0;

		for (import_library::imported_list::const_iterator func_it = functions.begin(); func_it != functions.end(); ++func_it) {
			const imported_function& func = *func_it;

			for (unsigned i = 0; i < X_IMP_TOTAL; i++) {
				if (!_stricmp(lib.get_name().c_str(), xl_imports[i].dll)) {
					if (!_stricmp(func.get_name().c_str(), xl_imports[i].func_name)) {
						xl.imports[i] = xl_rva_to_va(lib.get_rva_to_iat() + offset);
					}
				}
			}

			offset += 4;
		}
	}

	// validate all pointers
	for (unsigned i = 0; i < X_IMP_TOTAL; i++) {
		if (!xl.imports[i]) {
			xl_error("Missing import (%s->%s), make sure to include the SDK file(s) when compiling the application", xl_imports[i].dll, xl_imports[i].func_name);
		}
	}

#if 0
	// debug print all pointers
	for (unsigned i = 0; i < X_IMP_TOTAL; i++) {
		printf("Import (%s->%s): 0x%08X\n", xl_imports[i].dll, xl_imports[i].func_name, xl.imports[i]);
	}
#endif
}
