// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_local.h 656 2015-02-25 11:40:08Z tzubcic $

#pragma once

// common includes
#include "../xobf/x_common.h"

// standard windows headers
#include <windows.h>

// shared headers
#include "x_error_codes.h"
#include "../xobf/x_vm_shared.h"

// local headers
#include "xl_interface.h"
#include "xl_error.h"
#include "xl_pe_bliss.h"
#include "xl_context.h"
#include "xl_imports.h"
#include "xl_relocations.h"
#include "xl_vm.h"
#include "xl_blocks.h"
#include "xl_operands.h"
#include "xl_disassembly.h"
#include "xl_udis86.h"
#include "xl_data.h"
#include "xl_process_context.h"
#include "xl_virtualize.h"
#include "xl_context_reply.h"
#include "xl_block_parts.h"
#include "xl_x86.h"

// XGetopt - http://www.codeproject.com/Articles/1940/XGetopt-A-Unix-compatible-getopt-for-MFC-and-Win32
#include "XGetopt/XGetopt.h"

// pe bliss library - http://code.google.com/p/portable-executable-library/
#include "../libs/pe_bliss/pe_bliss.h"

// namespaces
using namespace pe_bliss;

typedef struct {
	// input and out filenames
	string					in_path;
	string					out_path;

	// other options
	unsigned				opt_num_vms;
	bool					opt_print_blocks;

	// pebliss input object
	pe_base					*pe_input;
	pe_type					pe_type;

	// raw data
	vector<UINT8>			raw_file;
	UINT32					raw_len;

	// is relocation data present
	bool					has_relocs;

	// original entry point
	UINT32					oep_rva;
	UINT32					oep_va;

	// new entry point
	UINT32					new_oep_rva;
	UINT32					new_oep_va;

	// preferred load address
	UINT32					module_base;

	// new section info
	section					*new_section;
	UINT32					nsc_va;
	UINT32					nsc_rva;

	// code section info
	section					*code_section;
	UINT32					code_rva;
	UINT32					code_va;
	UINT32					code_size;
	UINT32					code_off;
	UINT8					*code_data;

	// new section data
	vector<UINT8>			nsc_data;

	// context data
	vector<UINT8>			ctx_data;

	// context reply data information
	void					*ctx_reply_data;
	unsigned				ctx_reply_data_len;
	unsigned				ctx_reply_data_offset;

	// imported functions' RVAs
	UINT32					imports[X_IMP_TOTAL];

	// relocations from the input file
	vector<UINT32>			input_relocs;

	// virtual machines
	vector<xl_vm_t>			vms;

	// code blocks
	vector<xl_block_t>		blocks;

	// data allocation
	xl_data_t				data;

	// generic data
	unsigned				data_ids[XL_DATA_TOTAL];

	// new relocations
	vector<UINT32>			new_relocs;

	// shuffled block parts
	vector<xl_bp_t>			parts;

	// virtualized code
	xl_vm_code_t			vm_code;

	// local directory
	char					cur_dir[MAX_PATH];
} x_local_t;

// global object
extern x_local_t xl;
