// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_operands.cpp 652 2015-02-24 21:15:39Z tzubcic $

#include "xl_local.h"

void xl_add_operands(struct xl_disasm_s *dis)
{
	dis->uses_unsupported_registers = false;

	for (unsigned i = 0; i < 3; i++) {
		struct ud_operand		*uo;
		xl_op_t					op;

		uo = &dis->ud.operand[i];

		if (uo->type != UD_NONE) {
			// operand type
			op.type = uo->type;

			// owner instruction
			op.dis = dis;

			// by default relocation does not affect this operand
			op.is_reloc = false;

			// if relocation affects this operand, it's not handled by default
			op.reloc_handled = false;

			if (op.type == UD_OP_REG) {
				op.data.reg = uo->base;

				if (!op.data.reg) {
					dis->uses_unsupported_registers = true;
				}
			} else if (op.type == UD_OP_IMM) {
				if (uo->size == 8) {
					op.data.val = uo->lval.sbyte;
				} else if (uo->size == 16) {
					op.data.val = uo->lval.sword;
				} else if (uo->size == 32) {
					op.data.val = uo->lval.sdword;
				} else {
					xl_error("Unsupported operand");
				}
			} else if (op.type == UD_OP_JIMM) {
				if (uo->size == 8) {
					op.data.val = (UINT32) (dis->va + uo->lval.sbyte) + dis->len;
				} else if (uo->size == 16) {
					op.data.val = (UINT32) (dis->va + uo->lval.sword) + dis->len;
				} else if (uo->size == 32) {
					op.data.val = (UINT32) (dis->va + uo->lval.sdword) + dis->len;
				} else {
					xl_error("Unsupported operand");
				}
			} else if (op.type == UD_OP_CONST) {
				// imm & const are identical
				op.type = UD_OP_IMM;

				op.data.val = uo->lval.sdword;
			} else if (op.type == UD_OP_MEM) {
				op.data.mem.base = uo->base;
				op.data.mem.index = uo->index;
				op.data.mem.size = uo->size;

				if (uo->scale == 0) {
					op.data.mem.scale = 0;
				} else if (uo->scale == 2) {
					op.data.mem.scale = 1;
				} else if (uo->scale == 4) {
					op.data.mem.scale = 2;
				} else if (uo->scale == 8) {
					op.data.mem.scale = 3;
				} else {
					xl_error("Unsupported operand");
				}

				if (uo->offset == 0) {
					op.data.mem.offset = 0;
				} else if (uo->offset == 8) {
					op.data.mem.offset = uo->lval.sbyte;
				} else if (uo->offset == 16) {
					op.data.mem.offset = uo->lval.sword;
				} else if (uo->offset == 32) {
					op.data.mem.offset = uo->lval.sdword;
				} else {
					xl_error("Unsupported operand");
				}
			} else {
				xl_error("Unknown instruction operand");
			}

			dis->op.push_back(op);
		} else {
			break;
		}
	}
}
