// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_process_context.cpp 652 2015-02-24 21:15:39Z tzubcic $

// local headers
#include "xl_local.h"

// xobf headers
#include "../xobf/x_local.h"

// data processing thread
DWORD __stdcall xl_process_context_thread(void *);

void xl_process_context(void)
{
	HANDLE		thread_handle;
	DWORD		exit_code;

	// create context processing thread
	thread_handle = CreateThread(NULL, 0, xl_process_context_thread, NULL, 0, NULL);

	if (!thread_handle) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_THREAD);
	}

	// wait for the thread to finish and make sure it ended properly
	if (WaitForSingleObject(thread_handle, INFINITE) != WAIT_OBJECT_0) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_WAIT);
	}

	// get processing thread's exit code
	if (!GetExitCodeThread(thread_handle, &exit_code)) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT_GET_EXIT_CODE);
	}

	// validate exit code
	if (exit_code != X_ERR_SUCCESS) {
		xl_code_gen_error(exit_code);
	}

	// construct context reply
	x_construct_context_reply(X_CTX_REPLY_DATA_PROTOCOL);

	// cleanup context
	x_cleanup_context();

	// initialize context reply data for reading
	xl_initialize_context_reply(&xg.ctx_reply_data[0], xg.ctx_reply_data.size());
}

DWORD __stdcall xl_process_context_thread(void *unused)
{
	// initialize code gen context
	x_initialize_context(&xl.ctx_data[0], xl.ctx_data.size());

	// parse generated context data
	if (!x_parse_context_data()) {
		xl_code_gen_error(XL_ERR_PARSE_CONTEXT_DATA);
	}

	// process context
	if (!x_process_context()) {
		xl_code_gen_error(XL_ERR_PROCESS_CONTEXT);
	}

	return X_ERR_SUCCESS;
}
