// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_relocations.cpp 652 2015-02-24 21:15:39Z tzubcic $

#include "xl_local.h"

void x_parse_input_relocations(void)
{
	UINT32			table_va;
	uint16_t		rlc_data, rlc_rva, rlc_type;
	UINT32			rlc_va;

	// clear the input relocations
	xl.input_relocs.clear();

	// parse the PE file for relocations
	if (xl.has_relocs) {
		const relocation_table_list tables(get_relocations(*xl.pe_input));

		for (relocation_table_list::const_iterator it = tables.begin(); it != tables.end(); ++it) {
			const relocation_table& table = *it;

			table_va = xl.module_base + table.get_rva();

			const relocation_table::relocation_list& relocs = table.get_relocations();

			for (relocation_table::relocation_list::const_iterator reloc_it = relocs.begin(); reloc_it != relocs.end(); ++reloc_it) {
				rlc_data = (*reloc_it).get_item();

				rlc_type = rlc_data >> 12;
				rlc_rva = rlc_data & 0xFFF;

				if (rlc_type != IMAGE_REL_BASED_HIGHLOW) {
					xl_error("Unsupported PE input file");
				}

				rlc_va = table_va + rlc_rva;

				xl.input_relocs.push_back(rlc_va);
			}
		}
	}
}

void xl_add_relocation_section(void)
{
	section				new_relocs;
	DWORD				rva, rva_base;
	DWORD				last_reloc_rva_base = 0xFFFFFFFF;
	relocation_table	*rt = NULL;

	if (xl.has_relocs && xl.new_relocs.size()) {
		// sort the relocations
		sort(xl.new_relocs.begin(), xl.new_relocs.end());

		relocation_table_list tables(get_relocations(*xl.pe_input));

		for (unsigned i = 0; i < xl.new_relocs.size(); i++) {
			rva = xl.new_relocs[i] - xl.module_base;

			rva_base = rva & 0xFFFFF000;

			if (rva_base != last_reloc_rva_base) {
				if (rt) {
					tables.push_back(*rt);
				}

				rt = new relocation_table;

				rt->set_rva(rva_base);

				last_reloc_rva_base = rva_base;
			}

			rt->add_relocation(relocation_entry((uint16_t) (rva - rva_base), IMAGE_REL_BASED_HIGHLOW));
		}

		if (rt) {
			tables.push_back(*rt);
		}

		new_relocs.get_raw_data().resize(1);
		new_relocs.readable(true).writeable(false).executable(false).discardable(true);
		new_relocs.set_characteristics(new_relocs.get_characteristics() | 0x40);
		new_relocs.set_name(".xrel");
		new_relocs.readable(true);

		section& attached_section = xl.pe_input->add_section(new_relocs);

		rebuild_relocations(*xl.pe_input, tables, attached_section);
	}
}
