// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_virtualize.cpp 656 2015-02-25 11:40:08Z tzubcic $

#include "xl_local.h"

// reserve registers
#define RESERVE(x)					do { \
										for (unsigned i = 0; i < x; i++) { \
											VR[i] = xl_reserve_vm_reg(vm); \
										} \
									} while(0)

// free registers
#define FREE(x)						do { \
										for (unsigned i = 0; i < x; i++) { \
											xl_free_vm_reg(vm, VR[i]); \
										} \
									} while(0)

// function header and footer
#define XL_DEF_INS_VIRT(x, y)		void xl_virtualize_ ## x(xl_vm_t *vm, xl_disasm_t *dis) \
									{ \
										unsigned	no; \
										UINT32		VR[X_VM_REG_NUM], VR_NUM = y; \
										RESERVE(y); \
										no = dis->op.size();

#define XL_END_INS_VIRT					FREE(VR_NUM); \
									}

// unsupported operand error
#define UNSUPPORTED_OPERAND			xl_code_gen_error(XL_VIRT_UNSUPPORTED_OPERAND);
#define ELSE_UNSUPPORTED_OPERAND	else { UNSUPPORTED_OPERAND }

// operands
#define OP(x)						(dis->op[x])
#define OP0							OP(0)
#define OP1							OP(1)
#define OP2							OP(2)

// operand types
#define TYPEOP(x)					OP(x).type
#define TOP0						TYPEOP(0)
#define TOP1						TYPEOP(1)
#define TOP2						TYPEOP(2)

// integer values
#define IOP(x)						(OP(x).data.val)
#define IOP0						IOP(0)
#define IOP1						IOP(1)
#define IOP2						IOP(2)

// register operands
#define ROP(x)						(OP(x).data.reg)
#define ROP0						ROP(0)
#define ROP1						ROP(1)
#define ROP2						ROP(2)

// register information
#define XRI(x)						xl_x86_reg_to_vm(x)

// native vm register
#define NR(x)						X_VMR_N_ ## x

// add an instruction
#define I(mn, ref, cc, a1, a2, a3)	xl_add_vm_instruction(vm, ref, X_VMI_ ## mn, cc, a1, a2, a3)

// enable/disable eflags saving
#define USE_EFLAGS					xl_virt_enable_vm_flags(vm, X_VMF_SAVE_EFLAGS);
#define IGN_EFLAGS					xl_virt_disable_vm_flags(vm, X_VMF_SAVE_EFLAGS);

// ----------------------------------
// block part handling
// ----------------------------------
void xl_virt_jmp_to_x86(xl_vm_t *vm, UINT32 dst_va)
{
	UINT32		VR[X_VM_REG_NUM], VR_NUM = 5;
	RESERVE(VR_NUM);

	// update ESP and ret addr
	I(MOV, 0, 0, VR[0], NR(ESP), 0);
	I(ASGN, 0, 0, VR[1], -4, 0);
	I(ADD, 0, 0, 4, VR[0], VR[1]);
	I(MOV, 0, 0, NR(ESP), VR[0], 0);

	I(ASGN, 0, 0, VR[2], dst_va, 0);
	if (xl.has_relocs) {
		I(FIXRLC, 0, 0, VR[2], 0, 0);
	}
	I(WR, 0, 0, 4, VR[0], VR[2]);

	I(LEAVE, 0, 0, 0, 0, 0);

	FREE(VR_NUM);
}

void xl_virt_jmp_to_vm_reg(xl_vm_t *vm, UINT32 dst_reg)
{
	UINT32		VR[X_VM_REG_NUM], VR_NUM = 2;
	RESERVE(VR_NUM);

	// update ESP and ret addr
	I(MOV, 0, 0, VR[0], NR(ESP), 0);
	I(ASGN, 0, 0, VR[1], -4, 0);
	I(ADD, 0, 0, 4, VR[0], VR[1]);
	I(MOV, 0, 0, NR(ESP), VR[0], 0);

	I(WR, 0, 0, 4, VR[0], dst_reg);

	I(LEAVE, 0, 0, 0, 0, 0);

	FREE(VR_NUM);
}

void xl_virt_jmp_by_va(xl_vm_t *vm, UINT32 orig_va)
{
	UINT32		ref;
	bool		cond_true;
	UINT32		VR[X_VM_REG_NUM], VR_NUM = 2;
	UINT32		rnd, rnd2;
	RESERVE(VR_NUM);

	cond_true = (x_rand_range(0, 1) == 1);

	ref = orig_va;
	ref |= (1 << 31);

	rnd = x_rand();

	xl_virt_asgn_imm(vm, VR[0], rnd, false);

	if (cond_true) {
		rnd2 = rnd;

		xl_virt_asgn_imm(vm, VR[1], rnd2, false);

		I(VMCMP, 0, 0, VR[0], VR[1], 0);
		I(JMP, 0, X_CC_Z, ref, 0, 0);
	} else {
		do {
			rnd2 = x_rand();
		} while (rnd2 == rnd);

		xl_virt_asgn_imm(vm, VR[1], rnd2, false);

		I(VMCMP, 0, 0, VR[0], VR[1], 0);
		I(JMP, 0, X_CC_NZ, ref, 0, 0);
	}

	FREE(VR_NUM);
}

void xl_virt_emit_labeled_junk(xl_vm_t *vm, UINT32 ref)
{
	UINT32		mn;
	UINT32		reg1, reg2;

	mn = X_VMI_MOV;

	reg1 = x_rand_range(0, X_VMR_TOTAL - 1);
	reg2 = reg1;

	xl_add_vm_instruction(vm, ref, mn, 0, reg1, reg2, 0);
}

// ----------------------------------
// operand handling
// ----------------------------------
void xl_virt_asgn_imm(xl_vm_t *vm, UINT32 dest_reg, UINT32 val, bool is_reloc)
{
	UINT32		VR[X_VM_REG_NUM], VR_NUM = 1;
	RESERVE(VR_NUM);

	if (!xl.has_relocs) {
		is_reloc = false;
	}

	I(ASGN, 0, 0, dest_reg, val, 0);

	if (is_reloc) {
		I(FIXRLC, 0, 0, dest_reg, 0, 0);
	}

	FREE(VR_NUM);
}

UINT32 xl_virt_load_reg(xl_vm_t *vm, UINT32 dest_reg, UINT32 x86_reg)
{
	UINT32				VR[X_VM_REG_NUM], VR_NUM = 1;
	xl_vm_x86_reg_t		*info;
	UINT32				size;

	RESERVE(VR_NUM);

	info = XRI(x86_reg);
	size = info->size;

	I(MOV, 0, 0, dest_reg, info->vmreg, 0);

	if (info->offset) {
		I(ASGN, 0, 0, VR[0], info->offset * 8 , 0);
		I(SHR, 0, 0, 4, dest_reg, VR[0]);
	}

	if (info->size != 4) {
		if (info->size == 1) {
			I(ASGN, 0, 0, VR[0], 0xFF, 0);
		} else if (info->size == 2) {
			I(ASGN, 0, 0, VR[0], 0xFFFF, 0);
		}

		I(AND, 0, 0, 4, dest_reg, VR[0]);
	}

	FREE(VR_NUM);

	return size;
}

UINT32 xl_virt_load_reg_ptr(xl_vm_t *vm, UINT32 dest_reg, UINT32 x86_reg)
{
	UINT32				VR[X_VM_REG_NUM], VR_NUM = 1;
	xl_vm_x86_reg_t		*info;
	UINT32				size;

	RESERVE(VR_NUM);

	info = XRI(x86_reg);
	size = info->size;

	// dest_reg = &reg
	I(LEA, 0, 0, dest_reg, info->vmreg, 0);

	// dest_reg = &reg[offset]
	if (info->offset) {
		I(ASGN, 0, 0, VR[0], info->offset, 0);
		I(ADD, 0, 0, 4, dest_reg, VR[0]);
	}

	FREE(VR_NUM);

	return size;
}

void xL_virt_store_reg(xl_vm_t *vm, UINT32 x86_reg, UINT32 src_reg)
{
	UINT32				VR[X_VM_REG_NUM], VR_NUM = 1;
	UINT32				size;

	RESERVE(VR_NUM);

	size = xl_virt_load_reg_ptr(vm, VR[0], x86_reg);

	I(WR, 0, 0, size, VR[0], src_reg);

	FREE(VR_NUM);
}

void xl_virt_load_imm(xl_vm_t *vm, UINT32 dest_reg, xl_op_t *op)
{
	bool		true_reloc;

	if ((op->type != UD_OP_IMM) && (op->type != UD_OP_JIMM)) {
		xl_code_gen_error(XL_ERR_LOAD_X86_IMM_BAD_TYPE);
	}
	
	true_reloc = op->is_reloc;

	if (op->type == UD_OP_JIMM) {
		true_reloc = true;
	}

	xl_virt_asgn_imm(vm, dest_reg, op->data.val, true_reloc);

	if (op->is_reloc) {
		op->reloc_handled = true;
	}
}

void xl_virt_enable_vm_flags(xl_vm_t *vm, UINT32 flags)
{
	UINT32		fake_num;

	I(SETF, 0, 0, flags, 0, 0);

	fake_num = x_rand_range(1, 2);

	for (unsigned i = 0; i < fake_num; i++) {
		xl_virt_emit_labeled_junk(vm, 0);
	}
}

void xl_virt_disable_vm_flags(xl_vm_t *vm, UINT32 flags)
{
	UINT32		fake_num;

	fake_num = x_rand_range(1, 2);

	for (unsigned i = 0; i < fake_num; i++) {
		xl_virt_emit_labeled_junk(vm, 0);
	}

	I(CLRF, 0, 0, flags, 0, 0);
}

UINT32 xl_virt_load_x86_mem(xl_vm_t *vm, UINT32 dest_reg, xl_op_t *op)
{
	UINT32				VR[X_VM_REG_NUM], VR_NUM = 1;
	UINT32				size;

	RESERVE(VR_NUM);

	if (op->type != UD_OP_MEM) {
		xl_code_gen_error(XL_ERR_LOAD_X86_MEM_NOT_MEM);
	}

	// dest_reg = 0
	I(XOR, 0, 0, 4, dest_reg, dest_reg);

	// dest_reg = (index << scale)
	if (op->data.mem.index) {
		if (XRI(op->data.mem.index)->size != 4) {
			xl_code_gen_error(XL_ERR_LOAD_X86_MEM_BAD_REG_SIZE);
		}

		I(MOV, 0, 0, dest_reg, XRI(op->data.mem.index)->vmreg, 0);

		if (op->data.mem.scale) {
			xl_virt_asgn_imm(vm, VR[0], op->data.mem.scale, false);
			I(SHL, 0, 0, 4, dest_reg, VR[0]);
		}
	}

	// dest_reg += offset
	if (op->data.mem.offset) {
		xl_virt_asgn_imm(vm, VR[0], op->data.mem.offset, op->is_reloc);
		I(ADD, 0, 0, 4, dest_reg, VR[0]);

		if (op->is_reloc) {
			op->reloc_handled = true;
		}
	}

	// dest_reg += base
	if (op->data.mem.base) {
		if (XRI(op->data.mem.base)->size != 4) {
			xl_code_gen_error(XL_ERR_LOAD_X86_MEM_BAD_REG_SIZE);
		}

		xl_virt_load_reg(vm, VR[0], op->data.mem.base);
		I(ADD, 0, 0, 4, dest_reg, VR[0]);
	}

	if (op->data.mem.size) {
		size = xl_virt_get_x86_mem_size(op);
	}

	FREE(VR_NUM);

	return size;
}

UINT32 xl_virt_get_x86_mem_size(xl_op_t *op)
{
	UINT32		size;

	if (op->type != UD_OP_MEM) {
		xl_code_gen_error(XL_ERR_GET_X86_MEM_SIZE_NOT_MEM);
	}

	if (!op->data.mem.size) {
		xl_code_gen_error(XL_ERR_GET_X86_MEM_SIZE_NO_SIZE);
	}

	size = op->data.mem.size / 8;

	return size;
}

void xl_emit_fake_jmp(xl_vm_t *vm)
{
	UINT32		ref;
	bool		cond_true;
	UINT32		VR[X_VM_REG_NUM], VR_NUM = 2;
	UINT32		rnd, rnd2;
	static bool	emiting = false;

	if (emiting) {
		return;
	}

	emiting = true;

	RESERVE(VR_NUM);

	cond_true = (x_rand_range(0, 1) == 1);

	ref = (1 << 30);

	rnd = x_rand();

	xl_virt_asgn_imm(vm, VR[0], rnd, false);

	if (cond_true) {
		rnd2 = rnd;

		xl_virt_asgn_imm(vm, VR[1], rnd2, false);

		I(VMCMP, 0, 0, VR[0], VR[1], 0);
		I(JMP, 0, X_CC_NZ, ref, 0, 0);
	} else {
		do {
			rnd2 = x_rand();
		} while (rnd2 == rnd);

		xl_virt_asgn_imm(vm, VR[1], rnd2, false);

		I(VMCMP, 0, 0, VR[0], VR[1], 0);
		I(JMP, 0, X_CC_Z, ref, 0, 0);
	}

	FREE(VR_NUM);

	emiting = false;
}

// ----------------------------------
// push*
// ----------------------------------
XL_DEF_INS_VIRT(push, 3)
	if (no == 1) {
		UINT32		size;

		if (TOP0 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[0], &OP0);
			size = 4;
		} else if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		// sub esp, size
		I(MOV, 0, 0, VR[1], NR(ESP), 0);
		xl_virt_asgn_imm(vm, VR[2], -size, false);
		I(ADD, 0, 0, 4, VR[1], VR[2]);
		I(MOV, 0, 0, NR(ESP), VR[1], 0);

		// write value to [esp]
		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(pushad, 0)
	UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(pushfd, 0)
	UNSUPPORTED_OPERAND
XL_END_INS_VIRT

// ----------------------------------
// pop*
// ----------------------------------
XL_DEF_INS_VIRT(pop, 4)
	if (no == 1) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg_ptr(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		// read value from the stack
		I(MOV, 0, 0, VR[1], NR(ESP), 0);
		I(RD, 0, 0, size, VR[1], VR[1]);

		// add esp, size
		I(MOV, 0, 0, VR[2], NR(ESP), 0);
		I(ASGN, 0, 0, VR[3], size, 0);
		I(ADD, 0, 0, 4, VR[2], VR[3]);
		I(MOV, 0, 0, NR(ESP), VR[2], 0);

		// write value to dst
		I(WR, 0, 0, size, VR[0], VR[1]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(popad, 0)
	UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(popfd, 0)
	UNSUPPORTED_OPERAND
XL_END_INS_VIRT

// ----------------------------------
// mov*
// ----------------------------------
XL_DEF_INS_VIRT(mov, 2)
	if (no == 2) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg_ptr(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
		}  ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[0], VR[1]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(movzx, 3)
	if (no == 2) {
		UINT32		val_size, dest_size;

		if (TOP0 == UD_OP_REG) {
			dest_size = xl_virt_load_reg_ptr(vm, VR[0], ROP0);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			val_size = xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_MEM) {
			val_size = xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, val_size, VR[1], VR[1]);
		} ELSE_UNSUPPORTED_OPERAND

		if (val_size == 1) {
			xl_virt_asgn_imm(vm, VR[2], 0xFF, false);
		} else if (val_size == 2) {
			xl_virt_asgn_imm(vm, VR[2], 0xFFFF, false);
		} ELSE_UNSUPPORTED_OPERAND

		I(AND, 0, 0, 4, VR[1], VR[2]);

		I(WR, 0, 0, dest_size, VR[0], VR[1]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(movsx, 5)
	if (no == 2) {
		unsigned	dst_size;
		unsigned	src_size;

		if (TOP0 == UD_OP_REG) {
			dst_size = xl_virt_load_reg_ptr(vm, VR[0], ROP0);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			src_size = xl_virt_load_reg_ptr(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_MEM) {
			src_size = xl_virt_load_x86_mem(vm, VR[1], &OP1);
		} ELSE_UNSUPPORTED_OPERAND

		I(ASGN, 0, 0, VR[2], 0, 0);
		I(RD, 0, 0, src_size, VR[2], VR[1]);

		I(ASGN, 0, 0, VR[3], 1 << ((src_size * 8) - 1), 0);
		I(VMTEST, 0, 0, VR[2], VR[3], 0);

		if (dst_size == 2) {
			if (src_size == 1) {
				I(ASGN, 0, X_CC_NZ, VR[4], 0xFFFFFF00, 0);
			} ELSE_UNSUPPORTED_OPERAND

			I(OR, 0, X_CC_NZ, dst_size, VR[2], VR[4]);
		} else if (dst_size == 4) {
			if (src_size == 1) {
				I(ASGN, 0, X_CC_NZ, VR[4], 0xFFFFFF00, 0);
			} else if (src_size == 2) {
				I(ASGN, 0, X_CC_NZ, VR[4], 0xFFFF0000, 0);
			} ELSE_UNSUPPORTED_OPERAND

			I(OR, 0, X_CC_NZ, dst_size, VR[2], VR[4]);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, dst_size, VR[0], VR[2]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(xchg, 4)
	if (no == 2) {
		unsigned	size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg_ptr(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(RD, 0, 0, size, VR[1], VR[0]);

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[2], ROP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[2], &OP1);
		} ELSE_UNSUPPORTED_OPERAND

		I(RD, 0, 0, size, VR[3], VR[2]);

		I(WR, 0, 0, size, VR[0], VR[3]);
		I(WR, 0, 0, size, VR[2], VR[1]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

// ----------------------------------
// arithmetic
// ----------------------------------
XL_DEF_INS_VIRT(add, 2)
	if (no == 2) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(ADD, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(sub, 2)
	if (no == 2) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(SUB, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(inc, 5)
	if (no == 1) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		I(ASGN, 0, 0, VR[1], 1, 0);

		// save CF bit
		I(ASGN, 0, 0, VR[2], NR(EFLAGS), 0);
		I(ASGN, 0, 0, VR[3], X_X86F_CARRY, 0);
		I(AND, 0, 0, 4, VR[2], VR[3]);

		USE_EFLAGS
		I(ADD, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		// restore CF bit state
		I(ASGN, 0, 0, VR[4], NR(EFLAGS), 0);
		I(NOT, 0, 0, 4, VR[3], 0);
		I(AND, 0, 0, 4, VR[4], VR[3]);
		I(OR, 0, 0, 4, VR[4], VR[2]);

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(dec, 5)
	if (no == 1) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		I(ASGN, 0, 0, VR[1], 1, 0);

		// save CF bit
		I(ASGN, 0, 0, VR[2], NR(EFLAGS), 0);
		I(ASGN, 0, 0, VR[3], X_X86F_CARRY, 0);
		I(AND, 0, 0, 4, VR[2], VR[3]);

		USE_EFLAGS
		I(SUB, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		// restore CF bit state
		I(ASGN, 0, 0, VR[4], NR(EFLAGS), 0);
		I(NOT, 0, 0, 4, VR[3], 0);
		I(AND, 0, 0, 4, VR[4], VR[3]);
		I(OR, 0, 0, 4, VR[4], VR[2]);

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

// ----------------------------------
// logical
// ----------------------------------
XL_DEF_INS_VIRT(or, 2)
	if (no == 2) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(OR, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(xor, 2)
	if (no == 2) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(XOR, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(and, 2)
	if (no == 2) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(AND, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(not, 2)
	if (no == 1) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg_ptr(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(RD, 0, 0, size, VR[1], VR[0]);

		USE_EFLAGS
		I(NOT, 0, 0, size, VR[1], 0);
		IGN_EFLAGS

		I(WR, 0, 0, size, VR[0], VR[1]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(shl, 2)
	if (no == 2) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(SHL, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(shr, 2)
	if (no == 2) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(SHR, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(rol, 2)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			xl_virtualize_original(vm, dis);
		} else {
			UINT32		size;

			if (TOP0 == UD_OP_REG) {
				size = xl_virt_load_reg(vm, VR[0], ROP0);
			} else if (TOP0 == UD_OP_MEM) {
				size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
				I(RD, 0, 0, size, VR[0], VR[0]);
			} ELSE_UNSUPPORTED_OPERAND

			if (TOP1 == UD_OP_REG) {
				xl_virt_load_reg(vm, VR[1], ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				xl_virt_load_imm(vm, VR[1], &OP1);
			} ELSE_UNSUPPORTED_OPERAND

			USE_EFLAGS
			I(ROL, 0, 0, size, VR[0], VR[1]);
			IGN_EFLAGS

			if (TOP0 == UD_OP_REG) {
				xl_virt_load_reg_ptr(vm, VR[1], ROP0);
			} else if (TOP0 == UD_OP_MEM) {
				xl_virt_load_x86_mem(vm, VR[1], &OP0);
			} ELSE_UNSUPPORTED_OPERAND

			I(WR, 0, 0, size, VR[1], VR[0]);
		}
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(ror, 2)
	if (no == 2) {
		UINT32		size;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(ROR, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS

		if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg_ptr(vm, VR[1], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP0);
		} ELSE_UNSUPPORTED_OPERAND

		I(WR, 0, 0, size, VR[1], VR[0]);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

// ----------------------------------
// comparisons
// ----------------------------------
XL_DEF_INS_VIRT(cmp, 2)
	if (no == 2) {
		UINT32		size = 0;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(SUB, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(test, 2)
	if (no == 2) {
		UINT32		size = 0;

		if (TOP0 == UD_OP_REG) {
			size = xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			size = xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, size, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		if (TOP1 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[1], ROP1);
		} else if (TOP1 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[1], &OP1);
			I(RD, 0, 0, size, VR[1], VR[1]);
		} else if (TOP1 == UD_OP_IMM) {
			xl_virt_load_imm(vm, VR[1], &OP1);
		} ELSE_UNSUPPORTED_OPERAND

		USE_EFLAGS
		I(AND, 0, 0, size, VR[0], VR[1]);
		IGN_EFLAGS
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

// ----------------------------------
// jumps
// ----------------------------------
XL_DEF_INS_VIRT(jmp, 1)
	if (no == 1) {
		if (TOP0 == UD_OP_JIMM) {
			if (xl_addr_in_block(dis->block, IOP0)) {
				UINT32 ref = IOP0 | (1 << 31);
				I(JMP, 0, 0, ref, 0, 0);
			} else {
				xl_virt_jmp_to_x86(vm, IOP0);
			}
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, 4, VR[0], VR[0]);
			xl_virt_jmp_to_vm_reg(vm, VR[0]);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

// ----------------------------------
// procedures
// ----------------------------------
XL_DEF_INS_VIRT(call, 3)
	if (no == 1) {
		if (TOP0 == UD_OP_JIMM) {
			xl_virt_load_imm(vm, VR[0], &OP0);
		} else if (TOP0 == UD_OP_REG) {
			xl_virt_load_reg(vm, VR[0], ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			xl_virt_load_x86_mem(vm, VR[0], &OP0);
			I(RD, 0, 0, 4, VR[0], VR[0]);
		} ELSE_UNSUPPORTED_OPERAND

		// update ESP and ret addr
		I(MOV, 0, 0, VR[1], NR(ESP), 0);
		I(ASGN, 0, 0, VR[2], -8, 0);
		I(ADD, 0, 0, 4, VR[1], VR[2]);
		I(MOV, 0, 0, NR(ESP), VR[1], 0);
		I(WR, 0, 0, 4, VR[1], VR[0]);

		// issue call
		I(CALL, 0, 0, 0, 0, 0);
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

XL_DEF_INS_VIRT(ret, 2)
	if (no == 0) {
		I(LEAVE, 0, 0, 0, 0, 0);
	} else if (no == 1) {
		if (TOP0 == UD_OP_IMM) {
			// add esp, imm
			I(MOV, 0, 0, VR[0], NR(ESP), 0);
			I(ASGN, 0, 0, VR[1], IOP0, 0);
			I(ADD, 0, 0, 4, VR[0], VR[1]);
			I(MOV, 0, 0, NR(ESP), VR[0], 0);

			I(LEAVE, 0, 0, 0, 0, 0);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
XL_END_INS_VIRT

// ----------------------------------
// other
// ----------------------------------


// ----------------------------------
// copy instruction in its original form
// ----------------------------------
XL_DEF_INS_VIRT(original, 11)
	UINT32		buf_offset;
	UINT8		buf[128];
	UINT32		len, copied_len;
	xl_op_t		*op;

	// get random offset
	buf_offset = x_rand_range(0, X_VM_ORIG_EXEC_SIZE - 32);

	// get lengths
	len = dis->len;

	// fill buffer with random data
	x_random_fill(buf, len + 3);

	// copy instruction
	memcpy(buf, dis->ud.buffer, len);

	// VR[0] = &exec_buff
	I(EXEC_BUF, 0, 0, VR[0], 0, 0);
	I(ASGN, 0, 0, VR[1], buf_offset, 0);
	I(ADD, 0, 0, 4, VR[0], VR[1]);

	// update ESP and ret addr
	I(MOV, 0, 0, VR[6], NR(ESP), 0);
	I(ASGN, 0, 0, VR[7], -4, 0);
	I(ADD, 0, 0, 4, VR[6], VR[7]);
	I(MOV, 0, 0, NR(ESP), VR[6], 0);
	I(WR, 0, 0, 4, VR[6], VR[0]);

	// copy instruction data
	copied_len = 0;

	I(ASGN, 0, 0, VR[3], 4, 0);
	I(MOV, 0, 0, VR[4], VR[0], 0);

	while (copied_len < len) {
		I(ASGN, 0, 0, VR[2], *MakePtr(UINT32 *, buf, copied_len), 0);
		I(WR, 0, 0, 4, VR[4], VR[2]);
		I(ADD, 0, 0, 4, VR[4], VR[3]);

		copied_len += 4;
	}

	// fix relocations
	for (unsigned i = 0; i < dis->op.size(); i++) {
		op = &dis->op[i];

		if (op->is_reloc) {
			// fix addr
			I(ASGN, 0, 0, VR[10], *MakePtr(UINT32 *, buf, op->reloc_offset), 0);
			I(FIXRLC, 0, 0, VR[10], 0, 0);

			// write it to buf
			I(MOV, 0, 0, VR[8], VR[0], 0);
			I(ASGN, 0, 0, VR[9], op->reloc_offset, 0);
			I(ADD, 0, 0, 4, VR[8], VR[9]);
			I(WR, 0, 0, 4, VR[8], VR[10]);

			op->reloc_handled = true;
		}
	}

	// calculate end of instruction
	I(ASGN, 0, 0, VR[5], len, 0);
	I(ADD, 0, 0, 4, VR[0], VR[5]);

	// execute instruction
	I(EXEC, 0, 0, VR[0], 0, 0);

#if X_DEBUG > X_DEBUG_NONE
	printf("Unsupported instruction(%s): virtualizing in its original form\n", dis->txt.c_str());
#endif
XL_END_INS_VIRT
