// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_virtualize.h 656 2015-02-25 11:40:08Z tzubcic $

#pragma once

// common includes
#include "../xobf/x_common.h"

// local headers
#include "xl_vm.h"
#include "xl_disassembly.h"
#include "xl_operands.h"

// instruction virtualizer
#define DECLARE_VIRT(x)	void xl_virtualize_ ## x(xl_vm_t *, xl_disasm_t *)

// jump from vm to real code
void xl_virt_jmp_to_x86(xl_vm_t *, UINT32);
void xl_virt_jmp_to_vm_reg(xl_vm_t *, UINT32);

// jump to next block part
void xl_virt_jmp_by_va(xl_vm_t *, UINT32);

// emit a random labeled junk function
void xl_virt_emit_labeled_junk(xl_vm_t *, UINT32);

// assign an immediate operand
void xl_virt_asgn_imm(xl_vm_t *, UINT32, UINT32, bool);

// load/store x86 reg
UINT32 xl_virt_load_reg(xl_vm_t *, UINT32, UINT32);
UINT32 xl_virt_load_reg_ptr(xl_vm_t *, UINT32, UINT32);
void xL_virt_store_reg(xl_vm_t *, UINT32, UINT32);

// load x86 imm
void xl_virt_load_imm(xl_vm_t *, UINT32, xl_op_t *);

// vm flags modification
void xl_virt_enable_vm_flags(xl_vm_t *, UINT32);
void xl_virt_disable_vm_flags(xl_vm_t *, UINT32);

// load x86 mem
UINT32 xl_virt_load_x86_mem(xl_vm_t *, UINT32, xl_op_t *);
UINT32 xl_virt_get_x86_mem_size(xl_op_t *);

// fake jmp to random instruction
void xl_emit_fake_jmp(xl_vm_t *);

// push*
DECLARE_VIRT(push);
DECLARE_VIRT(pushad);
DECLARE_VIRT(pushfd);

// pop*
DECLARE_VIRT(pop);
DECLARE_VIRT(popad);
DECLARE_VIRT(popfd);

// mov*
DECLARE_VIRT(mov);
DECLARE_VIRT(movzx);
DECLARE_VIRT(movsx);
DECLARE_VIRT(xchg);

// arithmetic
DECLARE_VIRT(add);
DECLARE_VIRT(sub);
DECLARE_VIRT(inc);
DECLARE_VIRT(dec);

// logical
DECLARE_VIRT(or);
DECLARE_VIRT(xor);
DECLARE_VIRT(and);
DECLARE_VIRT(not);
DECLARE_VIRT(shl);
DECLARE_VIRT(shr);
DECLARE_VIRT(rol);
DECLARE_VIRT(ror);

// comparisons
DECLARE_VIRT(cmp);
DECLARE_VIRT(test);

// jumps
DECLARE_VIRT(jmp);

// procedures
DECLARE_VIRT(call);
DECLARE_VIRT(ret);

// other

// execute instruction in its original form
DECLARE_VIRT(original);
