// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_vm.cpp 660 2015-02-25 12:11:30Z tzubcic $

#include "xl_local.h"

xl_vm_x86_reg_t vm_x86_registers[] = {
	//  udis86		BASE			OFFSET	SIZE
	{	UD_R_EAX,	X_VMR_N_EAX,	0,		4 }, // EAX and sub-registers
	{	UD_R_AX,	X_VMR_N_EAX,	0,		2 },
	{	UD_R_AH,	X_VMR_N_EAX,	1,		1 },
	{	UD_R_AL,	X_VMR_N_EAX,	0,		1 },

	{	UD_R_EBX,	X_VMR_N_EBX,	0,		4 }, // EBX and sub-registers
	{	UD_R_BX,	X_VMR_N_EBX,	0,		2 },
	{	UD_R_BH,	X_VMR_N_EBX,	1,		1 },
	{	UD_R_BL,	X_VMR_N_EBX,	0,		1 },

	{	UD_R_ECX,	X_VMR_N_ECX,	0,		4 }, // ECX and sub-registers
	{	UD_R_CX,	X_VMR_N_ECX,	0,		2 },
	{	UD_R_CH,	X_VMR_N_ECX,	1,		1 },
	{	UD_R_CL,	X_VMR_N_ECX,	0,		1 },

	{	UD_R_EDX,	X_VMR_N_EDX,	0,		4 }, // EDX and sub-registers
	{	UD_R_DX,	X_VMR_N_EDX,	0,		2 },
	{	UD_R_DH,	X_VMR_N_EDX,	1,		1 },
	{	UD_R_DL,	X_VMR_N_EDX,	0,		1 },

	{	UD_R_EDI,	X_VMR_N_EDI,	0,		4 }, // EDI and sub-registers
	{	UD_R_DI,	X_VMR_N_EDI,	0,		2 },

	{	UD_R_ESI,	X_VMR_N_ESI,	0,		4 }, // ESI and sub-registers
	{	UD_R_SI,	X_VMR_N_ESI,	0,		2 },

	{	UD_R_EBP,	X_VMR_N_EBP,	0,		4 }, // EBP and sub-registers
	{	UD_R_EBP,	X_VMR_N_EBP,	0,		2 },

	{	UD_R_ESP,	X_VMR_N_ESP,	0,		4 }, // ESP and sub-registers
	{	UD_R_SP,	X_VMR_N_ESP,	0,		2 },
};

const size_t num_vm_x86_registers = sizeof(vm_x86_registers) / sizeof(vm_x86_registers[0]);

void xl_generate_virtual_machines(void)
{
	unsigned	amount;

	// clear the VMs
	xl.vms.clear();

	amount = x_value_range(xl.opt_num_vms, 1, X_VM_MAX_AMOUNT);
	amount = x_value_range(amount, 1, xl.blocks.size());

	if ((xl.blocks.size() > 1) && (amount == 1)) {
		printf("WARNING: It is suggested to use at least 2 virtual machines\n");
	}

	printf("[+] Generating %u virtual machine%s\n", amount, amount > 1 ? "s" : "");

	for (unsigned i = 0; i < amount; i++) {
		xl_generate_vm();
	}
}

void xl_generate_vm(void)
{
	xl_vm_t			vm;
	UINT32			*tls_slot;

	// vm id
	vm.id = xl.vms.size();

	// assign regular instruction ids
	for (unsigned i = 0; i < X_VMI_TOTAL; i++) {
		vm.ins_id[i] = i;
	}

	// assign regular register ids
	for (unsigned i = 0; i < X_VMR_TOTAL; i++) {
		vm.reg_id[i] = i;
	}

	// initial max instruction length
	vm.longest_instruction = 0;

	// register data for handler information
	vm.data_vmi_rvas = xl_register_data(XL_DT_INITIALIZED, X_VMI_TOTAL * 4);
	vm.data_vmi_lengths = xl_register_data(XL_DT_INITIALIZED, X_VMI_TOTAL);

	// register data for TLS slot
	vm.data_tls_slot = xl_register_data(XL_DT_INITIALIZED, 4);

	tls_slot = (UINT32 *) xl_get_data(vm.data_tls_slot);
	*tls_slot = (UINT32) TLS_OUT_OF_INDEXES;

#if 1
	// randomize vm architecture
	random_shuffle(&vm.ins_id[0], &vm.ins_id[X_VMI_TOTAL], x_rand_generator);
	random_shuffle(&vm.reg_id[0], &vm.reg_id[X_VMR_TOTAL], x_rand_generator);
#endif

	// convert random instruction id to internal id
	for (unsigned i = 0; i < X_VMI_TOTAL; i++) {
		vm.ins_id_real[vm.ins_id[i]] = i;
	}

	// vm registers are unused by default
	for (unsigned i = 0; i < X_VM_REG_NUM; i++) {
		vm.reg_reserved[i] = false;
	}

	xl.vms.push_back(vm);
}

void xl_analyze_block_for_virtualization(xl_block_t *b)
{
	// invalid bytecode address by default
	b->bc_va = 0;

	// invalid vm information
	b->vm = NULL;
}

void xl_assign_vm_to_blocks(void)
{
	unsigned			num_blocks, num_vms;
	unsigned			id;
	vector<unsigned>	ids;

	if (!xl.vms.size() || !xl.blocks.size()) {
		xl_code_gen_error(XL_ERR_ASGN_VM_TO_BLOCKS_NO_DATA);
	}

	num_blocks = xl.blocks.size();
	num_vms = xl.vms.size();

	for (unsigned i = 0; i < num_blocks; i++) {
		id = i % num_vms;
		ids.push_back(id);
	}

	random_shuffle(ids.begin(), ids.end(), x_rand_generator);

	for (unsigned i = 0; i < num_blocks; i++) {
		xl.blocks[i].vm = &xl.vms[ids[i]];
	}
}

#define AB(x, y)	do { \
						(*(UINT8 *) &x[offset]) = (y & 0xFF); \
						offset += 1; \
					} while(0)

#define AW(x, y)	do { \
						(*(UINT16 *) &x[offset]) = (y & 0xFFFF); \
						offset += 2; \
					} while(0)

#define AD(x, y)	do { \
						(*(UINT32 *) &x[offset]) = (y); \
						offset += 4; \
					} while(0)

void xl_assemble_vm_code(void)
{
	xl_vm_code_t		*code;
	xl_vm_instr_t		*ins;
	xl_vm_t				*vm;
	x_vm_ins_info_t		*ii;
	vector<UINT8>		id;
	vector<UINT8>		bd;
	UINT32				offset;
	unsigned			len;
	void				*data;

	code = &xl.vm_code;

	if (!code->ins.size()) {
		xl_code_gen_error(XL_ERR_ASSEMBLE_VM_CODE_NO_INS);
	}

	for (unsigned i = 0; i < code->ins.size(); i++) {
		// current instruction
		ins = &code->ins[i];

		// vm of current instruction
		vm = ins->vm;

		// clear the buffer
		id.clear();

		// reset data offset
		offset = 0;

		// mnemonic
		len = X_VM_MNEMONIC_LEN;
		id.resize(len);

		AW(id, xl_encode_mnemonic(vm, ins->mnemonic));

		// operands
		if (ins->mnemonic < X_VMI_TOTAL) {
			// instruction info
			ii = x_get_vm_ins_info(ins->mnemonic);

			for (unsigned j = 0; j < 4; j++) {
				len += x_get_vm_operand_len(ii->op_types[j]);
			}

			id.resize(len);

			for (unsigned j = 0; j < 4; j++) {
				if (ii->op_types[j] == X_VMI_OP_NONE) {
					break;
				}

				if (j == 0) {
					if (!ins->op[j]) {
						ins->op[j] = x_rand_range(X_CC_TOTAL, 0xFF);
					}
				}

				if (ii->op_types[j] == X_VMI_OP_REG) {
					AB(id, xl_encode_register(vm, ins->mnemonic, j, ins->op[j]));
				} else if (ii->op_types[j] == X_VMI_OP_BYTE) {
					AB(id, xl_encode_op_byte(vm, ins->mnemonic, j, ins->op[j]));
				} else if (ii->op_types[j] == X_VMI_OP_DWORD) {
					AD(id, xl_encode_op_dword(vm, ins->mnemonic, j, ins->op[j]));
				} else if (ii->op_types[j] == X_VMI_OP_REL_DWORD) {
					AD(id, xl_encode_op_dword(vm, ins->mnemonic, j, ins->op[j]));
				} else if (ii->op_types[j] == X_VMI_OP_SIZE) {
					if (ins->op[j] == 1 || ins->op[j] == 2) {
						// nothing
					} else if (ins->op[j] == 4) {
						ins->op[j] = x_rand_range(4, 0xFF);
					} else {
						xl_code_gen_error(XL_ERR_ASSEMBLE_VM_CODE_BAD_SIZE);
					}

					AB(id, xl_encode_op_byte(vm, ins->mnemonic, j, ins->op[j]));
				} else {
					xl_code_gen_error(XL_ERR_ASSEMBLE_VM_CODE_BAD_OP);
				}
			}
		}

		if (id.size() > vm->longest_instruction) {
			vm->longest_instruction = id.size();
		}

		xl_encrypt_instruction(vm, &id[0], id.size());

		for (unsigned j = 0; j < id.size(); j++) {
			bd.push_back(id[j]);
		}
	}

	if (!bd.size()) {
		xl_code_gen_error(XL_ERR_ASSEMBLE_VM_CODE_NO_BYTECODE);
	}

	// update longest lengths
	for (unsigned i = 0; i < xl.vms.size(); i++) {
		vm = &xl.vms[i];

		vm->longest_instruction += 3;
		vm->longest_instruction &= 0xFFFFFFFC;
	}

	// write byte code to new section
	code->data_id = xl_register_data(XL_DT_INITIALIZED, bd.size());

	data = xl_get_data(code->data_id);

	memcpy(data, &bd[0], bd.size());
}

void xl_encrypt_instruction(xl_vm_t *vm, UINT8 *buf, unsigned len)
{
	UINT32		*loc_dw;
	UINT8		loc_buf[1024];
	unsigned	encrypted;

	// copy to temp buffer
	memcpy(loc_buf, buf, len);

	// initial values
	encrypted = 0;
	loc_dw = (UINT32 *) loc_buf;

	while (encrypted < len) {
		// next dword
		encrypted += 4;
		loc_dw++;
	}

	// copy encrypted data
	memcpy(buf, loc_buf, len);
}

void xl_set_vm_data(void)
{
	xl_vm_t				*vm;
	UINT8				*length;
	x_vm_ins_info_t		*ii;
	uint16_t			ins_real;
	uint16_t			ins_rnd;

	for (unsigned i = 0; i < xl.vms.size(); i++) {
		vm = &xl.vms[i];

		length = (UINT8 *) xl_get_data(vm->data_vmi_lengths);

		for (unsigned j = 0; j < X_VMI_TOTAL; j++) {
			ins_real = vm->ins_id_real[vm->ins_id[j]];
			ins_rnd = vm->ins_id[j];
			ii = x_get_vm_ins_info(ins_real);

			length[ins_rnd] = x_encode_byte(0, x_get_vm_ins_size(ins_real), ins_rnd);
		}
	}
}

void xl_set_bytecode_entry_points(void)
{
	xl_block_t		*b;
	UINT32			bytecode_base;

	bytecode_base = xl_get_data_va(xl.vm_code.data_id);

	for (unsigned i = 0; i < xl.blocks.size(); i++) {
		b = &xl.blocks[i];

		b->bc_va = bytecode_base + b->bc_offset;
		b->bc_rva = b->bc_va - xl.module_base;
	}
}

void xl_assemble_jumps_to_vm(void)
{
	xl_block_t		*b;
	UINT32			jmp_offset;

	for (unsigned i = 0; i < xl.blocks.size(); i++) {
		b = &xl.blocks[i];

		// erase original code
		x_random_fill(b->start, b->len);

		// assemble jump
		*(b->start) = 0xE9;
		jmp_offset = b->bc_va - b->va - 5;
		*MakePtr(UINT32 *, b->start, 1) = jmp_offset;
	}
}

void xl_vm_resolve_references(void)
{
	xl_vm_instr_t		*ins;
	xl_vm_instr_t		*dst;
	x_vm_ins_info_t		*ii;
	UINT32				ref;
	UINT32				rel;
	bool				corrected;

	for (unsigned i = 0; i < xl.vm_code.ins.size(); i++) {
		ins = &xl.vm_code.ins[i];

		if (ins->mnemonic < X_VMI_TOTAL) {
			ii = x_get_vm_ins_info(ins->mnemonic);

			for (unsigned j = 0; j < 4; j++) {
				if (ii->op_types[j] == X_VMI_OP_NONE) {
					break;
				}

				if (ii->op_types[j] == X_VMI_OP_REL_DWORD) {
					ref = ins->op[j];
					corrected = false;

					if (ref == (1 << 30)) {
						dst = &xl.vm_code.ins[x_rand_range(0, xl.vm_code.ins.size() - 1)];

						rel = dst->offset - (ins->offset + X_VM_MNEMONIC_LEN + x_get_vm_ins_size(ins->mnemonic));
						ins->op[j] = rel;

						corrected = true;
					} else {
						for (unsigned k = 0; k < xl.vm_code.ins.size(); k++) {
							dst = &xl.vm_code.ins[k];

							if (dst->ref == ref) {
								rel = dst->offset - (ins->offset + X_VM_MNEMONIC_LEN + x_get_vm_ins_size(ins->mnemonic));
								ins->op[j] = rel;
								corrected = true;
								break;
							}
						}
					}

					if (!corrected) {
						xl_code_gen_error(XL_ERR_VM_RESOLVE_REFERENCES_UNRESOLVED);
					}
				}
			}
		}
	}
}

void xl_add_vm_instruction(xl_vm_t *vm, UINT32 reference, UINT32 mn, UINT32 cc, UINT32 arg1, UINT32 arg2, UINT32 arg3)
{
	xl_vm_instr_t		ins;

	// vm owner and mnemonic
	ins.vm = vm;
	ins.mnemonic = mn;

	// reference to this instruction
	ins.ref = reference;

	// operands
	ins.op[0] = cc;
	ins.op[1] = arg1;
	ins.op[2] = arg2;
	ins.op[3] = arg3;

	// offset
	ins.offset = xl.vm_code.offset;

	// add instruction
	xl.vm_code.ins.push_back(ins);

	// update offset
	xl.vm_code.offset += (X_VM_MNEMONIC_LEN + x_get_vm_ins_size(ins.mnemonic));

	// randomly emit invalid mnemonics
	if (x_rand_range(0, 2) == 2) {
		xl_add_invalid_vm_instruction(vm);
	}
}

void xl_add_invalid_vm_instruction(xl_vm_t *vm)
{
	xl_vm_instr_t		ins;

	// vm owner and mnemonic
	ins.vm = vm;
	ins.mnemonic = x_rand_range(X_VMI_TOTAL, 0xFFFF);

	// reference to this instruction
	ins.ref = 0;

	// offset
	ins.offset = xl.vm_code.offset;

	// add instruction
	xl.vm_code.ins.push_back(ins);

	// update offset
	xl.vm_code.offset += (X_VM_MNEMONIC_LEN);
}

UINT32 xl_vm_get_label(void)
{
	UINT32		ret;

	// first increment, 0 is not a valid reference
	xl.vm_code.next_ref++;
	ret = xl.vm_code.next_ref;

	return ret;
}

xl_vm_x86_reg_t *xl_x86_reg_to_vm(UINT32 reg)
{
	for (unsigned i = 0; i < num_vm_x86_registers; i++) {
		if (vm_x86_registers[i].udis_reg == reg) {
			return &vm_x86_registers[i];
		}
	}

	xl_code_gen_error(XL_ERR_X86_REG_TO_VM_HANDLING_ERROR);

	return NULL;
}

UINT16 xl_encode_mnemonic(xl_vm_t *vm, unsigned mn)
{
	UINT16		ret;

	if (mn >= X_VMI_TOTAL) {
		ret = mn;
	} else {
		ret = vm->ins_id[mn];
	}

	return ret;
}

UINT8 xl_encode_register(xl_vm_t *vm, unsigned mn, unsigned op_id, unsigned reg)
{
	UINT8		ret;

	if (mn >= X_VMI_TOTAL) {
		xl_code_gen_error(XL_ERR_ENCODE_REGISTER_BAD_MN);
	}

	if (reg >= X_VMR_TOTAL) {
		xl_code_gen_error(XL_ERR_ENCODE_REGISTER_BAD_REG);
	}

	if (op_id >= 4) {
		xl_code_gen_error(XL_ERR_ENCODE_OP_BAD_ID);
	}

	ret = (UINT8) vm->reg_id[reg];

	return ret;
}

UINT8 xl_encode_op_byte(xl_vm_t *vm, unsigned mn, unsigned op_id, UINT32 val)
{
	UINT8		ret;

	if (mn >= X_VMI_TOTAL) {
		xl_code_gen_error(XL_ERR_ENCODE_BYTE_BAD_MN);
	}

	if (op_id >= 4) {
		xl_code_gen_error(XL_ERR_ENCODE_OP_BAD_ID);
	}

	ret = (UINT8) val;

	return ret;
}

UINT32 xl_encode_op_dword(xl_vm_t *vm, unsigned mn, unsigned op_id, UINT32 val)
{
	UINT32		ret;

	if (mn >= X_VMI_TOTAL) {
		xl_code_gen_error(XL_ERR_ENCODE_DWORD_BAD_MN);
	}

	if (op_id >= 4) {
		xl_code_gen_error(XL_ERR_ENCODE_OP_BAD_ID);
	}

	ret = val;

	return ret;
}

UINT32 xl_reserve_vm_reg(xl_vm_t *vm)
{
	UINT32		id;

	while (1) {
		id = x_rand_range(0, X_VM_REG_NUM - 1);

		if (!vm->reg_reserved[id]) {
			vm->reg_reserved[id] = true;
			id += X_VMR_VM_START;

			break;
		}
	}

	return id;
}

void xl_free_vm_reg(xl_vm_t *vm, UINT32 vm_reg_id)
{
	vm_reg_id -= X_VMR_VM_START;

	if (vm_reg_id >= X_VM_REG_NUM) {
		xl_code_gen_error(XL_ERR_VM_REGISTER_HANDLING_ERROR);
	}

	if (!vm->reg_reserved[vm_reg_id]) {
		xl_code_gen_error(XL_ERR_VM_REGISTER_HANDLING_ERROR);
	}

	vm->reg_reserved[vm_reg_id] = false;
}

void xl_abort_if_vm_regs_not_free(xl_vm_t *vm)
{
	for (unsigned i = 0; i < X_VM_REG_NUM; i++) {
		if (vm->reg_reserved[i]) {
			xl_code_gen_error(XL_ERR_VM_REGISTER_HANDLING_ERROR);
		}
	}
}
