// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: xl_vm.h 660 2015-02-25 12:11:30Z tzubcic $

#pragma once

// common includes
#include "../xobf/x_common.h"

// local headers
#include "xl_blocks.h"
#include "xl_disassembly.h"

typedef struct xl_vm_s {
	// vm id
	UINT32					id;

	// instruction ids
	uint16_t				ins_id[X_VMI_TOTAL];
	uint16_t				ins_id_real[X_VMI_TOTAL];

	// register ids
	uint16_t				reg_id[X_VMR_TOTAL];

	// longest instruction
	unsigned				longest_instruction;

	// handler information
	unsigned				data_vmi_rvas;
	unsigned				data_vmi_lengths;

	// vm register state
	bool					reg_reserved[X_VM_REG_NUM];

	// TLS slot
	UINT32					data_tls_slot;
} xl_vm_t;

typedef struct {
	// instruction data
	UINT32							mnemonic;
	UINT32							op[4];

	// reference to the instruction
	UINT32							ref;

	// offset in vm code
	UINT32							offset;

	// vm owner
	xl_vm_t							*vm;
} xl_vm_instr_t;

typedef struct {
	// instructions
	vector<xl_vm_instr_t>		ins;

	// current offset
	UINT32						offset;

	// bytecode data id
	unsigned					data_id;

	// next available reference
	UINT32						next_ref;
} xl_vm_code_t;

typedef struct {
	// udis86 register
	UINT32							udis_reg;

	// VMx86 register base id, offset and size
	UINT32							vmreg;
	UINT32							offset;
	UINT32							size;
} xl_vm_x86_reg_t;

// generate virtual machines
void xl_generate_virtual_machines(void);

// generate a single virtual machine
void xl_generate_vm(void);

// separate analyzation procedure for virtualization blocks
void xl_analyze_block_for_virtualization(xl_block_t *);

// assign random vm to blocks
void xl_assign_vm_to_blocks(void);

// assemble vm bytecode
void xl_assemble_vm_code(void);

// encrypt an instruction
void xl_encrypt_instruction(xl_vm_t *, UINT8 *, unsigned);

// set vm initialized data
void xl_set_vm_data(void);

// set each block's bytecode entry address
void xl_set_bytecode_entry_points(void);

// jmp from unvirtualized code to vm
void xl_assemble_jumps_to_vm(void);

// correct relative jumps
void xl_vm_resolve_references(void);

// emit a vm instruction
void xl_add_vm_instruction(xl_vm_t *, UINT32, UINT32, UINT32, UINT32, UINT32, UINT32);
void xl_add_invalid_vm_instruction(xl_vm_t *);

// get a new vm bytecode label
UINT32 xl_vm_get_label(void);

// get x86 register info
xl_vm_x86_reg_t *xl_x86_reg_to_vm(UINT32);

// encode vm operands
UINT16 xl_encode_mnemonic(xl_vm_t *, unsigned);
UINT8 xl_encode_register(xl_vm_t *, unsigned, unsigned, unsigned);
UINT8 xl_encode_op_byte(xl_vm_t *, unsigned, unsigned, UINT32);
UINT32 xl_encode_op_dword(xl_vm_t *, unsigned, unsigned, UINT32);

// reserve/free vm register
UINT32 xl_reserve_vm_reg(xl_vm_t *);
void xl_free_vm_reg(xl_vm_t *, UINT32);
void xl_abort_if_vm_regs_not_free(xl_vm_t *);
