// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_asmjit.cpp 657 2015-02-25 11:42:33Z tzubcic $

#include "x_local.h"

// assembler
#define A			(*xg.code_gen)

const X86GpReg *x_udis_reg_to_asmjit_reg(unsigned reg)
{
	// 8bit registers
	if (reg == UD_R_AL) return &al;
	if (reg == UD_R_BL) return &bl;
	if (reg == UD_R_CL) return &cl;
	if (reg == UD_R_DL) return &dl;
	if (reg == UD_R_AH) return &ah;
	if (reg == UD_R_BH) return &bh;
	if (reg == UD_R_CH) return &ch;
	if (reg == UD_R_DH) return &dh;

	// 16bit registers
	if (reg == UD_R_AX) return &ax;
	if (reg == UD_R_BX) return &bx;
	if (reg == UD_R_CX) return &cx;
	if (reg == UD_R_DX) return &dx;
	if (reg == UD_R_SP) return &sp;
	if (reg == UD_R_BP) return &bp;
	if (reg == UD_R_SI) return &si;
	if (reg == UD_R_DI) return &di;

	// 32bit registers
	if (reg == UD_R_EAX) return &eax;
	if (reg == UD_R_EBX) return &ebx;
	if (reg == UD_R_ECX) return &ecx;
	if (reg == UD_R_EDX) return &edx;
	if (reg == UD_R_ESP) return &esp;
	if (reg == UD_R_EBP) return &ebp;
	if (reg == UD_R_ESI) return &esi;
	if (reg == UD_R_EDI) return &edi;

	x_exit_thread(X_ERR_UDIS_REG_TO_ASMJIT_UNK_REG);

	return NULL;
}

#define U2A(x)		*x_udis_reg_to_asmjit_reg(x)

X86Mem x_mem_op_to_asmjit(x_op_t *op)
{
	Mem				mem;

	if (op->type != UD_OP_MEM) {
		x_exit_thread(X_ERR_MEM_OP_TO_ASMJIT_NOT_MEM);
	}

	// [base_reg + index*scale + offset]
	if (op->data.mem.base && op->data.mem.index) {
		if (op->data.mem.size == 8) {
			mem = byte_ptr(U2A(op->data.mem.base), U2A(op->data.mem.index), op->data.mem.scale, op->data.mem.offset);
		} else if (op->data.mem.size == 16) {
			mem = word_ptr(U2A(op->data.mem.base), U2A(op->data.mem.index), op->data.mem.scale, op->data.mem.offset);
		} else if (op->data.mem.size == 32) {
			mem = dword_ptr(U2A(op->data.mem.base), U2A(op->data.mem.index), op->data.mem.scale, op->data.mem.offset);
		} else {
			mem = Mem(U2A(op->data.mem.base), U2A(op->data.mem.index), op->data.mem.scale, op->data.mem.offset);
		}
	} else {
		// [base_reg + offset]
		if (!op->data.mem.index && op->data.mem.base) {
			if (op->data.mem.size == 8) {
				mem = byte_ptr(U2A(op->data.mem.base), op->data.mem.offset);
			} else if (op->data.mem.size == 16) {
				mem = word_ptr(U2A(op->data.mem.base), op->data.mem.offset);
			} else if (op->data.mem.size == 32) {
				mem = dword_ptr(U2A(op->data.mem.base), op->data.mem.offset);
			} else {
				mem = Mem(U2A(op->data.mem.base), op->data.mem.offset);
			}
		// [offset]
		} else if (!op->data.mem.base && !op->data.mem.index) {
			if (op->data.mem.size == 8) {
				mem = byte_ptr_abs(op->data.mem.offset);
			} else if (op->data.mem.size == 16) {
				mem = word_ptr_abs(op->data.mem.offset);
			} else if (op->data.mem.size == 32) {
				mem = dword_ptr_abs(op->data.mem.offset);
			} else {
				mem = ptr_abs(op->data.mem.offset);
			}
		} else {
			x_exit_thread(X_ERR_MEM_OP_TO_ASMJIT_UNSUPPORTED_OP);
		}
	}

	return mem;
}

Label *x_new_label(void)
{
	Label		*label;

	// allocate a new label
	label = new Label(A.newLabel());

	// backup pointer so it can be deleted
	xg.ctx->allocated_labels.push_back(label);

	return label;
}

void x_bind_label(Label *label)
{
	A.bind(*label);
}

UINT32 x_write_memory(UINT8 *data, UINT32 len)
{
	UINT32		off;

	off = A.getCodeSize();

	A.embed(data, len);

	return off;
}

void x_emit_random_bytes(void)
{
	UINT8	data[8];

	x_random_fill(data, sizeof(data));

	x_write_memory(data, x_rand_range(2, sizeof(data)));
}
