// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_block_parts.cpp 657 2015-02-25 11:42:33Z tzubcic $

#include "x_local.h"

// assembler
#define A			(*xg.code_gen)

// ignore shuffle temporary variables
UINT32	ign_shuffle_start;

void x_init_shuffle_for_current_pass(unsigned pass)
{
	if (pass == 0) {
		xg.ctx->min_block_part_size = 8;
		xg.ctx->max_block_part_size = 16;
	} else {
		x_exit_thread(X_ERR_INIT_SHUFFLE_PASS_BAD_PASS);
	}

#if X_DEBUG > X_DEBUG_NONE
	xg.ctx->min_block_part_size = 10000000;
	xg.ctx->max_block_part_size = 10000000;
#endif
}

void x_split_blocks(void)
{
	unsigned		split_num, add_num;
	x_bp_t			part;
	x_disasm_t		*dis;
	bool			allowed, forced;

	// clear existing parts
	xg.ctx->parts.clear();

	// get random number of instructions for this block part
	split_num = x_rand_range(xg.ctx->min_block_part_size, xg.ctx->max_block_part_size);

	add_num = 0;

	x_init_code_block_part(&part, split_num);

	for (unsigned i = 0; i < xg.ctx->block.disasm.size(); i++) {
		dis = &xg.ctx->block.disasm[i];

		allowed = x_split_allowed(dis);
		forced = x_split_forced(dis);

		if (!allowed && forced) {
			x_exit_thread(X_ERR_SPLIT_BLOCKS_ALLOWED_FORCED);
		}

		if (((allowed && (add_num >= split_num)) || forced)) {
			// add a block part
			x_add_code_block_part(&part);

			// initialize a new block part
			x_init_code_block_part(&part, split_num);

			// choose random block size
			split_num = x_rand_range(xg.ctx->min_block_part_size, xg.ctx->max_block_part_size);
			add_num = 0;
		}

		part.code.dis.push_back(dis);
		add_num++;
	}

	// add leftover instructions
	if (add_num) {
		x_add_code_block_part(&part);
	}
}

void x_init_code_block_part(x_bp_t *bp, unsigned max_dis)
{
	// clear the disasm list
	bp->code.dis.clear();

	// reserve space
	// FIXME: check if this slows down the app
	// TODO: replace vector with num_dis and pointer to first dis
	bp->code.dis.reserve(max_dis);

	// jump type
	bp->code.bpj_type = x_rand_range(0, X_BPJ_TOTAL - 1);
}

void x_add_code_block_part(x_bp_t *bp)
{
	bp->code.order = 0;
	bp->code.next = NULL;
	bp->code.prev = NULL;

	bp->code.label = x_new_label();

	if (xg.ctx->parts.size()) {
		bp->code.order = xg.ctx->parts[xg.ctx->parts.size() - 1].code.order + 1;
	}

	xg.ctx->parts.push_back(*bp);
}

void x_update_block_part_links(void)
{
	x_bp_t		*bpi, *bpj;

	for (unsigned i = 0; i < xg.ctx->parts.size(); i++) {
		xg.ctx->parts[i].code.next = NULL;
		xg.ctx->parts[i].code.prev = NULL;
	}

	for (unsigned i = 0; i < xg.ctx->parts.size(); i++) {
		bpi = &xg.ctx->parts[i];

		for (unsigned j = 0; j < xg.ctx->parts.size(); j++) {
			bpj = &xg.ctx->parts[j];

			if (bpi->code.order == (bpj->code.order - 1)) {
				bpi->code.next = bpj;
				bpj->code.prev = bpi;
			}
		}
	}
}

void x_shuffle_block_parts(void)
{
	// shuffle
	random_shuffle(xg.ctx->parts.begin(), xg.ctx->parts.end(), x_rand_generator);

	// re-link
	x_update_block_part_links();
}

void x_process_block_parts(void)
{
	// shuffle block parts
	x_shuffle_block_parts();

	// emit all block parts
	for (unsigned i = 0; i < xg.ctx->parts.size(); i++) {
		x_emit_block_part(&xg.ctx->parts[i]);
	}
}

#define ___X_MC_ELSEIF(x)		else if (dis->type == UD_I ## x) x_mutate_ ## x(dis)

void x_emit_block_part(x_bp_t *bp)
{
	x_bp_code_t			*code;
	x_disasm_t			*dis;

	code = &bp->code;

	if (!code->dis.size()) {
		x_exit_thread(X_ERR_EMIT_BP_NO_DIS);
	}

	// reference to this part
	x_bind_label(code->label);

	if (code->prev) {
		x_get_back_from_previous_block_part(code->dis[0], code->prev->code.bpj_type);
	}

	// mutate all instructions in this block part
	for (unsigned i = 0; i < code->dis.size(); i++) {
		dis = code->dis[i];

		x_bind_label(dis->label);

		// correct function VAs
		x_set_function_addr(dis);

		// backup address for referencing after shuffling
		xg.ctx->asl[dis->va] = xg.ctx->new_code_va + A.getCodeSize();

		if (0) {

		}

		// push*
		___X_MC_ELSEIF(push);
		___X_MC_ELSEIF(pushad);
		___X_MC_ELSEIF(pushfd);

		// pop*
		___X_MC_ELSEIF(pop);
		___X_MC_ELSEIF(popad);
		___X_MC_ELSEIF(popfd);

		// mov*
		___X_MC_ELSEIF(mov);
		___X_MC_ELSEIF(movzx);
		___X_MC_ELSEIF(movsx);

		// arithmetic
		___X_MC_ELSEIF(add);
		___X_MC_ELSEIF(sub);
		___X_MC_ELSEIF(mul);
		___X_MC_ELSEIF(imul);
		___X_MC_ELSEIF(div);
		___X_MC_ELSEIF(idiv);
		___X_MC_ELSEIF(inc);
		___X_MC_ELSEIF(dec);
		___X_MC_ELSEIF(adc);
		___X_MC_ELSEIF(neg);
		___X_MC_ELSEIF(sbb);

		// logical
		___X_MC_ELSEIF(or);
		___X_MC_ELSEIF(xor);
		___X_MC_ELSEIF(and);
		___X_MC_ELSEIF(not);
		___X_MC_ELSEIF(shl);
		___X_MC_ELSEIF(shr);
		___X_MC_ELSEIF(ror);
		___X_MC_ELSEIF(rol);
		___X_MC_ELSEIF(shld);
		___X_MC_ELSEIF(shrd);

		// comparisons
		___X_MC_ELSEIF(cmp);
		___X_MC_ELSEIF(test);

		// jumps
		___X_MC_ELSEIF(jmp);
		___X_MC_ELSEIF(jo);
		___X_MC_ELSEIF(jno);
		___X_MC_ELSEIF(jb);
		___X_MC_ELSEIF(jae);
		___X_MC_ELSEIF(jz);
		___X_MC_ELSEIF(jnz);
		___X_MC_ELSEIF(jbe);
		___X_MC_ELSEIF(ja);
		___X_MC_ELSEIF(js);
		___X_MC_ELSEIF(jns);
		___X_MC_ELSEIF(jp);
		___X_MC_ELSEIF(jnp);
		___X_MC_ELSEIF(jl);
		___X_MC_ELSEIF(jge);
		___X_MC_ELSEIF(jle);
		___X_MC_ELSEIF(jg);

		// procedures
		___X_MC_ELSEIF(call);
		___X_MC_ELSEIF(ret);

		// other
		___X_MC_ELSEIF(nop);
		___X_MC_ELSEIF(lea);
		___X_MC_ELSEIF(xchg);
		___X_MC_ELSEIF(stc);
		___X_MC_ELSEIF(cld);
		___X_MC_ELSEIF(clc);
		___X_MC_ELSEIF(std);
		___X_MC_ELSEIF(cmc);
		___X_MC_ELSEIF(bt);

		// unsupported instruction
		else {
			x_exit_thread(X_ERR_EMIT_BP_UNSUPPORTED_INSTRUCTION);
		}

		// check if relocations have been handled
		for (unsigned j = 0; j < dis->op.size(); j++) {
			if (dis->op[j].is_reloc) {
				if (!dis->op[j].reloc_handled) {
					x_exit_thread(X_ERR_EMIT_BP_RELOC_NOT_HANDLED);
				}
			}
		}
	}

	// continue execution at next block part
	if (code->next) {
		x_continue_at_next_block_part(dis, code->next->code.label, code->bpj_type);
	}
}

void x_code_shuffling(bool state)
{
	UINT32				ign_shuffle_end;

	if (!state) {
		ign_shuffle_start = xg.ctx->new_code_va + A.getCodeSize();
	} else {
		ign_shuffle_end = xg.ctx->new_code_va + A.getCodeSize();
		x_add_ignore_shuffle(ign_shuffle_start, ign_shuffle_end);

		// force split
		x_force_shuffle();
	}
}

void x_add_ignore_shuffle(UINT32 start, UINT32 end)
{
	x_ign_shuffle_t		ign;

	ign.start = start;
	ign.end = end;

	xg.ctx->ign_shuffle.push_back(ign);
}

void x_force_shuffle(void)
{
	UINT32		va;

	va = xg.ctx->new_code_va + A.getCodeSize();

	xg.ctx->force_shuffle[va] = true;
}

bool x_split_allowed(x_disasm_t *dis)
{
	x_ign_shuffle_t		*ign;

	for (unsigned i = 0; i < xg.ctx->ign_shuffle.size(); i++) {
		ign = &xg.ctx->ign_shuffle[i];

		if ((dis->va > ign->start) && (dis->va < ign->end)) {
			return false;
		}
	}

	return true;
}

bool x_split_forced(x_disasm_t *dis)
{
	if (!xg.ctx->force_shuffle.count(dis->va)) {
		return false;
	}

	return true;
}

void x_continue_at_next_block_part(x_disasm_t *dis, Label *label, unsigned bpj_type)
{
	// restore register state
	x_restore_register_state(dis->va);

	if (bpj_type == X_BPJ_JMP) {
		A.jmp(*label);
		x_emit_random_bytes();
	} else if (bpj_type == X_BPJ_CALL) {
		A.call(*label);
		x_emit_random_bytes();
	} else {
		x_exit_thread(X_ERR_EMIT_BP_BAD_BPJ);
	}

	// clear register state
	x_clear_register_state();
}

void x_get_back_from_previous_block_part(x_disasm_t *dis, unsigned bpj_type)
{
	// restore register state
	x_restore_register_state(dis->va);

	if (bpj_type == X_BPJ_JMP) {
		x_emit_junk_operation();
	} else if (bpj_type == X_BPJ_CALL) {
		x_x86_reg_t		*tmp;

		x_emit_junk_operation();

		tmp = x_get_random_register_if_free();

		if (tmp) {
			A.pop(*(tmp->reg));
		} else {
			A.add(esp, 4);
		}
	} else {
		x_exit_thread(X_ERR_EMIT_BP_BAD_BPJ);
	}

	// clear register state
	x_clear_register_state();
}
