// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_block_parts.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// common includes
#include "x_common.h"

// jump type between block parts
enum {
	X_BPJ_JMP,
	X_BPJ_CALL,

	// total number of jump types
	X_BPJ_TOTAL
};

typedef struct {
	// order info
	unsigned				order;

	// block parts
	vector<x_disasm_t *>	dis;

	// block label
	Label					*label;

	// part links
	struct x_bp_s			*next;
	struct x_bp_s			*prev;

	// obfuscation type
	unsigned				bpj_type;
} x_bp_code_t;

typedef struct x_bp_s {
	// block part data
	x_bp_code_t		code;
} x_bp_t;

typedef struct {
	UINT32					start;
	UINT32					end;
} x_ign_shuffle_t;

// initialize code shuffling for current pass
void x_init_shuffle_for_current_pass(unsigned);

// split all blocks into smaller blocks
void x_split_blocks(void);

// initialize a block part
void x_init_code_block_part(x_bp_t *, unsigned);

// add a code block part
void x_add_code_block_part(x_bp_t *);

// link code block parts
void x_update_block_part_links(void);

// shuffle all blocks
void x_shuffle_block_parts(void);

// process all block parts
void x_process_block_parts(void);

// mutate a part
void x_emit_block_part(x_bp_t *);

// enable/disable code shuffling
void x_code_shuffling(bool);
void x_add_ignore_shuffle(UINT32, UINT32);

// force code shuffling
void x_force_shuffle(void);

// check if split/shuffle is allowed at current instruction
bool x_split_allowed(x_disasm_t *);
bool x_split_forced(x_disasm_t *);

// block part linking
void x_continue_at_next_block_part(x_disasm_t *, Label *, unsigned);
void x_get_back_from_previous_block_part(x_disasm_t *, unsigned);
