// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_blocks.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// common includes
#include "x_common.h"

// local headers
#include "x_disassembly.h"

typedef struct x_block_s {
	// block info
	UINT8					*start;
	UINT32					len;
	UINT32					rva;
	UINT32					va;
	UINT32					end_va;

	// real code info
	UINT8					*code_start;
	UINT32					code_len;
	UINT32					code_rva;
	UINT32					code_va;

	// block disassembly
	vector<x_disasm_t>		disasm;
} x_block_t;

// set block data
void x_set_block(UINT8 *, UINT8 *);

// disassemble the block
void x_disassemble_block(void);

// update disasm block pointer
void x_update_disasm_owner_block(void);

// analyze block's disassembly
void x_analyze_disassembly_in_block(void);

// analyze relocation info for mutation blocks
void x_analyze_relocation_for_mutation(void);

// verify that all instructions have been analyzed
void x_abort_if_instructions_not_analyzed(void);

// clear the block
void x_clear_block(void);

// link an address to a label
Label *x_addr_to_label(UINT32);
bool x_addr_in_block(UINT32);
