// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_context.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// common includes
#include "x_common.h"

// local headers
#include "x_vm.h"
#include "x_generic_functions.h"
#include "x_blocks.h"
#include "x_relocations.h"
#include "x_block_parts.h"
#include "x_data.h"
#include "x_asmjit_hook.h"
#include "x_junk.h"

typedef struct {
	// basic info about the module
	UINT32					module_base;
	UINT32					nsc_va;
	UINT32					nsc_rva;
	UINT32					oep_rva;
	UINT32					oep_va;
	bool					has_relocs;

	// virtualized blocks
	UINT32					num_blocks;
	vector<x_vm_block_t>	vm_blocks;

	// new section data size
	UINT32					nsc_data_size;
	UINT32					new_code_rva;
	UINT32					new_code_va;

	// imported functions
	UINT32					imports[X_IMP_TOTAL];

	// virtual machines
	UINT32					num_vms;
	vector<x_vm_t>			vms;

	// functions
	vector<x_func_info_t>	functions;

	// generic function ids
	unsigned				generic_functions[X_FUNC_GENERIC_TOTAL];

	// function data
	vector<UINT8>			gen_data;

	// code generation block
	x_block_t				block;

	// relocation info
	x_reloc_info_t			gen_reloc;

	// shuffled block parts
	vector<x_bp_t>			parts;

	// size of splitted blocks
	unsigned				min_block_part_size;
	unsigned				max_block_part_size;

	// final relocations
	vector<UINT32>			final_relocs;

	// track allocated labels
	vector<Label *>			allocated_labels;

	// hook state
	x_hook_state_t			hs;

	// use some things only during code generation, not mutation
	bool					generating_functions;

	// junk
	x_junk_functions_t		junk_instructions;
	x_junk_functions_t		junk_operations;

	// after-shuffle address linking
	map<UINT32, UINT32>		asl;

	// disable/force code shuffling
	vector<x_ign_shuffle_t>	ign_shuffle;
	map<UINT32, bool>		force_shuffle;

	// state of every instruction
	map<UINT32, UINT32>		rs;
} x_ctx_t;

// initialize context from buffer
void x_initialize_context(void *, unsigned);

// parse the whole block of data
bool x_parse_context_data(void);

// individual parsers
bool x_parse_context_data_vm_architectures(void);
bool x_parse_context_data_block_information(void);
bool x_parse_context_data_imports(void);
bool x_parse_context_data_data_addresses(void);

// initialize assembler
bool x_initialize_assembler(void);
bool x_initialize_temp_assembler(void);

// process context
bool x_process_context(void);

// assembler cleanup
void x_cleanup_assembler(void);

// context cleanup after code generation has finished
void x_cleanup_context(void);

// read individual values
bool x_ctx_read_byte(UINT8 *, bool deobfuscate = true);
bool x_ctx_read_word(UINT16 *, bool deobfuscate = true);
bool x_ctx_read_dword(UINT32 *, bool deobfuscate = true);
bool x_ctx_read_qword(UINT64 *, bool deobfuscate = true);
bool x_ctx_read_bool(bool *, bool deobfuscate = true);
