// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_context_reply.cpp 657 2015-02-25 11:42:33Z tzubcic $

#include "x_local.h"

void x_construct_context_reply(unsigned type)
{
	// clear the buffer
	xg.ctx_reply_data.clear();

	// context reply protocol version
	x_ctxr_write_dword(type, false);

	if (type == X_CTX_REPLY_DATA_PROTOCOL) {
		x_construct_context_reply_gen_data();
	} else if (type == X_CTX_REPLY_ERR_PROTOCOL) {

	} else {
		x_code_gen_error(X_ERR_CONSTRUCT_CTX_REPLY_BAD_TYPE);
	}
}

void x_construct_context_reply_gen_data(void)
{
	unsigned	len_offset;

	// placeholder for packet length
	len_offset = xg.ctx_reply_data.size();
	x_ctxr_write_dword(0, false);

	// emit new section
	x_ctxr_write_dword(xg.ctx->gen_data.size());
	x_ctxr_write_buf(&xg.ctx->gen_data[0], xg.ctx->gen_data.size());

	// emit relocations
	x_construct_context_reply_relocations();

	// emit new OEP rva
	x_ctxr_write_dword(x_get_function_rva(xg.ctx->generic_functions[X_FUNC_ENTRY_POINT]));

	// emit addresses of blocks' entry points
	x_construct_context_reply_exec_bytecode();

	// vmi handler table
	x_construct_context_reply_vmi_handlers();

	// update length
	*(uint32_t *) &xg.ctx_reply_data[len_offset] = xg.ctx_reply_data.size();
}

void x_construct_context_reply_relocations(void)
{
	x_ctxr_write_dword(xg.ctx->final_relocs.size());

	for (unsigned i = 0; i < xg.ctx->final_relocs.size(); i++) {
		x_ctxr_write_dword(xg.ctx->final_relocs[i]);
	}
}

void x_construct_context_reply_exec_bytecode(void)
{
	x_vm_block_t		*vmb;

	for (unsigned i = 0; i < xg.ctx->vm_blocks.size(); i++) {
		vmb = &xg.ctx->vm_blocks[i];

		x_ctxr_write_dword(x_get_function_va(vmb->exec_bc_function));
	}
}

void x_construct_context_reply_vmi_handlers(void)
{
	x_vm_t		*vm;

	for (unsigned i = 0; i < xg.ctx->vms.size(); i++) {
		vm = &xg.ctx->vms[i];

		for (unsigned j = 0; j < X_VMI_TOTAL; j++) {
			x_ctxr_write_dword(vm->vmi_handlers[j]);
		}
	}
}

void x_ctxr_write_byte(UINT8 ib, bool obfuscate)
{
	unsigned		offset;
	UINT8			b1;

	offset = xg.ctx_reply_data.size();

	if (obfuscate) {
		b1 = x_ctx_obf_8(ib, offset);
	} else {
		b1 = ib;
	}

	xg.ctx_reply_data.push_back(b1);
}

void x_ctxr_write_word(UINT16 iw, bool obfuscate)
{
	unsigned		offset;
	union {
		UINT16		w;
		struct {
			UINT8	b1;
			UINT8	b2;
		} s;
	} u;

	offset = xg.ctx_reply_data.size();

	if (obfuscate) {
		u.w = x_ctx_obf_16(iw, offset);
	} else {
		u.w = iw;
	}

	xg.ctx_reply_data.push_back(u.s.b1);
	xg.ctx_reply_data.push_back(u.s.b2);
}

void x_ctxr_write_dword(UINT32 id, bool obfuscate)
{
	unsigned		offset;
	union {
		UINT32		d;
		struct {
			UINT8	b1;
			UINT8	b2;
			UINT8	b3;
			UINT8	b4;
		} s;
	} u;

	offset = xg.ctx_reply_data.size();

	if (obfuscate) {
		u.d = x_ctx_obf_32(id, offset);
	} else {
		u.d = id;
	}

	xg.ctx_reply_data.push_back(u.s.b1);
	xg.ctx_reply_data.push_back(u.s.b2);
	xg.ctx_reply_data.push_back(u.s.b3);
	xg.ctx_reply_data.push_back(u.s.b4);
}

void x_ctxr_write_qword(UINT64 iq, bool obfuscate)
{
	union {
		UINT64		q;
		struct {
			UINT32	d1;
			UINT32	d2;
		} s;
	} u;

	u.q = iq;

	x_ctxr_write_dword(u.s.d1, obfuscate);
	x_ctxr_write_dword(u.s.d2, obfuscate);
}

void x_ctxr_write_bool(bool state, bool obfuscate)
{
	uint8_t		val;

	if (!state) {
		val = 0;
	} else {
		val = x_rand_range(1, 0xFF);
	}

	x_ctxr_write_byte(val, obfuscate);
}

void x_ctxr_write_buf(void *buf, unsigned buf_len, bool obfuscate)
{
	uint8_t		*u_buf;

	u_buf = (uint8_t *) buf;

	for (unsigned i = 0; i < buf_len; i++) {
		x_ctxr_write_byte(u_buf[i], obfuscate);
	}
}
