// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_ctx_shared.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// local headers
#include "x_types.h"
#include "x_misc.h"

// context data protocol
#define X_CTX_DATA_PROTOCOL			1

// context reply data protocol
#define X_CTX_REPLY_ERR_PROTOCOL			1
#define X_CTX_REPLY_DATA_PROTOCOL			2

// context obfuscation values
extern UINT8 x_ctx_obf8[256];
extern UINT16 x_ctx_obf16[256];
extern UINT32 x_ctx_obf32[256];

// data transmission obfuscation
XINLINE UINT8 x_ctx_obf_8(UINT8 ib, unsigned offset)
{
	UINT8		ret;
	unsigned	xor_index;
	unsigned	ror_amount;

	ret = ib;

	xor_index = offset & 0xFF;
	ror_amount = offset & 7;

	ret ^= x_ctx_obf8[xor_index];
	ret = x_ror_8(ret, ror_amount);

	return ret;
}

XINLINE UINT16 x_ctx_obf_16(UINT16 iw, unsigned offset)
{
	UINT16		ret;
	unsigned	xor_index;
	unsigned	ror_amount;

	ret = iw;

	xor_index = offset & 0xFF;
	ror_amount = offset & 15;

	ret ^= x_ctx_obf16[xor_index];
	ret = x_ror_16(ret, ror_amount);

	return ret;
}

XINLINE UINT32 x_ctx_obf_32(UINT32 id, unsigned offset)
{
	UINT32		ret;
	unsigned	xor_index;
	unsigned	ror_amount;

	ret = id;

	xor_index = offset & 0xFF;
	ror_amount = offset & 31;

	ret ^= x_ctx_obf32[xor_index];
	ret = x_ror_32(ret, ror_amount);

	return ret;
}

// data transmission de-obfuscation
XINLINE UINT8 x_ctx_de_obf_8(UINT8 ib, unsigned offset)
{
	UINT8		ret;
	unsigned	xor_index;
	unsigned	rol_amount;

	ret = ib;

	xor_index = offset & 0xFF;
	rol_amount = offset & 7;

	ret = x_rol_8(ret, rol_amount);
	ret ^= x_ctx_obf8[xor_index];

	return ret;
}

XINLINE UINT16 x_ctx_de_obf_16(UINT16 iw, unsigned offset)
{
	UINT16		ret;
	unsigned	xor_index;
	unsigned	rol_amount;

	ret = iw;

	xor_index = offset & 0xFF;
	rol_amount = offset & 15;

	ret = x_rol_16(ret, rol_amount);
	ret ^= x_ctx_obf16[xor_index];

	return ret;
}

XINLINE UINT32 x_ctx_de_obf_32(UINT32 id, unsigned offset)
{
	UINT32		ret;
	unsigned	xor_index;
	unsigned	rol_amount;

	ret = id;

	xor_index = offset & 0xFF;
	rol_amount = offset & 31;

	ret = x_rol_32(ret, rol_amount);
	ret ^= x_ctx_obf32[xor_index];

	return ret;
}
