// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_disassembly.cpp 657 2015-02-25 11:42:33Z tzubcic $

#include "x_local.h"

void x_add_disasm(x_block_t *b, ud_t *u)
{
	x_disasm_t		dis;
	UINT32			block_offset;

	// instruction id
	dis.id = b->disasm.size();

	// block owner
	dis.block = b;

	// basic information
	dis.len = ud_insn_len(u);
	dis.txt = ud_insn_asm(u);
	dis.type = u->mnemonic;

	// addresses
	dis.va = (UINT32) u->insn_offset;
	dis.rva = dis.va - b->code_va;

	// instruction order
	dis.order = b->disasm.size();

	// instruction reference
	dis.label = x_new_label();

	// copy instruction bytes
	block_offset = dis.va - b->va;

	memcpy(dis.ud.buffer, b->start + block_offset, dis.len);

	// stripped information
	x_strip_ud_info(u, &dis.ud);

	// by default no relocations
	dis.reloc_count = 0;

	// by default not analyzed
	dis.is_analyzed = false;

	b->disasm.push_back(dis);
}

void x_update_operands_owner_disasm(x_disasm_t *d)
{
	for (unsigned i = 0; i < d->op.size(); i++) {
		d->op[i].dis = d;
	}
}

void x_update_links_disasm(struct x_block_s *b)
{
	x_disasm_t		*last, *dis;

	last = NULL;

	for (unsigned i = 0; i < b->disasm.size(); i++) {
		dis = &b->disasm[i];

		dis->next = NULL;

		if (!last) {
			dis->prev = NULL;
		} else {
			dis->prev = last;
			dis->prev->next = dis;
		}

		last = dis;
	}
}

#define ___X_CI_ELSE(name)					else if (dis->type == UD_I ## name) x_check_ ## name(dis)
#define ___X_CI_ELSE_GROUP(name, group)		else if (dis->type == UD_I ## name) x_check_ ## group(dis)

void x_check_instruction(x_disasm_t *dis)
{
	// not using esp by default
	dis->uses_esp = false;

	// check for esp
	for (unsigned i = 0; i < dis->op.size(); i++) {
		if (x_is_esp_in_operand(&dis->op[i])) {
			dis->uses_esp = true;
			break;
		}
	}

	// unsupported
	if (dis->type == UD_Inone) {
		x_exit_thread(X_ERR_CHECK_INS_INONE);
	}

	// push*
	___X_CI_ELSE(push);
	___X_CI_ELSE(pushad);
	___X_CI_ELSE(pushfd);

	// pop*
	___X_CI_ELSE(pop);
	___X_CI_ELSE(popad);
	___X_CI_ELSE(popfd);

	// mov*
	___X_CI_ELSE(mov);
	___X_CI_ELSE(movzx);
	___X_CI_ELSE(movsx);

	// arithmetic
	___X_CI_ELSE(add);
	___X_CI_ELSE(sub);
	___X_CI_ELSE(mul);
	___X_CI_ELSE(imul);
	___X_CI_ELSE(div);
	___X_CI_ELSE(idiv);
	___X_CI_ELSE(inc);
	___X_CI_ELSE(dec);
	___X_CI_ELSE(adc);
	___X_CI_ELSE(neg);
	___X_CI_ELSE(sbb);

	// logical
	___X_CI_ELSE(or);
	___X_CI_ELSE(xor);
	___X_CI_ELSE(and);
	___X_CI_ELSE(not);
	___X_CI_ELSE(shl);
	___X_CI_ELSE(shr);
	___X_CI_ELSE(ror);
	___X_CI_ELSE(rol);
	___X_CI_ELSE(shld);
	___X_CI_ELSE(shrd);

	// comparisons
	___X_CI_ELSE(cmp);
	___X_CI_ELSE(test);

	// jumps
	___X_CI_ELSE(jmp);
	___X_CI_ELSE_GROUP(jo, jcc);
	___X_CI_ELSE_GROUP(jno, jcc);
	___X_CI_ELSE_GROUP(jb, jcc);
	___X_CI_ELSE_GROUP(jae, jcc);
	___X_CI_ELSE_GROUP(jz, jcc);
	___X_CI_ELSE_GROUP(jnz, jcc);
	___X_CI_ELSE_GROUP(jbe, jcc);
	___X_CI_ELSE_GROUP(ja, jcc);
	___X_CI_ELSE_GROUP(js, jcc);
	___X_CI_ELSE_GROUP(jns, jcc);
	___X_CI_ELSE_GROUP(jp, jcc);
	___X_CI_ELSE_GROUP(jnp, jcc);
	___X_CI_ELSE_GROUP(jl, jcc);
	___X_CI_ELSE_GROUP(jge, jcc);
	___X_CI_ELSE_GROUP(jle, jcc);
	___X_CI_ELSE_GROUP(jg, jcc);

	// procedures
	___X_CI_ELSE(call);
	___X_CI_ELSE(ret);

	// other
	___X_CI_ELSE(nop);
	___X_CI_ELSE(lea);
	___X_CI_ELSE(xchg);
	___X_CI_ELSE(stc);
	___X_CI_ELSE(cld);
	___X_CI_ELSE(clc);
	___X_CI_ELSE(std);
	___X_CI_ELSE(cmc);
	___X_CI_ELSE(bt);

	// unsupported
	else {
		x_exit_thread(X_ERR_CHECK_INS_UNSUPPORTED);
	}
}

Label *x_get_random_disasm(void)
{
	Label		*ret;
	unsigned	id;

	if (xg.ctx->generating_functions) {
		ret = x_get_random_function();
	} else {
		id = x_rand_range(0, xg.ctx->block.disasm.size() - 1);
		ret = xg.ctx->block.disasm[id].label;
	}

	return ret;
}
