// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_disassembly.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// common includes
#include "x_common.h"

// local headers
#include "x_operands.h"
#include "x_udis86.h"

// forward declaration
struct x_block_s;

typedef struct x_disasm_s {
	// order id of this instruction
	unsigned				id;

	// block owner
	struct x_block_s		*block;

	// instruction info
	UINT32					type;
	UINT32					rva;
	UINT32					va;
	UINT32					len;
	string					txt;

	// operands
	vector<x_op_t>			op;

	// instruction reference
	Label					*label;
	UINT32					order;
	struct x_disasm_s		*prev;
	struct x_disasm_s		*next;

	// stripped udis86 info
	x_ud_t					ud;

	// was the instruction analyzed
	bool					is_analyzed;

	// eflags
	bool					needs_flags;
	bool					mods_flags;

	// esp
	bool					needs_esp;
	bool					mods_esp;
	bool					uses_esp;

	// eip
	bool					mods_eip;

	// skip virtualization
	bool					uses_unsupported_registers;
	bool					uses_unsupported_prefixes;

	// relocation count
	unsigned				reloc_count;
} x_disasm_t;

// add a new disassembly to the list
void x_add_disasm(struct x_block_s *, ud_t *);

// update operand disams pointer
void x_update_operands_owner_disasm(x_disasm_t *);

// update disasm prev/next links
void x_update_links_disasm(struct x_block_s *);

// check each instruction
void x_check_instruction(x_disasm_t *);

// get reference to random instruction
Label *x_get_random_disasm(void);
