// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_functions.cpp 657 2015-02-25 11:42:33Z tzubcic $

#include "x_local.h"

// assembler
#define A			(*xg.code_gen)

unsigned x_register_function(x_gen_func_t gen, void *data, const char *name)
{
	x_func_info_t		f;
	unsigned			id;

	id = xg.ctx->functions.size();

	f.gen = gen;
	f.data = data;
	f.label = x_new_label();

#if X_DEBUG >= X_DEBUG_BASIC
	f.func_name = name;
#endif

	xg.ctx->functions.push_back(f);

	return id;
}

void x_generate_functions(void)
{
	vector<unsigned>		func_gen_order;
	unsigned				id;

	if (!xg.ctx->functions.size()) {
		x_exit_thread(X_ERR_GEN_FUNCTIONS_NO_FUNCS);
	}

	// generate function generation order
	for (unsigned i = 0; i < xg.ctx->functions.size(); i++) {
		func_gen_order.push_back(i);
	}

	// shuffle generation order
	random_shuffle(func_gen_order.begin(), func_gen_order.end(), x_rand_generator);

	// set state
	xg.ctx->generating_functions = true;
	xg.ctx->hs.inside_hook = false;

	for (unsigned i = 0; i < func_gen_order.size(); i++) {
		// real function id
		id = func_gen_order[i];

		// RVA
		xg.ctx->functions[id].gen_rva = A.getCodeSize();

		// initially rva is corrected
		xg.ctx->functions[id].rva_corrected = true;

		// bind function reference
		x_bind_label(xg.ctx->functions[id].label);

		// sanity check
		x_abort_if_all_regs_not_free();

		// enable junk code
		x_set_junk_state(true);

		// generate function
		xg.ctx->functions[id].gen(xg.ctx->functions[id].data);

		// disable junk code
		x_set_junk_state(false);

		// sanity check
		x_abort_if_all_regs_not_free();

		// functions should not be empty
		if (xg.ctx->functions[id].gen_rva == A.getCodeSize()) {
			x_exit_thread(X_ERR_GEN_FUNCTIONS_EMPTY_FUNC);
		}
	}

	// set state
	xg.ctx->generating_functions = false;

	// make sure there's enough space for code
	xg.ctx->gen_data.resize(A.getCodeSize());

	// emit code
	A.relocCode(&xg.ctx->gen_data[0], xg.ctx->new_code_va);

	// add block
	x_set_block(&xg.ctx->gen_data[0], &xg.ctx->gen_data[0] + A.getCodeSize());
}

Label *x_get_function(unsigned id)
{
	Label		*l;

	if (id >= xg.ctx->functions.size()) {
		x_exit_thread(X_ERR_GET_FUNC_BAD_ID);
	}

	l = xg.ctx->functions[id].label;

	return l;
}

Label *x_get_random_function(void)
{
	Label		*ret;
	unsigned	id;

	id = x_rand_range(0, xg.ctx->functions.size() - 1);

	ret = x_get_function(id);

	return ret;
}

UINT32 x_get_function_va(unsigned id)
{
	UINT32		ret;

	if (id >= xg.ctx->functions.size()) {
		x_exit_thread(X_ERR_GET_FUNC_VA_BAD_ID);
	}

	ret = xg.ctx->new_code_va + xg.ctx->functions[id].gen_rva;

	return ret;
}

UINT32 x_get_function_rva(unsigned id)
{
	UINT32		ret;

	if (id >= xg.ctx->functions.size()) {
		x_exit_thread(X_ERR_GET_FUNC_RVA_BAD_ID);
	}

	ret = xg.ctx->new_code_rva + xg.ctx->functions[id].gen_rva;

	return ret;
}

void x_set_function_addr(x_disasm_t *dis)
{
	for (unsigned i = 0; i < xg.ctx->functions.size(); i++) {
		if (!xg.ctx->functions[i].rva_corrected) {
			if (x_get_function_va(i) == dis->va) {
				xg.ctx->functions[i].gen_rva = A.getCodeSize();
				xg.ctx->functions[i].rva_corrected = true;
				break;
			}
		}
	}
}

void x_invalidate_function_addresses(void)
{
	for (unsigned i = 0; i < xg.ctx->functions.size(); i++) {
		xg.ctx->functions[i].rva_corrected = false;
	}
}

void x_verify_function_addresses(void)
{
	for (unsigned i = 0; i < xg.ctx->functions.size(); i++) {
		if (!xg.ctx->functions[i].rva_corrected) {
			x_exit_thread(X_ERR_VER_FUNC_RVA_NOT_CORRECTED);
		}
	}
}
