// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_functions.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// common includes
#include "x_common.h"

// local headers
#include "x_asmjit.h"
#include "x_disassembly.h"

// function generator
typedef void (*x_gen_func_t)(void *);

typedef struct {
	// generator
	x_gen_func_t		gen;

	// additional data used for generation
	void				*data;

	// reference for other functions
	Label				*label;

	// addresses
	UINT32				gen_rva;

	// rva corrected
	bool				rva_corrected;

#if X_DEBUG > X_DEBUG_NONE
	const char			*func_name;
#endif
} x_func_info_t;

// register a function
unsigned x_register_function(x_gen_func_t, void *, const char *);

// generate functions
void x_generate_functions(void);

// get a function reference
Label *x_get_function(unsigned);

// get a random function reference
Label *x_get_random_function(void);

// get function's va
UINT32 x_get_function_va(unsigned);

// get function's rva
UINT32 x_get_function_rva(unsigned);

// set function's addresses
void x_set_function_addr(x_disasm_t *);

// invalidate function rvas
void x_invalidate_function_addresses(void);

// abort if function addresses have not been corrected
void x_verify_function_addresses(void);

#if X_DEBUG == X_DEBUG_NONE
 #define x_register_func(gen, data)		x_register_function(gen, data, NULL)
#else
 #define x_register_func(gen, data)		x_register_function(gen, data, #gen)
#endif
