// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_junk.cpp 657 2015-02-25 11:42:33Z tzubcic $

#include "x_local.h"

// assembler
#define A			(*xg.code_gen)

#define DISABLE_JUNK(x)		do { \
								x_set_junk_state(false); \
								(x); \
								x_set_junk_state(true); \
							} while(0);

void x_register_junk_instructions(void)
{
	// clear vector
	xg.ctx->junk_instructions.funcs.clear();

	// add all random instructions
	x_add_junk_instruction(x_random_ins_1);
	
	// shuffle function order
	random_shuffle(xg.ctx->junk_instructions.funcs.begin(), xg.ctx->junk_instructions.funcs.end(), x_rand_generator);

	// initialize count
	xg.ctx->junk_instructions.emit_count = x_rand_range(0, xg.ctx->junk_instructions.funcs.size() - 1);
}

void x_add_junk_instruction(x_junk_function_t func)
{
	xg.ctx->junk_instructions.funcs.push_back(func);
}

void x_emit_junk_instruction(void)
{
	unsigned		num;
	bool			emited;

	if (xg.ctx->junk_instructions.funcs.size()) {
		// shuffle function order
		random_shuffle(xg.ctx->junk_instructions.funcs.begin(), xg.ctx->junk_instructions.funcs.end(), x_rand_generator);

		num = x_rand_range(1,1);

		for (unsigned i = 0; i < num; i++) {
			emited = false;

			while (!emited) {
				emited = xg.ctx->junk_instructions.funcs[xg.ctx->junk_instructions.emit_count % xg.ctx->junk_instructions.funcs.size()]();
				xg.ctx->junk_instructions.emit_count++;
			}
		}
	}
}

bool x_random_ins_1(void)
{
	A.nop();

	return true;
}

void x_register_junk_operations(void)
{
	// clear vector
	xg.ctx->junk_operations.funcs.clear();

	// random comparisons
	x_add_junk_operation(x_random_op_1);

	// shuffle function order
	random_shuffle(xg.ctx->junk_operations.funcs.begin(), xg.ctx->junk_operations.funcs.end(), x_rand_generator);

	// initialize count
	xg.ctx->junk_operations.emit_count = x_rand_range(0, xg.ctx->junk_operations.funcs.size() - 1);
}

void x_add_junk_operation(x_junk_function_t func)
{
	xg.ctx->junk_operations.funcs.push_back(func);
}

void x_emit_junk_operation(void)
{
	bool			emited;

	if (xg.ctx->junk_operations.funcs.size()) {
		// shuffle function order
		random_shuffle(xg.ctx->junk_operations.funcs.begin(), xg.ctx->junk_operations.funcs.end(), x_rand_generator);

		emited = false;

		while (!emited) {
			emited = xg.ctx->junk_operations.funcs[xg.ctx->junk_operations.emit_count % xg.ctx->junk_operations.funcs.size()]();
			xg.ctx->junk_operations.emit_count++;
		}
	}
}

bool x_random_op_1(void)
{
	A.nop();

	return true;
}
