// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_misc.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// common includes
#include "x_common.h"

// maximum random value
#define X_RAND_MAX		0xFFFFFFFF

// helper macros
#define MakePtr(cast, ptr, addValue)	((cast) ((uintptr_t) (ptr) + (uintptr_t) (addValue)))
#define MakeDelta(cast, x, y)			((cast) ((uintptr_t) (x) - (uintptr_t)(y)))

// fill memory with random data
void x_random_fill(void *, unsigned);

// init number generator
void x_seed_rand(void);

// random number generator next value
extern uint32_t x_rand_next;

// random number generator for random_shuffle
unsigned x_rand_generator(unsigned);

// crc32 checksum
uint32_t x_crc32(uint32_t, const void *, size_t, unsigned);

// terminate current thread with exit code
void x_exit_thread(unsigned);

// ror/rol implementation
XINLINE UINT8 x_rol_8(UINT8 ib, unsigned bits)
{
	UINT8		ret;

	bits &= 7;

	ret = ((ib << bits) | (ib >> (8 - bits)));

	return ret;
}

XINLINE UINT16 x_rol_16(UINT16 iw, unsigned bits)
{
	UINT16		ret;

	bits &= 15;

	ret = ((iw << bits) | (iw >> (16 - bits)));

	return ret;
}

XINLINE UINT32 x_rol_32(UINT32 id, unsigned bits)
{
	UINT32		ret;

	bits &= 31;

	ret = ((id << bits) | (id >> (32 - bits)));

	return ret;
}

XINLINE UINT8 x_ror_8(UINT8 ib, unsigned bits)
{
	UINT8		ret;

	bits &= 7;

	ret =  ((ib >> bits) | (ib << (8 - bits)));

	return ret;
}

XINLINE UINT16 x_ror_16(UINT16 iw, unsigned bits)
{
	UINT16		ret;

	bits &= 15;

	ret =  ((iw >> bits) | (iw << (16 - bits)));

	return ret;
}

XINLINE UINT32 x_ror_32(UINT32 id, unsigned bits)
{
	UINT32		ret;

	bits &= 31;

	ret =  ((id >> bits) | (id << (32 - bits)));

	return ret;
}

// force vector to release memory
template <typename T>
void FreeVectorMemory( T & t )
{
    T tmp;
    t.swap( tmp );
}

// compute a random number
uint32_t x_rand(void);

XINLINE uint32_t x_rand_range(uint32_t mn, uint32_t mx)
{
	double scaled = x_rand() / (X_RAND_MAX + 1.0);
    return (unsigned) ((mx - mn + 1) * scaled + mn);
}

// ensure value range
XINLINE unsigned x_value_range(unsigned val, unsigned mn, unsigned mx)
{
	if (val < mn)
		val = mn;

	if (val > mx)
		val = mx;

	return val;
}
