// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_mutate.cpp 657 2015-02-25 11:42:33Z tzubcic $

#include "x_local.h"

// assembler
#define A							(*xg.code_gen)

// mutator definitions
#define X_DEF_INS_MUTATE(name)	void x_mutate_ ## name(struct x_disasm_s *dis) \
								{ \
									unsigned	no; \
									no = dis->op.size();

#define X_END_INS_MUTATE		}

// unsupported operand error
#define UNSUPPORTED_OPERAND			x_exit_thread(X_ERR_MUT_UNSUPPORTED_OPERAND);
#define ELSE_UNSUPPORTED_OPERAND	else { /*printf(">> %s\n", dis->txt.c_str());*/ UNSUPPORTED_OPERAND }

// operands
#define OP(x)						dis->op[x]
#define OP0							OP(0)
#define OP1							OP(1)
#define OP2							OP(2)

// operand types
#define TYPEOP(x)					OP(x).type
#define TOP0						TYPEOP(0)
#define TOP1						TYPEOP(1)
#define TOP2						TYPEOP(2)

// operand relocation info
#define RLCOP(x)					OP(x).is_reloc

// register operands
#define R_UDIS_TO_ASMJIT(x)			(x_udis_reg_to_asmjit_reg(x))
#define ROP(x)						(*R_UDIS_TO_ASMJIT((OP(x).data.reg)))
#define ROP0						ROP(0)
#define ROP1						ROP(1)
#define ROP2						ROP(2)

// integer values
#define IOP(x)						(OP(x).data.val)
#define IOP0						IOP(0)
#define IOP1						IOP(1)
#define IOP2						IOP(2)

// memory operands
#define MOP(x)						x_mem_op_to_asmjit(&OP(x))
#define MOP0						MOP(0)
#define MOP1						MOP(1)
#define MOP2						MOP(2)

// using esp
#define UESP						(dis->uses_esp)

// track relocation management
static bool reloc_added = false;

XINLINE void x_check_mut_reloc_start(x_op_t *op)
{
	if (xg.ctx->has_relocs) {
		if (reloc_added) {
			x_exit_thread(X_ERR_MUT_REL_START_HANDLING_ERROR);
		}

		if (op->is_reloc) {
			if (op->type == UD_OP_IMM || op->type == UD_OP_JIMM) {
				x_add_reloc_value(op->data.val);
			} else if (op->type == UD_OP_MEM) {
				x_add_reloc_value(op->data.mem.offset);
			} else {
				x_exit_thread(X_ERR_MUT_REL_START_HANDLING_ERROR);
			}

			reloc_added = true;

			op->reloc_handled = true;
		}
	}
}

XINLINE void x_check_mut_reloc_end(void)
{
	if (xg.ctx->has_relocs) {
		if (reloc_added) {
			x_end_gen_reloc_value();

			reloc_added = false;
		}
	}
}

// relocation helper macros
#define MR_START(x)					x_check_mut_reloc_start(&OP(x))
#define MR_END						x_check_mut_reloc_end();

// ----------------------------------
// push*
// ----------------------------------
X_DEF_INS_MUTATE(push)
	if (no == 1) {
		if (TOP0 == UD_OP_IMM) {
			MR_START(0);
			A.push(OP(0).data.val);
			MR_END
		} else if (TOP0 == UD_OP_MEM) {
			A.push(MOP0);
		} else if (TOP0 == UD_OP_REG) {
			A.push(ROP0);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(pushad)
	if (no == 0) {
		A.pusha();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(pushfd)
	if (no == 0) {
		A.pushf();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

// ----------------------------------
// pop*
// ----------------------------------
X_DEF_INS_MUTATE(pop)
	if (no == 1) {
		if (TOP0 == UD_OP_REG) {
			A.pop(ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			A.pop(MOP0);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(popad)
	if (no == 0) {
		A.popa();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(popfd)
	if (no == 0) {
		A.popf();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

// ----------------------------------
// mov*
// ----------------------------------
X_DEF_INS_MUTATE(mov)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_IMM) {
				MR_START(1);
				A.mov(ROP0, IOP1);
				MR_END
			} else if (TOP1 == UD_OP_MEM) {
				A.mov(ROP0, MOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.mov(ROP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				MR_START(1);
				A.mov(MOP0, IOP1);
				MR_END
			} else if (TOP1 == UD_OP_REG) {
				A.mov(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(movzx)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_MEM) {
				A.movzx(ROP0, MOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.movzx(ROP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(movsx)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_MEM) {
				A.movsx(ROP0, MOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

// ----------------------------------
// arithmetic
// ----------------------------------
X_DEF_INS_MUTATE(add)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				A.add(MOP0, IOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.add(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.add(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				MR_START(1);
				A.add(ROP0, IOP1);
				MR_END
			} else if (TOP1 == UD_OP_MEM) {
				A.add(ROP0, MOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(sub)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				A.sub(MOP0, IOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.sub(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.sub(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.sub(ROP0, IOP1);
			} else if (TOP1 == UD_OP_MEM) {
				A.sub(ROP0, MOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(mul)
	if (no == 1) {
		if (TOP0 == UD_OP_REG) {
			A.mul(ROP0);
		} ELSE_UNSUPPORTED_OPERAND
	} else if (no == 2) {
		UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(imul)
	if (no == 3) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				if (TOP2 == UD_OP_IMM) {
					A.imul(ROP0, ROP1, IOP2);
				} ELSE_UNSUPPORTED_OPERAND
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(div)
	if (no == 2) {
		UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(idiv)
	if (no == 2) {
		UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(inc)
	if (no == 1) {
		if (TOP0 == UD_OP_REG) {
			A.inc(ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			A.inc(MOP0);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(dec)
	if (no == 1) {
		if (TOP0 == UD_OP_REG) {
			A.dec(ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			A.dec(MOP0);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(adc)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.adc(ROP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(neg)
	if (no == 1) {
		if (TOP0 == UD_OP_REG) {
			A.neg(ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			A.neg(MOP0);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(sbb)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.sbb(ROP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

// ----------------------------------
// logical
// ----------------------------------
X_DEF_INS_MUTATE(or)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				A.or_(MOP0, IOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.or_(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.or_(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.or_(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(xor)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				A.xor_(MOP0, IOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.xor_(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.xor_(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.xor_(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(and)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				A.and_(MOP0, IOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.and_(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.and_(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.and_(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(not)
	if (no == 1) {
		if (TOP0 == UD_OP_REG) {
			A.not_(ROP0);
		} else if (TOP0 == UD_OP_MEM) {
			A.not_(MOP0);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(shl)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				A.shl(MOP0, IOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.shl(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.shl(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.shl(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(shr)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				A.shr(MOP0, IOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.shr(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.shr(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.shr(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(ror)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				A.ror(MOP0, IOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.ror(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.ror(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.ror(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(rol)
	if (no == 2) {
		if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_IMM) {
				A.rol(MOP0, IOP1);
			} else if (TOP1 == UD_OP_REG) {
				A.rol(MOP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.rol(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.rol(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(shld)
	if (no == 3) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				if (TOP2 == UD_OP_IMM) {
					A.shld(ROP0, ROP1, IOP2);
				} ELSE_UNSUPPORTED_OPERAND
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(shrd)
	if (no == 3) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				if (TOP2 == UD_OP_IMM) {
					A.shrd(ROP0, ROP1, IOP2);
				} ELSE_UNSUPPORTED_OPERAND
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

// ----------------------------------
// comparisons
// ----------------------------------
X_DEF_INS_MUTATE(cmp)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.cmp(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.cmp(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_REG) {
				A.cmp(MOP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.cmp(MOP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(test)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.test(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.test(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} else if (TOP0 == UD_OP_MEM) {
			if (TOP1 == UD_OP_REG) {
				A.test(MOP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.test(MOP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

// ----------------------------------
// jumps
// ----------------------------------
X_DEF_INS_MUTATE(jmp)
	if (no == 1) {
		if (TOP0 == UD_OP_JIMM) {
			Label		*tmp;

			if (tmp = x_addr_to_label(IOP0)) {
				A.jmp(*tmp);
			} else {
				A.jmp(IOP0);
			}
		} else if (TOP0 == UD_OP_MEM) {
			A.push(MOP0);
			A.ret();
		} else if (TOP0 == UD_OP_REG) {
			A.jmp(ROP0);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(jo)
	if (no == 1) {
		if (TOP0 == UD_OP_JIMM) {
			Label	*tmp;

			// real jump
			if (tmp = x_addr_to_label(IOP0)) {
				A.jo(*tmp);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

#define X_DEFINE_MUT_JCC(x)		X_DEF_INS_MUTATE(x) \
									if (no == 1) { \
										if (TOP0 == UD_OP_JIMM) { \
											Label	*tmp; \
											 \
											if (tmp = x_addr_to_label(IOP0)) { \
												A.x(*tmp); \
											} ELSE_UNSUPPORTED_OPERAND \
										} ELSE_UNSUPPORTED_OPERAND \
									} ELSE_UNSUPPORTED_OPERAND \
								X_END_INS_MUTATE

X_DEFINE_MUT_JCC(jno)
X_DEFINE_MUT_JCC(jb)
X_DEFINE_MUT_JCC(jae)
X_DEFINE_MUT_JCC(jz)
X_DEFINE_MUT_JCC(jnz)
X_DEFINE_MUT_JCC(jbe)
X_DEFINE_MUT_JCC(ja)
X_DEFINE_MUT_JCC(js)
X_DEFINE_MUT_JCC(jns)
X_DEFINE_MUT_JCC(jp)
X_DEFINE_MUT_JCC(jnp)
X_DEFINE_MUT_JCC(jl)
X_DEFINE_MUT_JCC(jge)
X_DEFINE_MUT_JCC(jle)
X_DEFINE_MUT_JCC(jg)

// ----------------------------------
// procedures
// ----------------------------------
X_DEF_INS_MUTATE(call)
	if (no == 1) {
		if (TOP0 == UD_OP_JIMM) {
			Label		*tmp;

			if (tmp = x_addr_to_label(IOP0)) {
				A.call(*tmp);
			} else {
				A.call(IOP0);
			}
		} else if (TOP0 == UD_OP_REG) {
			A.call(ROP0);
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(ret)
	if (no == 0) {
		A.ret();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

// ----------------------------------
// other
// ----------------------------------
X_DEF_INS_MUTATE(nop)
	if (no == 0) {
		A.nop();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(lea)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_MEM) {
				A.lea(ROP0, MOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(xchg)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.xchg(ROP0, ROP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(stc)
	if (no == 0) {
		A.stc();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(cld)
	if (no == 0) {
		A.cld();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(clc)
	if (no == 0) {
		A.clc();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(std)
	if (no == 0) {
		A.std();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(cmc)
	if (no == 0) {
		A.cmc();
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE

X_DEF_INS_MUTATE(bt)
	if (no == 2) {
		if (TOP0 == UD_OP_REG) {
			if (TOP1 == UD_OP_REG) {
				A.bt(ROP0, ROP1);
			} else if (TOP1 == UD_OP_IMM) {
				A.bt(ROP0, IOP1);
			} ELSE_UNSUPPORTED_OPERAND
		} ELSE_UNSUPPORTED_OPERAND
	} ELSE_UNSUPPORTED_OPERAND
X_END_INS_MUTATE
