// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_registers.cpp 657 2015-02-25 11:42:33Z tzubcic $

#include "x_local.h"

// assembler
#define A			(*xg.code_gen)

x_x86_reg_t x_all_registers[] = {
	// base reg		low 16		low 8		hi 8	default state
	{ &eax,			&ax,		&al,		&ah,	false },
	{ &ebx,			&bx,		&bl,		&bh,	false },
	{ &edx,			&dx,		&dl,		&dh,	false },
	{ &esi,			&si,		NULL,		NULL,	false },
	{ &edi,			&di,		NULL,		NULL,	false },
	{ &ebp,			&bp,		NULL,		NULL,	false },
};

const unsigned x_num_all_registers = sizeof(x_all_registers) / sizeof(x_all_registers[0]);

x_x86_reg_t *x_get_random_register(bool must_be_free, bool must_have_low8, X86Assembler *ASM, const X86GpReg *excl1, const X86GpReg *excl2, const X86GpReg *excl3, const X86GpReg *excl4)
{
	unsigned		num_available;
	x_x86_reg_t		*regs[x_num_all_registers];
	x_x86_reg_t		*ret;

	num_available = 0;

	for (unsigned i = 0; i < x_num_all_registers; i++) {
		if (!x_all_registers[i].needed_count) {
			if (x_all_registers[i].reg == &ecx) {
				continue;
			}

			if (x_all_registers[i].reg == excl1) {
				continue;
			}

			if (x_all_registers[i].reg == excl2) {
				continue;
			}

			if (x_all_registers[i].reg == excl3) {
				continue;
			}

			if (x_all_registers[i].reg == excl4) {
				continue;
			}

			if (must_have_low8) {
				if (!x_all_registers[i].lo_8) {
					continue;
				}
			}

			regs[num_available] = &x_all_registers[i];
			num_available++;
		}
	}

	if (!num_available) {
		if (must_be_free) {
			x_exit_thread(X_ERR_GET_RANDOM_REG_NOT_AVAILABLE);
		}

		for (unsigned i = 0; i < x_num_all_registers; i++) {
			if (x_all_registers[i].reg == &ecx) {
				continue;
			}

			if (x_all_registers[i].reg == excl1) {
				continue;
			}

			if (x_all_registers[i].reg == excl2) {
				continue;
			}

			if (x_all_registers[i].reg == excl3) {
				continue;
			}

			if (x_all_registers[i].reg == excl4) {
				continue;
			}

			if (must_have_low8) {
				if (!x_all_registers[i].lo_8) {
					continue;
				}
			}

			regs[num_available] = &x_all_registers[i];
			num_available++;
		}
	}

	random_shuffle(&regs[0], &regs[num_available], x_rand_generator);

	ret = regs[x_rand_range(0, num_available - 1)];

	if (ret->needed_count) {
		ASM->push(*(ret->reg));
	}

	ret->needed_count++;

	return ret;
}

unsigned x_get_num_free_registers(void)
{
	unsigned		num_available;

	num_available = 0;

	for (unsigned i = 0; i < x_num_all_registers; i++) {
		if (!x_all_registers[i].needed_count) {
			num_available++;
		}
	}

	return num_available;
}

x_x86_reg_t *x_alloc_register(const X86GpReg *excl1, const X86GpReg *excl2, const X86GpReg *excl3, const X86GpReg *excl4)
{
	unsigned		num_available;
	x_x86_reg_t		*regs[x_num_all_registers];
	x_x86_reg_t		*ret;

	num_available = 0;

	for (unsigned i = 0; i < x_num_all_registers; i++) {
		if (x_all_registers[i].reg == &ecx) {
			continue;
		}

		if (x_all_registers[i].reg == excl1) {
			continue;
		}

		if (x_all_registers[i].reg == excl2) {
			continue;
		}

		if (x_all_registers[i].reg == excl3) {
			continue;
		}

		if (x_all_registers[i].reg == excl4) {
			continue;
		}

		regs[num_available] = &x_all_registers[i];
		num_available++;
	}

	if (!num_available) {
		x_exit_thread(X_ERR_ALLOC_REGISTER_NOT_AVAILABLE);
	}

	random_shuffle(&regs[0], &regs[num_available], x_rand_generator);

	ret = regs[x_rand_range(0, num_available - 1)];

	return ret;
}

void x_need_register(x_x86_reg_t *reg)
{
	if (reg->needed_count) {
		x_exit_thread(X_ERR_NEED_REGISTER_ALREADY_NEEDED);
	}

	reg->needed_count++;
}

void x_release_register(x_x86_reg_t *reg, X86Assembler *ASM)
{
	if (!reg->needed_count) {
		x_exit_thread(X_ERR_RELEASE_REGISTER_NOT_NEEDED);
	}

	if (reg->needed_count > 1) {
		ASM->pop(*(reg->reg));
	}

	reg->needed_count--;
}

bool x_is_register_needed(const X86GpReg *reg)
{
	for (unsigned i = 0; i < x_num_all_registers; i++) {
		if (x_all_registers[i].reg == reg) {
			return (x_all_registers[i].needed_count != 0);
		}

		if (i == x_num_all_registers - 1) {
			x_exit_thread(X_ERR_IS_REGISTER_NEEDED_BAD_REG);
		}
	}

	// will not get here
	return false;
}

x_x86_reg_t *x_get_random_register_if_free(void)
{
	unsigned		num_available;
	x_x86_reg_t		*regs[x_num_all_registers];
	x_x86_reg_t		*ret;

	num_available = 0;

	for (unsigned i = 0; i < x_num_all_registers; i++) {
		if (x_all_registers[i].reg == &ecx) {
			continue;
		}

		if (x_all_registers[i].needed_count) {
			continue;
		}

		regs[num_available] = &x_all_registers[i];
		num_available++;
	}

	if (!num_available) {
		ret = NULL;
	} else {
		random_shuffle(&regs[0], &regs[num_available], x_rand_generator);
		ret = regs[x_rand_range(0, num_available - 1)];
	}

	return ret;
}

x_x86_reg_t *x_get_any_register(bool must_have_low8)
{
	unsigned		num_available;
	x_x86_reg_t		*regs[x_num_all_registers];
	x_x86_reg_t		*ret;

	num_available = 0;

	for (unsigned i = 0; i < x_num_all_registers; i++) {
		if (x_all_registers[i].reg == &ecx) {
			continue;
		}

		if (must_have_low8) {
			if (!x_all_registers[i].lo_8) {
				continue;
			}
		}

		regs[num_available] = &x_all_registers[i];
		num_available++;
	}

	if (!num_available) {
		ret = NULL;
	} else {
		random_shuffle(&regs[0], &regs[num_available], x_rand_generator);
		ret = regs[x_rand_range(0, num_available - 1)];
	}

	return ret;
}

void x_abort_if_all_regs_not_free(void)
{
	for (unsigned i = 0; i < x_num_all_registers; i++) {
		if (x_all_registers[i].needed_count) {
			x_exit_thread(X_ERR_ABORT_IF_ALL_REGS_NOT_FREE);
		}
	}
}

void x_add_register_state(void)
{
	UINT32		state;
	UINT32		va;

	// instruction va
	va = xg.ctx->new_code_va + A.getCodeSize();

	// initial state
	state = 0;

	for (unsigned i = 0; i < x_num_all_registers; i++) {
		if (x_all_registers[i].needed_count) {
			state |= (1 << i);
		}
	}

	xg.ctx->rs[va] = state;
}

void x_restore_register_state(UINT32 va)
{
	UINT32		state;

	if (!xg.ctx->rs.count(va)) {
		x_exit_thread(X_ERR_RESTORE_REGISTER_STATE_INVALID_VA);
	}

	state = xg.ctx->rs[va];

	for (unsigned i = 0; i < x_num_all_registers; i++) {
		if (state & (1 << i)) {
			x_all_registers[i].needed_count = 1;
		}
	}
}

void x_clear_register_state(void)
{
	for (unsigned i = 0; i < x_num_all_registers; i++) {
		x_all_registers[i].needed_count = 0;
	}
}
