// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_registers.h 652 2015-02-24 21:15:39Z tzubcic $

#pragma once

// common includes
#include "x_common.h"

// local headers
#include "x_asmjit.h"

typedef struct {
	// base register
	const X86GpReg		*reg;

	// sub-registers
	const X86GpReg		*lo_16;
	const X86GpReg		*lo_8;
	const X86GpReg		*hi_8;

	// state
	unsigned			needed_count;
} x_x86_reg_t;

// get a random register
x_x86_reg_t *x_get_random_register(bool, bool, X86Assembler *, const X86GpReg *excl1 = NULL, const X86GpReg *excl2 = NULL, const X86GpReg *excl3 = NULL, const X86GpReg *excl4 = NULL);

// get number of free registers
unsigned x_get_num_free_registers(void);

// get a random register without marking it as used
x_x86_reg_t *x_alloc_register(const X86GpReg *excl1 = NULL, const X86GpReg *excl2 = NULL, const X86GpReg *excl3 = NULL, const X86GpReg *excl4 = NULL);

// mark a register as needed
void x_need_register(x_x86_reg_t *);

// mark a register as not needed
void x_release_register(x_x86_reg_t *, X86Assembler *);

// get a random register if it's free
x_x86_reg_t *x_get_random_register_if_free(void);

// get any register regardless its state
x_x86_reg_t *x_get_any_register(bool must_have_low8 = false);

// check if a register is needed
bool x_is_register_needed(const X86GpReg *);

// sanity check
void x_abort_if_all_regs_not_free(void);

// keep track of register state
void x_add_register_state(void);
void x_restore_register_state(UINT32);
void x_clear_register_state(void);
