// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_vm.cpp 659 2015-02-25 12:05:31Z tzubcic $

#include "x_local.h"

void x_init_virtual_machines(void)
{
	for (unsigned i = 0; i < xg.ctx->vms.size(); i++) {
		x_init_vm(&xg.ctx->vms[i]);
	}
}

void x_init_vm(x_vm_t *vm)
{
	// allocate registers
	vm->reg_stack = x_alloc_register(&eax, &ebx, &edx);
	vm->reg_ins = x_alloc_register(&eax, &ebx, &edx, vm->reg_stack->reg);

	// initialize stack
	x_init_vm_stack(vm);

	// convert random instruction id to internal id
	for (unsigned i = 0; i < X_VMI_TOTAL; i++) {
		vm->ins_id_real[vm->ins_id[i]] = i;
	}

	// invalid function ids by default
	for (unsigned i = 0; i < X_VM_FUNC_TOTAL; i++) {
		vm->functions[i] = ~0;
	}
}

void x_register_vm_functions(void)
{
	x_vm_t			*vm;

	for (unsigned i = 0; i < xg.ctx->vms.size(); i++) {
		vm = &xg.ctx->vms[i];

		// enter vm
		vm->functions[X_VM_FUNC_ENTER] = x_register_func(x_gen_function_vm_enter, vm);

		// leave vm
		vm->functions[X_VM_FUNC_LEAVE] = x_register_func(x_gen_function_vm_leave, vm);

		// leave vm for calling procedure
		vm->functions[X_VM_FUNC_LEAVE_CALL_RET] = x_register_func(x_gen_function_vm_leave_call_ret, vm);

		// leave vm for executing original native code
		vm->functions[X_VM_FUNC_LEAVE_ORIG_RET] = x_register_func(x_gen_function_vm_leave_orig_ret, vm);

		// store EIP to vm stack
		vm->functions[X_VM_FUNC_STORE_EIP] = x_register_func(x_gen_function_vm_store_eip, vm);

		// get EIP from vm stack
		vm->functions[X_VM_FUNC_GET_EIP] = x_register_func(x_gen_function_vm_get_eip, vm);

		// init tls slot
		vm->functions[X_VM_FUNC_INIT_THREAD_DATA] = x_register_func(x_gen_function_vm_init_thread_data, vm);

		// get thread data
		vm->functions[X_VM_FUNC_GET_THREAD_DATA] = x_register_func(x_gen_function_vm_get_thread_data, vm);

		// instruction handlers
		vm->functions[X_VM_FUNC_I_MOV] = x_register_func(x_gen_function_vm_ins_mov, vm);
		vm->functions[X_VM_FUNC_I_LEA] = x_register_func(x_gen_function_vm_ins_lea, vm);
		vm->functions[X_VM_FUNC_I_ASGN] = x_register_func(x_gen_function_vm_ins_asgn, vm);
		vm->functions[X_VM_FUNC_I_RD] = x_register_func(x_gen_function_vm_ins_rd, vm);
		vm->functions[X_VM_FUNC_I_WR] = x_register_func(x_gen_function_vm_ins_wr, vm);
		vm->functions[X_VM_FUNC_I_ADD] = x_register_func(x_gen_function_vm_ins_add, vm);
		vm->functions[X_VM_FUNC_I_SUB] = x_register_func(x_gen_function_vm_ins_sub, vm);
		vm->functions[X_VM_FUNC_I_SHR] = x_register_func(x_gen_function_vm_ins_shr, vm);
		vm->functions[X_VM_FUNC_I_SHL] = x_register_func(x_gen_function_vm_ins_shl, vm);
		vm->functions[X_VM_FUNC_I_AND] = x_register_func(x_gen_function_vm_ins_and, vm);
		vm->functions[X_VM_FUNC_I_XOR] = x_register_func(x_gen_function_vm_ins_xor, vm);
		vm->functions[X_VM_FUNC_I_OR] = x_register_func(x_gen_function_vm_ins_or, vm);
		vm->functions[X_VM_FUNC_I_ROL] = x_register_func(x_gen_function_vm_ins_rol, vm);
		vm->functions[X_VM_FUNC_I_ROR] = x_register_func(x_gen_function_vm_ins_ror, vm);
		vm->functions[X_VM_FUNC_I_NOT] = x_register_func(x_gen_function_vm_ins_not, vm);
		vm->functions[X_VM_FUNC_I_JMP] = x_register_func(x_gen_function_vm_ins_jmp, vm);
		vm->functions[X_VM_FUNC_I_LEAVE] = x_register_func(x_gen_function_vm_ins_leave, vm);
		vm->functions[X_VM_FUNC_I_CALL] = x_register_func(x_gen_function_vm_ins_call, vm);
		vm->functions[X_VM_FUNC_I_EXEC_BUF] = x_register_func(x_gen_function_vm_ins_exec_buf, vm);
		vm->functions[X_VM_FUNC_I_EXEC] = x_register_func(x_gen_function_vm_ins_exec, vm);
		vm->functions[X_VM_FUNC_I_SETF] = x_register_func(x_gen_function_vm_ins_setf, vm);
		vm->functions[X_VM_FUNC_I_CLRF] = x_register_func(x_gen_function_vm_ins_clrf, vm);
		vm->functions[X_VM_FUNC_I_FIXRLC] = x_register_func(x_gen_function_vm_ins_fixrlc, vm);
		vm->functions[X_VM_FUNC_I_VMTEST] = x_register_func(x_gen_function_vm_ins_vmtest, vm);
		vm->functions[X_VM_FUNC_I_VMCMP] = x_register_func(x_gen_function_vm_ins_vmcmp, vm);
	}
}

void x_register_block_vm_entry_functions(void)
{
	x_vm_block_t	*vmb;

	for (unsigned i = 0; i < xg.ctx->vm_blocks.size(); i++) {
		vmb = &xg.ctx->vm_blocks[i];

		vmb->exec_bc_function = x_register_func(x_gen_function_vm_execute_bytecode, vmb);
	}
}

void x_generate_vm_handler_table(void)
{
	x_vm_t				*vm;
	UINT32				*handler;
	x_vm_ins_info_t		*ii;
	uint16_t			ins_real;
	uint16_t			ins_rnd;

	for (unsigned i = 0; i < xg.ctx->vms.size(); i++) {
		vm = &xg.ctx->vms[i];

		handler = vm->vmi_handlers;

		for (unsigned j = 0; j < X_VMI_TOTAL; j++) {
			ins_real = vm->ins_id_real[vm->ins_id[j]];
			ins_rnd = vm->ins_id[j];
			ii = x_get_vm_ins_info(ins_real);

			handler[ins_rnd] = x_encode_dword(0, x_get_function_rva(vm->functions[ii->func_id]), ins_rnd);
		}
	}
}
