// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_vm.h 660 2015-02-25 12:11:30Z tzubcic $

#pragma once

// common includes
#include "x_common.h"

// shared headers
#include "x_vm_shared.h"

// local headers
#include "x_registers.h"
#include "x_vm_stack.h"

typedef struct x_vm_s {
	// instruction ids
	uint16_t		ins_id[X_VMI_TOTAL];

	// register ids
	uint16_t		reg_id[X_VMR_TOTAL];
	uint16_t		ins_id_real[X_VMI_TOTAL];

	// vm functions
	unsigned		functions[X_VM_FUNC_TOTAL];

	// registers used through multiple vm functions
	x_x86_reg_t		*reg_stack;
	x_x86_reg_t		*reg_ins;

	// vm stack layout
	x_vm_stack_t	stack;

	// longest instruction length
	unsigned		longest_instruction;

	// data addresses
	UINT32			data_rva_vmi_rvas;
	UINT32			data_rva_vmi_lengths;
	UINT32			data_va_tls_slot;

	// handler rvas
	UINT32			vmi_handlers[X_VMI_TOTAL];
} x_vm_t;

typedef struct {
	// vm owner
	UINT32			vm_id;
	x_vm_t			*vm;

	// bytecode address
	UINT32			bc_rva;
	UINT32			bc_va;

	// exec bytecode function
	unsigned		exec_bc_function;
} x_vm_block_t;

// initialize all virtual machines
void x_init_virtual_machines(void);

// initialize a vm
void x_init_vm(x_vm_t *);

// register vm functions
void x_register_vm_functions(void);

// register block vm entry functions
void x_register_block_vm_entry_functions(void);

// generate handler rva table
void x_generate_vm_handler_table(void);
