// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_vm_functions.cpp 660 2015-02-25 12:11:30Z tzubcic $

#include "x_local.h"

// assembler
#define A			(*xg.code_gen)
#define B			(*xg.temp_gen)

static bool old_junk_state;

XINLINE void x_func_reloc_add(UINT32 val)
{
	if (xg.ctx->has_relocs) {
		old_junk_state = xg.ctx->hs.junk_enabled;

		x_set_junk_state(false);
		x_add_reloc_value(val);
	}
}

XINLINE void x_func_reloc_end(void)
{
	if (xg.ctx->has_relocs) {
		x_end_gen_reloc_value();
		x_set_junk_state(old_junk_state);
	}
}

#define VMR(x)		(vm->reg_id[x])
#define SOFF(x)		(x_vm_stack_offset(vm, x))
#define VMROFF(x)	(SOFF(X_VM_STACK_CONTEXT) + (VMR(x) * 4))

#define DISABLE_JUNK(x)		do { \
								x_set_junk_state(false); \
								(x); \
								x_set_junk_state(true); \
							} while(0);

#define FETCH_OP(x, y, z)		x_gen_fetch_op(vm, x, X_VMI_ ## y, z)
#define DECODE_BYTE(x, y, z)	x_vm_decode_op_byte(vm, x, X_VMI_ ## y, z);
#define DECODE_DWORD(x, y, z)	x_vm_decode_op_dword(vm, x, X_VMI_ ## y, z);
#define DECODE_REG(x, y, z)		x_vm_decode_register(vm, x, X_VMI_ ## y, z);

void x_gen_function_vm_enter(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*mn;
	x_x86_reg_t		*handler;
	
	vm = (x_vm_t *) data;

	x_need_register(vm->reg_stack);
	x_need_register(vm->reg_ins);

	// save flags and registers
	x_code_shuffling(false);
	DISABLE_JUNK(A.pusha())
	DISABLE_JUNK(A.pushf())
	x_code_shuffling(true);

	x_emit_junk_operation();

	// get vm thread data
	x_gen_get_vm_thread_data(vm);

	x_emit_junk_operation();

	// backup flags and registers
	x_gen_backup_x86_context(vm);

	x_emit_junk_operation();

	// save load address difference
	x_gen_set_base_diff(vm);

	x_emit_junk_operation();

	// reset flags
	x_gen_reset_vm_flags(vm);

	x_emit_junk_operation();

	// new stack
	x_gen_setup_stack(vm);

	x_emit_junk_operation();

	// fetch the instruction
	Label *fetch_ins = x_new_label();
	x_bind_label(fetch_ins);

	// fetch vm instruction pointer
	x_gen_fetch_eip(vm);

	x_emit_junk_operation();

	// memcpy(dec_buff, ins, longest_instruction_length); reg_ins = dec_buff;
	x_gen_copy_instruction_to_dec_buff(vm);

	x_emit_junk_operation();

	// decrypt(dec_buff, longest_instruction_length);
	x_gen_decrypt_instruction(vm);

	x_emit_junk_operation();

	mn = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// fetch and decode mnemonic
	x_gen_get_mnemonic(vm, mn);

	// skip mnemonic, ins_ptr += mnemonic_length;
	x_emul_add_val(vm->reg_ins, X_VM_MNEMONIC_LEN);

	x_emit_junk_operation();

	// vm.EIP += ins_length;
	x_gen_set_next_eip(vm, mn, fetch_ins);

	x_emit_junk_operation();

	// check condition code
	x_gen_check_ins_condition_code(vm, fetch_ins);

	x_emit_junk_operation();

	handler = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// handler = &handlers[mnemonic];
	x_gen_get_handler(vm, mn, handler);

	x_emit_junk_operation();

	x_release_register(mn, &A);

	// ins->handler();
	A.call(*(handler->reg));

	x_emit_junk_operation();

	x_release_register(handler, &A);
	x_release_register(vm->reg_ins, &A);

	A.jmp(*fetch_ins);

	x_release_register(vm->reg_stack, &A);
}

#define PSH(x)		do { \
						x_emit_junk_operation(); \
						A.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(x))); \
					} while(0)

void x_gen_function_vm_leave(void *data)
{
	x_vm_t		*vm;
	
	vm = (x_vm_t *) data;

	x_need_register(vm->reg_stack);

	x_emit_junk_operation();

	PSH(X_VMR_N_ESP);
	PSH(X_VMR_N_EFLAGS);
	PSH(X_VMR_N_EAX);
	PSH(X_VMR_N_ECX);
	PSH(X_VMR_N_EDX);
	PSH(X_VMR_N_EBX);
	PSH(X_VMR_N_ESP);
	PSH(X_VMR_N_EBP);
	PSH(X_VMR_N_ESI);
	PSH(X_VMR_N_EDI);

	x_emit_junk_operation();

	x_release_register(vm->reg_stack, &A);

	x_emit_junk_operation();

	x_code_shuffling(false);
	A.popa();
	DISABLE_JUNK(A.popf())
	DISABLE_JUNK(A.pop(esp))

	DISABLE_JUNK(A.ret())
	x_code_shuffling(true);
}

void x_gen_function_vm_leave_call_ret(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*ret;
	x_x86_reg_t		*stack;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_stack);

	ret = x_get_random_register(true, false, &A, vm->reg_stack->reg);
	stack = x_get_random_register(true, false, &A, vm->reg_stack->reg, ret->reg);

	A.pop(*(ret->reg));

	x_gen_fetch_mem(stack, 4, vm->reg_stack, NULL, 0, VMROFF(X_VMR_N_ESP));
	x_gen_write_dword(stack, NULL, 0, 4, ret);
	
	x_release_register(stack, &A);
	x_release_register(ret, &A);

	x_emit_junk_operation();
	x_emit_junk_operation();

	PSH(X_VMR_N_ESP);
	PSH(X_VMR_N_EFLAGS);
	PSH(X_VMR_N_EAX);
	PSH(X_VMR_N_ECX);
	PSH(X_VMR_N_EDX);
	PSH(X_VMR_N_EBX);
	PSH(X_VMR_N_ESP);
	PSH(X_VMR_N_EBP);
	PSH(X_VMR_N_ESI);
	PSH(X_VMR_N_EDI);

	x_code_shuffling(false);
	A.popa();
	DISABLE_JUNK(A.popf())
	DISABLE_JUNK(A.pop(esp))

	DISABLE_JUNK(A.ret())
	x_code_shuffling(true);

	// global registers
	x_release_register(vm->reg_stack, &A);
}

#undef PSH

void x_gen_function_vm_leave_orig_ret(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*buf;
	x_x86_reg_t		*ret_addr;
	x_x86_reg_t		*val;
	UINT32			rnd;
	
	vm = (x_vm_t *) data;

	x_need_register(vm->reg_stack);

	val = x_get_random_register(true, true, &A);
	buf = x_get_random_register(true, false, &A);
	ret_addr = x_get_random_register(true, false, &A);

	// get &orig_ins[len] and ret addr
	A.pop(*(ret_addr->reg));
	A.pop(*(buf->reg));

	rnd = x_rand();
	rnd &= 0xFFFFFF00;
	rnd |= 0x68;
	x_emul_mov_imm(val, rnd);

	A.mov(byte_ptr(*(buf->reg)), *(val->lo_8));

	x_emul_add_val(buf, 1);
	A.mov(dword_ptr(*(buf->reg)), *(ret_addr->reg));
	x_emul_add_val(buf, 4);

	rnd = x_rand();
	rnd &= 0xFFFFFF00;
	rnd |= 0xC3;
	x_emul_mov_imm(val, rnd);

	A.mov(byte_ptr(*(buf->reg)), *(val->lo_8));

	x_release_register(ret_addr, &A);
	x_release_register(buf, &A);
	x_release_register(val, &A);

	A.jmp(*x_get_function(vm->functions[X_VM_FUNC_LEAVE]));

	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_store_eip(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*cur_eip;
	x_x86_reg_t		*bkp;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_stack);
	x_need_register(vm->reg_ins);

	cur_eip = x_get_random_register(true, false, &A);

	// get current eip
	x_gen_fetch_mem(cur_eip, 4, vm->reg_stack, NULL, 0, VMROFF(X_VMR_VM_EIP));

	bkp = x_get_random_register(true, false, &A);

	// start of backup table
	x_gen_lea(bkp, vm->reg_stack, NULL, 0, SOFF(X_VM_STACK_EIP_BACKUP));

	Label		*repeat = x_new_label();

	// find first free slot
	A.sub(*(bkp->reg), 4);
	x_bind_label(repeat);
	x_emul_add_val(bkp, 4);
	x_code_shuffling(false);
	A.cmp(dword_ptr(*(bkp->reg)), 0);
	DISABLE_JUNK(A.jne(*repeat))
	x_code_shuffling(true);

	// store current eip
	A.mov(dword_ptr(*(bkp->reg)), *(cur_eip->reg));

	x_release_register(bkp, &A);
	x_release_register(cur_eip, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_get_eip(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*loc_stack;
	x_x86_reg_t		*bkp;
	x_x86_reg_t		*eip;
	
	vm = (x_vm_t *) data;

	x_code_shuffling(false);
	DISABLE_JUNK(A.push(x_rand()))
	DISABLE_JUNK(A.pusha())
	DISABLE_JUNK(A.pushf())
	x_code_shuffling(true);

	loc_stack = x_get_random_register(true, false, &A);

	// get thread data base address
	A.call(*x_get_function(vm->functions[X_VM_FUNC_GET_THREAD_DATA]));
	A.pop(*(loc_stack->reg));

	bkp = x_get_random_register(true, false, &A);

	// start of backup table
	x_gen_lea(bkp, loc_stack, NULL, 0, SOFF(X_VM_STACK_EIP_BACKUP));

	Label		*repeat = x_new_label();

	// find first free slot
	x_bind_label(repeat);
	x_emul_add_val(bkp, 4);
	x_code_shuffling(false);
	A.cmp(dword_ptr(*(bkp->reg)), 0);
	DISABLE_JUNK(A.jne(*repeat))
	x_code_shuffling(true);

	// zero found, revert to previous entry
	A.sub(*(bkp->reg), 4);

	eip = x_get_random_register(true, false, &A);

	// get eip
	A.push(dword_ptr(*(bkp->reg)));
	A.pop(*(eip->reg));

	// set eip at [esp]
	A.mov(dword_ptr(esp, 36), *(eip->reg));

	x_release_register(eip, &A);

	// remove it from backup table
	A.mov(dword_ptr(*(bkp->reg)), 0);

	x_release_register(bkp, &A);
	x_release_register(loc_stack, &A);

	// restore original context
	x_code_shuffling(false);
	A.popf();
	DISABLE_JUNK(A.popa())

	// continue normal execution in vm
	DISABLE_JUNK(A.jmp(*x_get_function(vm->functions[X_VM_FUNC_ENTER])))
	x_code_shuffling(true);
}

void x_gen_function_vm_init_thread_data(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*r1, *r2;
	UINT32			slot_addr;
	Label			*alloc_ok;

	vm = (x_vm_t *) data;

	x_set_junk_state(false);
	x_code_shuffling(false);

	// global registers
	x_need_register(vm->reg_stack);

	r1 = x_get_random_register(true, false, &A, &eax);
	r2 = x_get_random_register(true, false, &A, &eax);

	alloc_ok = x_new_label();

	// r1 = IAT.TlsAlloc
	x_func_reloc_add(xg.ctx->imports[X_IMP_TLSALLOC]);
	A.mov(*(r1->reg), xg.ctx->imports[X_IMP_TLSALLOC]);
	x_func_reloc_end();

	// r1 = TlsAlloc
	A.mov(*(r1->reg), dword_ptr(*(r1->reg)));

	if (x_is_register_needed(&eax)) {
		A.push(eax);
	}

	// r1 = TlsAlloc()
	A.call(*(r1->reg));
	A.mov(*(r1->reg), eax);

	if (x_is_register_needed(&eax)) {
		A.pop(eax);
	}

	A.cmp(*(r1->reg), (UINT32) TLS_OUT_OF_INDEXES);
	A.jne(*alloc_ok);

	A.call(*x_get_function(xg.ctx->generic_functions[X_FUNC_EXIT_PROCCESS]));

	A.bind(*alloc_ok);

	slot_addr = vm->data_va_tls_slot;

	x_func_reloc_add(slot_addr);
	A.mov(*(r2->reg), slot_addr);
	x_func_reloc_end();

	A.mov(dword_ptr(*(r2->reg)), *(r1->reg));

	A.ret();

	x_release_register(r2, &A);
	x_release_register(r1, &A);

	// global registers
	x_release_register(vm->reg_stack, &A);

	x_code_shuffling(true);
	x_set_junk_state(true);
}

void x_gen_function_vm_get_thread_data(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*slot_id, *addr;
	Label			*registered, *allocated, *alloc_failed;
	
	vm = (x_vm_t *) data;

	x_set_junk_state(false);
	x_code_shuffling(false);

	// global registers
	x_need_register(vm->reg_stack);

	slot_id = x_get_random_register(true, false, &A, &eax);
	addr = x_get_random_register(true, false, &A, &eax);

	registered = x_new_label();
	allocated = x_new_label();
	alloc_failed = x_new_label();

	// read slot id
	x_gen_read_dword(vm->data_va_tls_slot, slot_id);

	// check if it's initialized
	A.cmp(*(slot_id->reg), (UINT32) TLS_OUT_OF_INDEXES);
	A.jne(*registered);
	A.call(*x_get_function(vm->functions[X_VM_FUNC_INIT_THREAD_DATA]));
	x_bind_label(registered);

	// read slot id
	x_gen_read_dword(vm->data_va_tls_slot, slot_id);

	// slot_addr = TlsGetValue(slot_id)
	if (x_is_register_needed(&eax)) {
		A.push(eax);
	}

	A.push(*(slot_id->reg));
	x_gen_call_import(X_IMP_TLSGETVALUE);
	A.mov(*(addr->reg), eax);

	if (x_is_register_needed(&eax)) {
		A.pop(eax);
	}

	// check if it's already allocated
	A.test(*(addr->reg), *(addr->reg));
	A.jnz(*allocated);

	// read slot id
	x_gen_read_dword(vm->data_va_tls_slot, slot_id);
	
	if (x_is_register_needed(&eax)) {
		A.push(eax);
	}

	// addr = VirtualAlloc(NULL, X_VM_THREAD_DATA_SIZE, MEM_COMMIT | MEM_RESERVE, PAGE_EXECUTE_READWRITE);
	A.push(PAGE_EXECUTE_READWRITE);
	A.push(MEM_COMMIT | MEM_RESERVE);
	A.push(X_VM_THREAD_DATA_SIZE);
	A.push(0);
	x_gen_call_import(X_IMP_VIRTUALALLOC);
	A.mov(*(addr->reg), eax);

	if (x_is_register_needed(&eax)) {
		A.pop(eax);
	}

	// check if allocation succeeded
	A.test(*(addr->reg), *(addr->reg));
	A.jz(*alloc_failed);

	if (x_is_register_needed(&eax)) {
		A.push(eax);
	}
	
	// read slot id
	x_gen_read_dword(vm->data_va_tls_slot, slot_id);

	if (x_is_register_needed(&eax)) {
		A.push(eax);
	}

	// TlsSetValue(slot_id, addr);
	A.push(*(addr->reg));
	A.push(*(slot_id->reg));
	x_gen_call_import(X_IMP_TLSSETVALUE);

	if (x_is_register_needed(&eax)) {
		A.pop(eax);
	}

	A.bind(*allocated);

	// return addr on stack
	A.push(dword_ptr(esp));
	A.mov(dword_ptr(esp, 4), *(addr->reg));

	A.ret();

	A.bind(*alloc_failed);

	A.call(*x_get_function(xg.ctx->generic_functions[X_FUNC_EXIT_PROCCESS]));

	x_release_register(addr, &A);
	x_release_register(slot_id, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_stack, &A);

	x_code_shuffling(true);
	x_set_junk_state(true);
}

void x_gen_function_vm_execute_bytecode(void *data)
{
	x_vm_block_t	*vmb;
	x_vm_t			*vm;
	
	vmb = (x_vm_block_t *) data;
	vm = vmb->vm;

	// TODO: enable code shuffling, add pushf/pusha + junk code + popa/popf
	x_code_shuffling(false);

	x_func_reloc_add(vmb->bc_va);
	A.push(vmb->bc_va);
	x_func_reloc_end();

	DISABLE_JUNK(A.jmp(*x_get_function(vm->functions[X_VM_FUNC_ENTER])))

	x_code_shuffling(true);
}

void x_gen_function_vm_ins_mov(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, MOV, 1);
	DECODE_REG(dst, MOV, 1);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, MOV, 2);
	DECODE_REG(src, MOV, 2);
	x_gen_dereference_register(vm, src);

	// dst = src
	x_gen_write_dword(dst, NULL, 0, 0, src);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_lea(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, LEA, 1);
	DECODE_REG(dst, LEA, 1);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src address
	FETCH_OP(src, LEA, 2);
	DECODE_REG(src, LEA, 2);
	x_gen_get_register_address(vm, src);

	// dst = &src
	A.mov(dword_ptr(*(dst->reg)), *(src->reg));

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_asgn(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, ASGN, 1);
	DECODE_REG(dst, ASGN, 1);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, ASGN, 2);
	DECODE_DWORD(src, ASGN, 2);

	// dst = imm
	A.mov(dword_ptr(*(dst->reg)), *(src->reg));

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_rd(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, RD, 2);
	DECODE_REG(dst, RD, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src address
	FETCH_OP(src, RD, 3);
	DECODE_REG(src, RD, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, RD, 1);
	DECODE_BYTE(size, RD, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
	Label		*op2 = x_new_label();
	Label		*op4 = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	A.mov(*(src->lo_8), byte_ptr(*(src->reg)));
	A.mov(byte_ptr(*(dst->reg)), *(src->lo_8));
	A.jmp(*end);

	// word
	x_bind_label(op2);
	A.mov(*(src->lo_16), word_ptr(*(src->reg)));
	A.mov(word_ptr(*(dst->reg)), *(src->lo_16));
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	A.mov(*(src->reg), dword_ptr(*(src->reg)));
	A.mov(dword_ptr(*(dst->reg)), *(src->reg));
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_wr(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, WR, 2);
	DECODE_REG(dst, WR, 2);
	x_gen_dereference_register(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, WR, 3);
	DECODE_REG(src, WR, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, WR, 1);
	DECODE_BYTE(size, WR, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
	Label		*op2 = x_new_label();
	Label		*op4 = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	A.mov(byte_ptr(*(dst->reg)), *(src->lo_8));
	A.jmp(*end);

	// word
	x_bind_label(op2);
	A.mov(word_ptr(*(dst->reg)), *(src->lo_16));
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	A.mov(dword_ptr(*(dst->reg)), *(src->reg));
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_add(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, ADD, 2);
	DECODE_REG(dst, ADD, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, ADD, 3);
	DECODE_REG(src, ADD, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, ADD, 1);
	DECODE_BYTE(size, ADD, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.add(byte_ptr(*(dst->reg)), *(src->lo_8));
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.add(byte_ptr(*(dst->reg)), *(src->lo_8));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.add(word_ptr(*(dst->reg)), *(src->lo_16));
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.add(word_ptr(*(dst->reg)), *(src->lo_16));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.add(dword_ptr(*(dst->reg)), *(src->reg));
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.add(dword_ptr(*(dst->reg)), *(src->reg));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_sub(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, SUB, 2);
	DECODE_REG(dst, SUB, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, SUB, 3);
	DECODE_REG(src, SUB, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, SUB, 1);
	DECODE_BYTE(size, SUB, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.sub(byte_ptr(*(dst->reg)), *(src->lo_8));
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.sub(byte_ptr(*(dst->reg)), *(src->lo_8));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.sub(word_ptr(*(dst->reg)), *(src->lo_16));
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.sub(word_ptr(*(dst->reg)), *(src->lo_16));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.sub(dword_ptr(*(dst->reg)), *(src->reg));
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.sub(dword_ptr(*(dst->reg)), *(src->reg));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_shr(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, SHR, 2);
	DECODE_REG(dst, SHR, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, SHR, 3);
	DECODE_REG(src, SHR, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, SHR, 1);
	DECODE_BYTE(size, SHR, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.shr(byte_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.shr(byte_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.shr(word_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.shr(word_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.shr(dword_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.shr(dword_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_or(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, OR, 2);
	DECODE_REG(dst, OR, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, OR, 3);
	DECODE_REG(src, OR, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, OR, 1);
	DECODE_BYTE(size, OR, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.or_(byte_ptr(*(dst->reg)), *(src->lo_8));
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.or_(byte_ptr(*(dst->reg)), *(src->lo_8));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.or_(word_ptr(*(dst->reg)), *(src->lo_16));
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.or_(word_ptr(*(dst->reg)), *(src->lo_16));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.or_(dword_ptr(*(dst->reg)), *(src->reg));
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.or_(dword_ptr(*(dst->reg)), *(src->reg));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_rol(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, ROL, 2);
	DECODE_REG(dst, ROL, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, ROL, 3);
	DECODE_REG(src, ROL, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, ROL, 1);
	DECODE_BYTE(size, ROL, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.rol(byte_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.rol(byte_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.rol(word_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.rol(word_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.rol(dword_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.rol(dword_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_ror(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, ROR, 2);
	DECODE_REG(dst, ROR, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, ROR, 3);
	DECODE_REG(src, ROR, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, ROR, 1);
	DECODE_BYTE(size, ROR, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.ror(byte_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.ror(byte_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.ror(word_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.ror(word_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.ror(dword_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.ror(dword_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_shl(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, SHL, 2);
	DECODE_REG(dst, SHL, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, SHL, 3);
	DECODE_REG(src, SHL, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, SHL, 1);
	DECODE_BYTE(size, SHL, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.shl(byte_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.shl(byte_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.shl(word_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.shl(word_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.mov(cl, *(src->lo_16));
	A.shl(dword_ptr(*(dst->reg)), cl);
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.mov(cl, *(src->lo_16));
	B.shl(dword_ptr(*(dst->reg)), cl);
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_and(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, AND, 2);
	DECODE_REG(dst, AND, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, AND, 3);
	DECODE_REG(src, AND, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, AND, 1);
	DECODE_BYTE(size, AND, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.and_(byte_ptr(*(dst->reg)), *(src->lo_8));
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.and_(byte_ptr(*(dst->reg)), *(src->lo_8));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.and_(word_ptr(*(dst->reg)), *(src->lo_16));
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.and_(word_ptr(*(dst->reg)), *(src->lo_16));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.and_(dword_ptr(*(dst->reg)), *(src->reg));
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.and_(dword_ptr(*(dst->reg)), *(src->reg));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_xor(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*src;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, XOR, 2);
	DECODE_REG(dst, XOR, 2);
	x_gen_get_register_address(vm, dst);

	src = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get src value
	FETCH_OP(src, XOR, 3);
	DECODE_REG(src, XOR, 3);
	x_gen_dereference_register(vm, src);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg, src->reg);

	// get operand size
	FETCH_OP(size, XOR, 1);
	DECODE_BYTE(size, XOR, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.xor_(byte_ptr(*(dst->reg)), *(src->lo_8));
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.xor_(byte_ptr(*(dst->reg)), *(src->lo_8));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.xor_(word_ptr(*(dst->reg)), *(src->lo_16));
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.xor_(word_ptr(*(dst->reg)), *(src->lo_16));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.xor_(dword_ptr(*(dst->reg)), *(src->reg));
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.xor_(dword_ptr(*(dst->reg)), *(src->reg));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(src, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_not(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*size;
	x_x86_reg_t		*dst;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, NOT, 2);
	DECODE_REG(dst, NOT, 2);
	x_gen_get_register_address(vm, dst);

	size = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	// get operand size
	FETCH_OP(size, NOT, 1);
	DECODE_BYTE(size, NOT, 1);

	Label		*end = x_new_label();
	Label		*op1 = x_new_label();
		Label	*op1_save = x_new_label();
	Label		*op2 = x_new_label();
		Label	*op2_save = x_new_label();
	Label		*op4 = x_new_label();
		Label	*op4_save = x_new_label();

	x_code_shuffling(false);
	A.cmp(*(size->reg), 1);
	DISABLE_JUNK(A.je(*op1))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(size->reg), 2);
	DISABLE_JUNK(A.je(*op2))
	x_code_shuffling(true);
	A.jmp(*op4);

	x_release_register(size, &A);

	// byte
	x_bind_label(op1);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op1_save))
	x_code_shuffling(true);
	A.not_(byte_ptr(*(dst->reg)));
	A.jmp(*end);
	x_bind_label(op1_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.not_(byte_ptr(*(dst->reg)));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// word
	x_bind_label(op2);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op2_save))
	x_code_shuffling(true);
	A.not_(word_ptr(*(dst->reg)));
	A.jmp(*end);
	x_bind_label(op2_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.not_(word_ptr(*(dst->reg)));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	// dword
	x_bind_label(op4);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_SAVE_EFLAGS);
	DISABLE_JUNK(A.jnz(*op4_save))
	x_code_shuffling(true);
	A.not_(dword_ptr(*(dst->reg)));
	A.jmp(*end);
	x_bind_label(op4_save);
	B.push(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	B.popf();
	B.not_(dword_ptr(*(dst->reg)));
	B.pushf();
	B.pop(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_N_EFLAGS)));
	x_gen_exec_code_in_buffer(vm);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_jmp(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*rel;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	rel = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get relative address
	FETCH_OP(rel, JMP, 1);
	DECODE_DWORD(rel, JMP, 1);
	
	A.add(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_VM_EIP)), *(rel->reg));

	x_release_register(rel, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_leave(void *data)
{
	x_vm_t			*vm;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_stack);

	x_code_shuffling(false);
	DISABLE_JUNK(A.jmp(*x_get_function(vm->functions[X_VM_FUNC_LEAVE])))
	x_code_shuffling(true);

	// global registers
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_call(void *data)
{
	x_vm_t		*vm;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_stack);

	// add return vm eip to stack
	A.call(*x_get_function(vm->functions[X_VM_FUNC_STORE_EIP]));

	x_code_shuffling(false);

	// continue execution at function
	DISABLE_JUNK(A.call(*x_get_function(vm->functions[X_VM_FUNC_LEAVE_CALL_RET])))

	// restore return vm eip from stack
	DISABLE_JUNK(A.jmp(*x_get_function(vm->functions[X_VM_FUNC_GET_EIP])))

	x_code_shuffling(true);

	// global registers
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_exec_buf(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*tmp;

	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get register address
	FETCH_OP(dst, EXEC_BUF, 1);
	DECODE_REG(dst, EXEC_BUF, 1);
	x_gen_get_register_address(vm, dst);

	tmp = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	x_gen_lea(tmp, vm->reg_stack, NULL, 0, SOFF(X_VM_STACK_ORIG));
	A.mov(dword_ptr(*(dst->reg)), *(tmp->reg));

	x_release_register(tmp, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_exec(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*end;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	// add return vm eip to stack
	A.call(*x_get_function(vm->functions[X_VM_FUNC_STORE_EIP]));

	end = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get buffer address
	FETCH_OP(end, EXEC, 1);
	DECODE_REG(end, EXEC, 1);
	x_gen_dereference_register(vm, end);

	// push end of code - [esp] gets set to &buf[0] by virtualizer
	A.push(*(end->reg));

	x_release_register(end, &A);

	x_code_shuffling(false);

	// execute code
	A.call(*x_get_function(vm->functions[X_VM_FUNC_LEAVE_ORIG_RET]));

	// restore vm eip from stack
	DISABLE_JUNK(A.jmp(*x_get_function(vm->functions[X_VM_FUNC_GET_EIP])))

	x_code_shuffling(true);

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_setf(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*flags;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	flags = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get flags
	FETCH_OP(flags, SETF, 1);
	DECODE_BYTE(flags, SETF, 1);

	// set flags
	A.or_(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_VM_FLAGS)), *(flags->reg));

	x_release_register(flags, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_clrf(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*flags;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	flags = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get flags
	FETCH_OP(flags, CLRF, 1);
	DECODE_BYTE(flags, CLRF, 1);

	// clear flags
	A.not_(*(flags->reg));
	A.and_(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_VM_FLAGS)), *(flags->reg));

	x_release_register(flags, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_fixrlc(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*dst;
	x_x86_reg_t		*tmp;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	dst = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(dst, FIXRLC, 1);
	DECODE_REG(dst, FIXRLC, 1);
	x_gen_get_register_address(vm, dst);

	tmp = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	x_gen_fetch_mem(tmp, 4, dst, NULL, 0, 0);
	x_gen_add_base_diff(vm, tmp);
	A.mov(dword_ptr(*(dst->reg)), *(tmp->reg));

	x_release_register(tmp, &A);
	x_release_register(dst, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_vmtest(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*r1;
	x_x86_reg_t		*r2;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	r1 = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst value
	FETCH_OP(r1, VMTEST, 1);
	DECODE_REG(r1, VMTEST, 1);
	x_gen_dereference_register(vm, r1);

	r2 = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, r1->reg);

	// get src value
	FETCH_OP(r2, VMTEST, 2);
	DECODE_REG(r2, VMTEST, 2);
	x_gen_dereference_register(vm, r2);

	Label	*is_zero = x_new_label();
	Label	*not_zero = x_new_label();
	Label	*end = x_new_label();

	// perform test
	x_code_shuffling(false);
	A.test(*(r1->reg), *(r2->reg));

	DISABLE_JUNK(A.jz(*is_zero))
	x_code_shuffling(true);
	A.and_(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_VM_FLAGS)), ~(X_VMF_ZERO));
	A.jmp(*end);
	x_bind_label(is_zero);
	A.or_(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_VM_FLAGS)), X_VMF_ZERO);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(r2, &A);
	x_release_register(r1, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_function_vm_ins_vmcmp(void *data)
{
	x_vm_t			*vm;
	x_x86_reg_t		*r1;
	x_x86_reg_t		*r2;
	
	vm = (x_vm_t *) data;

	// global registers
	x_need_register(vm->reg_ins);
	x_need_register(vm->reg_stack);

	r1 = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// get dst address
	FETCH_OP(r1, VMCMP, 1);
	DECODE_REG(r1, VMCMP, 1);
	x_gen_dereference_register(vm, r1);

	r2 = x_get_random_register(true, true, &A, vm->reg_stack->reg, vm->reg_ins->reg, r1->reg);

	// get src value
	FETCH_OP(r2, VMCMP, 2);
	DECODE_REG(r2, VMCMP, 2);
	x_gen_dereference_register(vm, r2);

	Label	*is_zero = x_new_label();
	Label	*not_zero = x_new_label();
	Label	*end = x_new_label();

	// perform comparison
	x_code_shuffling(false);
	A.cmp(*(r1->reg), *(r2->reg));
	DISABLE_JUNK(A.jz(*is_zero))
	x_code_shuffling(true);
	A.and_(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_VM_FLAGS)), ~(X_VMF_ZERO));
	A.jmp(*end);
	x_bind_label(is_zero);
	A.or_(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_VM_FLAGS)), X_VMF_ZERO);
	A.jmp(*end);

	x_bind_label(end);

	x_release_register(r2, &A);
	x_release_register(r1, &A);

	x_gen_obfuscated_ret();

	// global registers
	x_release_register(vm->reg_ins, &A);
	x_release_register(vm->reg_stack, &A);
}

void x_gen_get_vm_thread_data(x_vm_t *vm)
{
	// get vm context base
	A.call(*x_get_function(vm->functions[X_VM_FUNC_GET_THREAD_DATA]));
	A.pop(*(vm->reg_stack->reg));
}

#define BCKP(x)	do { \
					bkp = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg); \
					A.pop(*(bkp->reg)); \
					x_emit_junk_operation(); \
					x_gen_write_dword(vm->reg_stack, NULL, 0, VMROFF(x), bkp); \
					x_release_register(bkp, &A); \
				} while(0)

void x_gen_backup_x86_context(x_vm_t *vm)
{
	x_x86_reg_t		*bkp;

	BCKP(X_VMR_N_EFLAGS);
	BCKP(X_VMR_N_EDI);
	BCKP(X_VMR_N_ESI);
	BCKP(X_VMR_N_EBP);

	bkp = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);
	A.pop(*(bkp->reg));
	x_emul_add_val(bkp, 4);
	x_gen_write_dword(vm->reg_stack, NULL, 0, VMROFF(X_VMR_N_ESP), bkp);
	x_release_register(bkp, &A);

	BCKP(X_VMR_N_EBX);
	BCKP(X_VMR_N_EDX);
	BCKP(X_VMR_N_ECX);
	BCKP(X_VMR_N_EAX);

	BCKP(X_VMR_VM_EIP);
}

#undef BCKP

void x_gen_set_base_diff(x_vm_t *vm)
{
	x_x86_reg_t		*diff;
	UINT32			rnd;

	diff = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	rnd = x_rand();

	x_func_reloc_add(rnd);
	A.push(rnd);
	x_func_reloc_end();

	A.pop(*(diff->reg));
	A.sub(*(diff->reg), rnd);

	x_gen_write_dword(vm->reg_stack, NULL, 0, SOFF(X_VM_STACK_BASE_DIFF), diff);

	x_release_register(diff, &A);
}

void x_gen_reset_vm_flags(x_vm_t *vm)
{
	x_x86_reg_t		*flag_reset_reg;

	flag_reset_reg = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	A.xor_(*(flag_reset_reg->reg), *(flag_reset_reg->reg));
	x_gen_write_dword(vm->reg_stack, NULL, 0, VMROFF(X_VMR_VM_FLAGS), flag_reset_reg);

	x_release_register(flag_reset_reg, &A);
}

void x_gen_setup_stack(x_vm_t *vm)
{
	x_emul_add_val(vm->reg_stack, vm->stack.esp_offset);
	A.mov(esp, *(vm->reg_stack->reg));
	x_emul_add_val(vm->reg_stack, -vm->stack.esp_offset);
}

void x_gen_fetch_eip(x_vm_t *vm)
{
	x_gen_fetch_mem(vm->reg_ins, 4, vm->reg_stack, NULL, 0, VMROFF(X_VMR_VM_EIP));
}

void x_gen_copy_instruction_to_dec_buff(x_vm_t *vm)
{
	x_x86_reg_t		*dec_buf;
	x_x86_reg_t		*cnt;
	x_x86_reg_t		*tmp;

	dec_buf = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	// decryption buff
	x_gen_lea(dec_buf, vm->reg_stack, NULL, 0, SOFF(X_VM_STACK_DECRYPT_BUFF));

	cnt = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dec_buf->reg);

	A.xor_(*(cnt->reg), *(cnt->reg));

	Label *copy_again = x_new_label();
	x_bind_label(copy_again);

	tmp = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dec_buf->reg, cnt->reg);

	x_gen_fetch_mem(tmp, 4, vm->reg_ins, cnt, 0, 0);
	x_gen_write_dword(dec_buf, cnt, 0, 0, tmp);

	x_release_register(tmp, &A);

	x_emul_add_val(cnt, 4);
	
	x_code_shuffling(false);
	A.cmp(*(cnt->reg), vm->longest_instruction);
	DISABLE_JUNK(A.jb(*copy_again))
	x_code_shuffling(true);

	x_release_register(cnt, &A);

	A.mov(*(vm->reg_ins->reg), *(dec_buf)->reg);

	x_release_register(dec_buf, &A);
}

void x_gen_decrypt_instruction(x_vm_t *vm)
{
	unsigned		decrypted;
	x_x86_reg_t		*val;

	// initial values
	decrypted = 0;

	while (decrypted < vm->longest_instruction) {
		val = x_get_random_register(false, false, &A, vm->reg_ins->reg);
		
		x_gen_fetch_mem(val, 4, vm->reg_ins, NULL, 0, decrypted);
		x_gen_write_dword(vm->reg_ins, NULL, 0, decrypted, val);

		x_release_register(val, &A);

		decrypted += 4;
	}
}

void x_gen_get_mnemonic(x_vm_t *vm, x_x86_reg_t *dst)
{
	x_gen_fetch_mem(dst, 2, vm->reg_ins, NULL, 0, 0);
	x_vm_decode_mnemonic(vm, dst);
}

void x_gen_set_next_eip(x_vm_t *vm, x_x86_reg_t *mn, Label *invalid_mnemonic)
{
	x_x86_reg_t		*len, *tbl, *mn_len, *tmp;
	Label			*valid_mnemonic;

	valid_mnemonic = x_new_label();

	// skip invalid mnemonics
	tmp = x_get_random_register(false, false, &A, mn->reg);

	x_emul_mov_imm(tmp, X_VMI_TOTAL);

	x_code_shuffling(false);
	A.cmp(*(mn->reg), *(tmp->reg));
	DISABLE_JUNK(A.jb(*valid_mnemonic))
	x_code_shuffling(true);

	len = x_get_random_register(false, false, &A, vm->reg_stack->reg);
	x_emul_mov_imm(len, X_VM_MNEMONIC_LEN);
	A.add(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_VM_EIP)), *(len->reg));
	x_release_register(len, &A);

	x_release_register(tmp, &A);

	A.jmp(*invalid_mnemonic);

	x_bind_label(valid_mnemonic);

	len = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, mn->reg);

	tbl = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, mn->reg, len->reg);

	x_emul_mov_imm(tbl, vm->data_rva_vmi_lengths);
	x_emul_add(tbl, mn);
	x_gen_add_base_addr(vm, tbl);
	x_gen_fetch_mem(len, 1, tbl, NULL, 0, 0);

	x_release_register(tbl, &A);

	x_vm_decode_byte(vm, len, mn);

	mn_len = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, mn->reg, len->reg);
	x_emul_mov_imm(mn_len, X_VM_MNEMONIC_LEN);
	x_emul_add(len, mn_len);
	x_release_register(mn_len, &A);

	x_emit_junk_operation();

	A.add(dword_ptr(*(vm->reg_stack->reg), VMROFF(X_VMR_VM_EIP)), *(len->reg));

	x_release_register(len, &A);
}

void x_gen_get_handler(x_vm_t *vm, x_x86_reg_t *mn, x_x86_reg_t *dst)
{
	x_x86_reg_t		*tbl;

	tbl = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, mn->reg, dst->reg);

	A.mov(*(tbl->reg), *(mn->reg));
	A.shl(*(tbl->reg), 2);
	x_emul_add_val(tbl, vm->data_rva_vmi_rvas);
	x_gen_add_base_addr(vm, tbl);

	x_gen_fetch_mem(dst, 4, tbl, NULL, 0, 0);

	x_release_register(tbl, &A);

	x_vm_decode_dword(vm, dst, mn);

	x_gen_add_base_addr(vm, dst);
}

void x_gen_add_base_addr(x_vm_t *vm, x_x86_reg_t *val)
{
	x_x86_reg_t		*tmp;

	x_gen_add_base_diff(vm, val);

	tmp = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, val->reg);

	x_emul_mov_imm(tmp, xg.ctx->module_base);
	x_emul_add(val, tmp);

	x_release_register(tmp, &A);
}

void x_gen_add_base_diff(x_vm_t *vm, x_x86_reg_t *val)
{
	x_x86_reg_t		*tmp;

	tmp = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, val->reg);

	x_gen_fetch_mem(tmp, 4, vm->reg_stack, NULL, 0, SOFF(X_VM_STACK_BASE_DIFF));
	x_emul_add(val, tmp);

	x_release_register(tmp, &A);
}

void x_gen_get_register_address(x_vm_t *vm, x_x86_reg_t *dst)
{
	x_x86_reg_t		*tmp;

	tmp = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	x_gen_lea(tmp, vm->reg_stack, dst, 2, SOFF(X_VM_STACK_CONTEXT));
	A.mov(*(dst->reg), *(tmp->reg));

	x_release_register(tmp, &A);
}

void x_gen_dereference_register(x_vm_t *vm, x_x86_reg_t *dst)
{
	x_x86_reg_t		*tmp;

	tmp = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg, dst->reg);

	x_gen_fetch_mem(tmp, 4, vm->reg_stack, dst, 2, SOFF(X_VM_STACK_CONTEXT));
	A.mov(*(dst->reg), *(tmp->reg));

	x_release_register(tmp, &A);
}

void x_gen_test_vm_flags(x_vm_t *vm, UINT32 flags)
{
	x_x86_reg_t		*tmp;

	tmp = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	x_gen_fetch_mem(tmp, 4, vm->reg_stack, NULL, 0, VMROFF(X_VMR_VM_FLAGS));

	A.test(*(tmp->reg), flags);

	x_release_register(tmp, &A);
}

void x_gen_check_ins_condition_code(x_vm_t *vm, Label *true_label)
{
	x_x86_reg_t		*cc;

	cc = x_get_random_register(true, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	x_gen_fetch_mem(cc, 1, vm->reg_ins, NULL, 0, 0);
	x_vm_decode_op_byte(vm, cc, 0, 0);

	Label		*l_zero = x_new_label();
	Label		*l_not_zero = x_new_label();
	Label		*end = x_new_label();

	// check CC
	x_code_shuffling(false);
	A.cmp(*(cc->reg), X_CC_Z);
	DISABLE_JUNK(A.je(*l_zero))
	x_code_shuffling(true);
	x_code_shuffling(false);
	A.cmp(*(cc->reg), X_CC_NZ);
	DISABLE_JUNK(A.je(*l_not_zero))
	x_code_shuffling(true);

	// no CC
	A.jmp(*end);

	// check for condition == ZERO
	x_bind_label(l_zero);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_ZERO);
	DISABLE_JUNK(A.jz(*true_label))
	x_code_shuffling(true);
	A.jmp(*end);

	// check for condition != ZERO
	x_bind_label(l_not_zero);
	x_code_shuffling(false);
	x_gen_test_vm_flags(vm, X_VMF_ZERO);
	DISABLE_JUNK(A.jnz(*true_label))
	x_code_shuffling(true);
	A.jmp(*end);

	// end of processing flags
	x_bind_label(end);

	x_release_register(cc, &A);
}

void x_gen_obfuscated_ret(void)
{
	x_x86_reg_t			*ret;

	ret = x_get_random_register(true, false, &A);

	A.pop(*(ret->reg));

	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();

	A.jmp(*(ret->reg));

	x_release_register(ret, &A);
}

void x_gen_fetch_mem(x_x86_reg_t *dst, unsigned src_size, x_x86_reg_t *base, x_x86_reg_t *index, unsigned scale, UINT32 offset)
{
	bool		zeroed = false;

	if (index) {
		A.mov(*(dst->reg), *(index->reg));

		if (scale) {
			A.shl(*(dst->reg), scale);
		}

		zeroed = true;
	}

	if (!zeroed) {
		A.xor_(*(dst->reg), *(dst->reg));
	}

	if (base) {
		x_emul_add(dst, base);
	}

	if (offset) {
		x_x86_reg_t		*off;

		off = x_get_random_register(false, false, &A, dst->reg);

		x_emul_mov_imm(off, offset);
		x_emul_add(dst, off);

		x_release_register(off, &A);
	}

	A.push(dword_ptr(*(dst->reg)));
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	x_emit_junk_instruction();
	A.pop(*(dst->reg));

	if (src_size != 4) {
		if (src_size == 1) {
			A.and_(*(dst->reg), 0xFF);
		} else if (src_size == 2) {
			A.and_(*(dst->reg), 0xFFFF);
		} else {
			x_exit_thread(X_ERR_FETCH_MEM_BAD_SRC_SIZE);
		}
	}
}

void x_gen_lea(x_x86_reg_t *dst, x_x86_reg_t *base, x_x86_reg_t *index, unsigned scale, UINT32 offset)
{
	bool		zeroed = false;

	if (index) {
		A.mov(*(dst->reg), *(index->reg));

		x_emit_junk_instruction();
		x_emit_junk_instruction();

		if (scale) {
			A.shl(*(dst->reg), scale);
		}

		zeroed = true;
	}

	if (!zeroed) {
		A.xor_(*(dst->reg), *(dst->reg));
	}

	if (base) {
		x_emul_add(dst, base);
	}

	if (offset) {
		x_x86_reg_t		*off;

		off = x_get_random_register(false, false, &A, dst->reg);

		x_emul_mov_imm(off, offset);
		x_emul_add(dst, off);

		x_release_register(off, &A);
	}
}

void x_gen_write_dword(x_x86_reg_t *base, x_x86_reg_t *index, unsigned scale, UINT32 offset, x_x86_reg_t *val)
{
	x_x86_reg_t		*tmp;

	tmp = x_get_random_register(false, false, &A, val->reg, base ? base->reg : NULL, index ? index->reg : NULL);

	x_gen_lea(tmp, base, index, scale, offset);

	x_emit_junk_instruction();

	A.mov(dword_ptr(*(tmp->reg)), *(val->reg));

	x_release_register(tmp, &A);
}

void x_gen_fetch_op(x_vm_t *vm, x_x86_reg_t *dst, unsigned ins, unsigned op)
{
	UINT32		offset, size;

	offset = x_get_vm_op_offset(ins, op);
	size = x_get_vm_op_size(ins, op);

	x_gen_fetch_mem(dst, size, vm->reg_ins, NULL, 0, offset);
}

void x_gen_exec_code_in_buffer(x_vm_t *vm)
{
	UINT8			buf[X_VM_INS_EXEC_SIZE];
	unsigned		len;
	unsigned		copied_len;
	x_x86_reg_t		*tmp;

	// write ret instruction
	B.ret();

	len = B.getCodeSize();

	if (!len || (len > X_VM_INS_EXEC_SIZE)) {
		x_exit_thread(X_ERR_EXEC_CODE_IN_BUFFER_BAD_LENGTH);
	}

	// fill buf with random data to avoid data leaks from stack
	x_random_fill(buf, len);

	// generate code
	B.relocCode(buf, NULL);

	copied_len = 0;
	
	while (copied_len < len) {
		x_x86_reg_t		*tmp;

		tmp = x_get_random_register(false, false, &A, vm->reg_stack->reg);
		
		x_emul_mov_imm(tmp, *(MakePtr(UINT32 *, buf, copied_len)));

		x_gen_write_dword(vm->reg_stack, NULL, 0, SOFF(X_VM_STACK_INS_EXEC) + copied_len, tmp);

		x_release_register(tmp, &A);

		copied_len += 4;
	}

	tmp = x_get_random_register(false, false, &A, vm->reg_stack->reg, vm->reg_ins->reg);

	x_emul_mov_imm(tmp, SOFF(X_VM_STACK_INS_EXEC));
	x_emul_add(tmp, vm->reg_stack);
	A.call(*(tmp->reg));

	x_release_register(tmp, &A);

	// reinitialize assembler
	x_initialize_temp_assembler();
}

void x_vm_decode_mnemonic(x_vm_t *vm, x_x86_reg_t *val)
{
	
}

void x_vm_decode_register(x_vm_t *vm, x_x86_reg_t *val, unsigned mn, unsigned op_id)
{
	
}

void x_vm_decode_op_byte(x_vm_t *vm, x_x86_reg_t *val, unsigned mn, unsigned op_id)
{
	
}

void x_vm_decode_op_dword(x_vm_t *vm, x_x86_reg_t *val, unsigned mn, unsigned op_id)
{
	
}

void x_vm_decode_byte(x_vm_t *vm, x_x86_reg_t *val, x_x86_reg_t *ind)
{
	
}

void x_vm_decode_dword(x_vm_t *vm, x_x86_reg_t *val, x_x86_reg_t *ind)
{
	
}
