// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_vm_functions.h 653 2015-02-24 21:24:38Z tzubcic $

#pragma once

// common includes
#include "x_common.h"

// local headers
#include "x_vm.h"
#include "x_registers.h"

// enter vm function
void x_gen_function_vm_enter(void *);

// leave vm function
void x_gen_function_vm_leave(void *);

// leave vm for calling procedure
void x_gen_function_vm_leave_call_ret(void *);

// leave vm for executing original native code
void x_gen_function_vm_leave_orig_ret(void *);

// store eip to stack
void x_gen_function_vm_store_eip(void *);

// get eip from stack
void x_gen_function_vm_get_eip(void *);

// init thread specific data function
void x_gen_function_vm_init_thread_data(void *);

// get thread local data
void x_gen_function_vm_get_thread_data(void *);

// execute bytecode
void x_gen_function_vm_execute_bytecode(void *);

// instruction handlers
void x_gen_function_vm_ins_mov(void *);
void x_gen_function_vm_ins_lea(void *);
void x_gen_function_vm_ins_asgn(void *);
void x_gen_function_vm_ins_rd(void *);
void x_gen_function_vm_ins_wr(void *);
void x_gen_function_vm_ins_add(void *);
void x_gen_function_vm_ins_sub(void *);
void x_gen_function_vm_ins_shr(void *);
void x_gen_function_vm_ins_shl(void *);
void x_gen_function_vm_ins_and(void *);
void x_gen_function_vm_ins_xor(void *);
void x_gen_function_vm_ins_or(void *);
void x_gen_function_vm_ins_rol(void *);
void x_gen_function_vm_ins_ror(void *);
void x_gen_function_vm_ins_not(void *);
void x_gen_function_vm_ins_jmp(void *);
void x_gen_function_vm_ins_leave(void *);
void x_gen_function_vm_ins_call(void *);
void x_gen_function_vm_ins_exec_buf(void *);
void x_gen_function_vm_ins_exec(void *);
void x_gen_function_vm_ins_setf(void *);
void x_gen_function_vm_ins_clrf(void *);
void x_gen_function_vm_ins_fixrlc(void *);
void x_gen_function_vm_ins_vmtest(void *);
void x_gen_function_vm_ins_vmcmp(void *);

// helper functions
void x_gen_get_vm_thread_data(x_vm_t *);
void x_gen_backup_x86_context(x_vm_t *);
void x_gen_set_base_diff(x_vm_t *);
void x_gen_reset_vm_flags(x_vm_t *);
void x_gen_setup_stack(x_vm_t *);
void x_gen_fetch_eip(x_vm_t *);
void x_gen_copy_instruction_to_dec_buff(x_vm_t *);
void x_gen_decrypt_instruction(x_vm_t *);
void x_gen_get_mnemonic(x_vm_t *, x_x86_reg_t *);
void x_gen_set_next_eip(x_vm_t *, x_x86_reg_t *, Label *);
void x_gen_get_handler(x_vm_t *, x_x86_reg_t *, x_x86_reg_t *);
void x_gen_add_base_addr(x_vm_t *, x_x86_reg_t *);
void x_gen_add_base_diff(x_vm_t *, x_x86_reg_t *);
void x_gen_get_register_address(x_vm_t *, x_x86_reg_t *);
void x_gen_dereference_register(x_vm_t *, x_x86_reg_t *);
void x_gen_test_vm_flags(x_vm_t *, UINT32);
void x_gen_check_ins_condition_code(x_vm_t *, Label *);
void x_gen_obfuscated_ret(void);
void x_gen_fetch_mem(x_x86_reg_t *, unsigned, x_x86_reg_t *, x_x86_reg_t *, unsigned, UINT32);
void x_gen_lea(x_x86_reg_t *, x_x86_reg_t *, x_x86_reg_t *, unsigned, UINT32);
void x_gen_write_dword(x_x86_reg_t *, x_x86_reg_t *, unsigned, UINT32, x_x86_reg_t *);
void x_gen_fetch_op(x_vm_t *, x_x86_reg_t *, unsigned, unsigned);

// exec code in buffer
void x_gen_exec_code_in_buffer(x_vm_t *);

// decode data
void x_vm_decode_mnemonic(x_vm_t *, x_x86_reg_t *);
void x_vm_decode_register(x_vm_t *, x_x86_reg_t *, unsigned, unsigned);
void x_vm_decode_op_byte(x_vm_t *, x_x86_reg_t *, unsigned, unsigned);
void x_vm_decode_op_dword(x_vm_t *, x_x86_reg_t *, unsigned, unsigned);
void x_vm_decode_byte(x_vm_t *, x_x86_reg_t *, x_x86_reg_t *);
void x_vm_decode_dword(x_vm_t *, x_x86_reg_t *, x_x86_reg_t *);
