// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_vm_shared.cpp 659 2015-02-25 12:05:31Z tzubcic $

// common includes
#include "x_common.h"

// local headers
#include "x_vm_shared.h"

x_vm_ins_info_t vm_ins_info[] = {
	//  instruction id			condition			1st operand			2nd operand			3rd operand			func id
	{	X_VMI_MOV,				X_VMI_OP_BYTE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VMI_OP_NONE,		X_VM_FUNC_I_MOV			},
	{	X_VMI_LEA,				X_VMI_OP_BYTE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VMI_OP_NONE,		X_VM_FUNC_I_LEA			},
	{	X_VMI_ASGN,				X_VMI_OP_BYTE,		X_VMI_OP_REG,		X_VMI_OP_DWORD,		X_VMI_OP_NONE,		X_VM_FUNC_I_ASGN		},
	{	X_VMI_RD,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_RD			},
	{	X_VMI_WR,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_WR			},
	{	X_VMI_ADD,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_ADD			},
	{	X_VMI_SUB,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_SUB			},
	{	X_VMI_SHR,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_SHR			},
	{	X_VMI_SHL,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_SHL			},
	{	X_VMI_AND,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_AND			},
	{	X_VMI_XOR,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_XOR			},
	{	X_VMI_OR,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_OR			},
	{	X_VMI_ROL,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_ROL			},
	{	X_VMI_ROR,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VM_FUNC_I_ROR			},
	{	X_VMI_NOT,				X_VMI_OP_BYTE,		X_VMI_OP_SIZE,		X_VMI_OP_REG,		X_VMI_OP_NONE,		X_VM_FUNC_I_NOT			},
	{	X_VMI_JMP,				X_VMI_OP_BYTE,		X_VMI_OP_REL_DWORD,	X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VM_FUNC_I_JMP			},
	{	X_VMI_LEAVE,			X_VMI_OP_BYTE,		X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VM_FUNC_I_LEAVE		},
	{	X_VMI_CALL,				X_VMI_OP_BYTE,		X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VM_FUNC_I_CALL		},
	{	X_VMI_EXEC_BUF,			X_VMI_OP_BYTE,		X_VMI_OP_REG,		X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VM_FUNC_I_EXEC_BUF	},
	{	X_VMI_EXEC,				X_VMI_OP_BYTE,		X_VMI_OP_REG,		X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VM_FUNC_I_EXEC		},
	{	X_VMI_SETF,				X_VMI_OP_BYTE,		X_VMI_OP_BYTE,		X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VM_FUNC_I_SETF		},
	{	X_VMI_CLRF,				X_VMI_OP_BYTE,		X_VMI_OP_BYTE,		X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VM_FUNC_I_CLRF		},
	{	X_VMI_FIXRLC,			X_VMI_OP_BYTE,		X_VMI_OP_REG,		X_VMI_OP_NONE,		X_VMI_OP_NONE,		X_VM_FUNC_I_FIXRLC		},
	{	X_VMI_VMTEST,			X_VMI_OP_BYTE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VMI_OP_NONE,		X_VM_FUNC_I_VMTEST		},
	{	X_VMI_VMCMP,			X_VMI_OP_BYTE,		X_VMI_OP_REG,		X_VMI_OP_REG,		X_VMI_OP_NONE,		X_VM_FUNC_I_VMCMP		},
};

const size_t num_vm_ins_info = sizeof(vm_ins_info) / sizeof(vm_ins_info[0]);

unsigned x_get_vm_operand_len(UINT32 op_type)
{
	unsigned	ret;

	if (op_type == X_VMI_OP_NONE) {
		ret = 0;
	} else if (op_type == X_VMI_OP_REG) {
		ret = 1;
	} else if (op_type == X_VMI_OP_BYTE) {
		ret = 1;
	} else if (op_type == X_VMI_OP_DWORD) {
		ret = 4;
	} else if (op_type == X_VMI_OP_REL_DWORD) {
		ret = 4;
	} else if (op_type == X_VMI_OP_SIZE) {
		ret = 1;
	} else {
		x_exit_thread(X_ERR_GET_VM_OP_LEN_BAD_OP);
	}

	return ret;
}

x_vm_ins_info_t *x_get_vm_ins_info(UINT32 id)
{
	for (unsigned i = 0; i < num_vm_ins_info; i++) {
		if (vm_ins_info[i].id == id) {
			return &vm_ins_info[i];
		}
	}

	x_exit_thread(X_ERR_GET_VN_INS_INFO_BAD_INS);

	return NULL;
}

UINT32 x_get_vm_ins_size(UINT32 id)
{
	x_vm_ins_info_t		*ii;
	UINT32				size;

	ii = x_get_vm_ins_info(id);

	size = 0;

	for (unsigned i = 0; i < 4; i++) {
		size += x_get_vm_operand_len(ii->op_types[i]);
	}

	return size;
}

UINT32 x_get_vm_op_offset(UINT32 id, UINT32 op)
{
	x_vm_ins_info_t		*ii;
	UINT32				off;

	ii = x_get_vm_ins_info(id);

	if (op > 3) {
		x_exit_thread(X_ERR_GET_OP_OFFSET_BAD_OP_ID);
	}

	off = 0;

	for (unsigned i = 0; i < op; i++) {
		if (ii->op_types[i] == X_VMI_OP_NONE) {
			x_exit_thread(X_ERR_GET_OP_OFFSET_BAD_OP_ID);
		}

		off += x_get_vm_operand_len(ii->op_types[i]);
	}

	return off;
}

UINT32 x_get_vm_op_size(UINT32 id, UINT32 op)
{
	x_vm_ins_info_t		*ii;
	UINT32				size;

	ii = x_get_vm_ins_info(id);

	if (op > 3) {
		x_exit_thread(X_ERR_GET_OP_SIZE_BAD_OP_ID);
	}

	if (ii->op_types[op] == X_VMI_OP_NONE) {
		x_exit_thread(X_ERR_GET_OP_SIZE_BAD_OP_ID);
	}

	size = x_get_vm_operand_len(ii->op_types[op]);

	return size;
}

UINT8 x_encode_byte(UINT32 seed, UINT32 val, unsigned ind)
{
	UINT8		ret;

	val &= 0xFF;

	ret = (UINT8) val;

	return ret;
}

UINT32 x_encode_dword(UINT32 seed, UINT32 val, unsigned ind)
{
	UINT32		ret;

	ret = val;

	return ret;
}
