// x86obf - a code virtualizer/obfuscator for 32bit PE files
// Copyright (c) 2015 Tomislav Zubcic - chaplja.blogspot.com / x86obf@gmail.com
// $Id: x_vm_stack.cpp 657 2015-02-25 11:42:33Z tzubcic $

#include "x_local.h"

void x_init_vm_stack(x_vm_t *vm)
{
	UINT32		curr_offset, curr_size;

	// initialize order
	for (unsigned i = 0; i < X_VM_STACK_TOTAL; i++) {
		vm->stack.order[i] = i;
	}

	// shuffle order
	random_shuffle(&vm->stack.order[0], &vm->stack.order[X_VM_STACK_TOTAL], x_rand_generator);

	// offset from top - leave 4 bytes because sometimes read is forced to dword
	curr_offset = 4;

	for (unsigned i = 0; i < X_VM_STACK_TOTAL; i++) {
		if (vm->stack.order[i] == X_VM_STACK_CONTEXT) {
			curr_size = X_VM_CONTEXT_SIZE;
		} else if (vm->stack.order[i] == X_VM_STACK_EIP_BACKUP) {
			curr_size = X_VM_EIP_BUFFER_SIZE;
		} else if (vm->stack.order[i] == X_VM_STACK_DECRYPT_BUFF) {
			curr_size = X_VM_BC_DEC_BUFF_SIZE;
		} else if (vm->stack.order[i] == X_VM_STACK_BASE_DIFF) {
			curr_size = X_VM_BASE_DIFF_SIZE;
		} else if (vm->stack.order[i] == X_VM_STACK_RAND) {
			curr_size = X_VM_RAND_SIZE;
		} else if (vm->stack.order[i] == X_VM_STACK_INS_EXEC) {
			curr_size = X_VM_INS_EXEC_SIZE;
		} else if (vm->stack.order[i] == X_VM_STACK_ORIG) {
			curr_size = X_VM_ORIG_EXEC_SIZE;
		} else if (vm->stack.order[i] == X_VM_STACK_ENC_KEY) {
			curr_size = X_VM_ENC_KEY_SIZE;
		} else {
			x_exit_thread(X_ERR_INIT_VM_STACK_BAD_ID);
		}

		curr_offset += curr_size;

		vm->stack.offset[i] = curr_offset;
	}

	vm->stack.esp_offset = X_VM_THREAD_DATA_SIZE - curr_offset;
}

UINT32 x_vm_stack_offset(x_vm_t *vm, unsigned id)
{
	UINT32		ret;

	if (id >= X_VM_STACK_TOTAL) {
		x_exit_thread(X_ERR_VM_STACK_BAD_ID);
	}

	ret = X_VM_THREAD_DATA_SIZE;

	for (unsigned i = 0; i < X_VM_STACK_TOTAL; i++) {
		if (vm->stack.order[i] == id) {
			ret -= vm->stack.offset[i];
			break;
		}
	}

	return ret;
}
